// Assuming React and ReactDOM are available globally
const { useState, useEffect } = React;
const widgetsData = drupalSettings.dashboard.widgets;


// App Component

function App() {
  const [widgets, setWidgets] = useState({});

  useEffect(() => {
    // This code runs after the component mounts
    const activeWidgetsField = document.getElementById('active-widgets-field');
    if (activeWidgetsField && activeWidgetsField.value) {
      try {
        const initialWidgets = JSON.parse(activeWidgetsField.value);
        setWidgets(initialWidgets);
      } catch (error) {
        console.error('Error parsing widgets data:', error);
      }
    }
  }, []);

  const syncActiveWidgetsToForm = (widgets) => {
    const activeWidgetsField = document.getElementById('active-widgets-field');
    if (activeWidgetsField) {
      // Convert the widgets array to a JSON string
      activeWidgetsField.value = JSON.stringify(widgets);
    }
  };

  const changeColspan = (uuid, newColspan) => {
    const updatedWidgets = Object.entries(widgets).reduce((acc, [key, widget]) => {
      if (key === uuid) {
        acc[key] = { ...widget, colspan: newColspan };
      } else {
        acc[key] = widget;
      }
      return acc;
    }, {});

    setWidgets(updatedWidgets);
    syncActiveWidgetsToForm(updatedWidgets);
  };


  const addWidget = (widgetData) => {
    const uuid = (()=>([1e7]+-1e3+-4e3+-8e3+-1e11).replace(/[018]/g,a=>(a^Math.random()*16>>a/4).toString(16)))();
    event.preventDefault(); // Prevent the form from submitting
    const newWidgetData = {
      ...widgetData,
      uuid: widgetData.uuid || uuid, // Use the provided UUID or generate a new one
      title: widgetData.title || '',
      widget_name: widgetData.title,
      weight: widgetData.weight || '2',
      configured: widgetData.configured || false,
      plugin_specific_config: widgetData.plugin_specific_config || null,
      colspan: widgetData.colspan || 'colspan-3'
    };
    setWidgets({
      ...widgets,
      [uuid]: newWidgetData // Add new widget with uuid as the key
    });
    syncActiveWidgetsToForm({ ...widgets, [uuid]: newWidgetData });
  };

  const removeWidget = (uuid) => {
    event.preventDefault();
    const { [uuid]: _, ...newWidgets } = widgets; // Remove the widget with the given uuid
    setWidgets(newWidgets);
    syncActiveWidgetsToForm(newWidgets);
  };

  const sortWidgets = (draggedWidgetUuid, targetWidgetUuid) => {
    // Convert widgets object to an array for sorting
    const widgetsArray = Object.entries(widgets).map(([key, value]) => ({
      uuid: key,
      ...value
    }));

    // Find indexes using the uuid
    const draggedWidgetIndex = widgetsArray.findIndex(widget => widget.uuid === draggedWidgetUuid);
    const targetWidgetIndex = widgetsArray.findIndex(widget => widget.uuid === targetWidgetUuid);

    // Ensure both widgets were found
    if (draggedWidgetIndex !== -1 && targetWidgetIndex !== -1) {
      // Perform the swap
      const [draggedWidget] = widgetsArray.splice(draggedWidgetIndex, 1);
      widgetsArray.splice(targetWidgetIndex, 0, draggedWidget);

      // Convert array back to object
      const newWidgets = widgetsArray.reduce((acc, widget) => {
        acc[widget.uuid] = widget;
        return acc;
      }, {});

      setWidgets(newWidgets);
      syncActiveWidgetsToForm(newWidgets);
    }
  };



  return React.createElement('div', { className: 'widget-configurator' },
    React.createElement('div', null,
      //React.createElement('div', null, 'Add Widgets'),
      React.createElement(AvailableWidgets, { widgetsData, onAddWidget: addWidget })
    ),
    React.createElement('div', null,
      //React.createElement('div', null, 'Dashboard Preview (Drag and Drop to Sort)'),
      React.createElement(ActiveWidgets, {
        widgets,
        onRemove: removeWidget,
        onSort: sortWidgets,
        onColspanChange: changeColspan // Pass the function to handle colspan changes
      })
    ),


  );
}


// Render the App
ReactDOM.render(React.createElement(App), document.getElementById('root'));

// AvailableWidgets Component
function AvailableWidgets({ widgetsData, onAddWidget }) {
  return React.createElement('div', { className: 'available-widgets' },
    Object.entries(widgetsData).map(([category, widgets]) => {
      // Ensure widgets is an array
      const widgetsArray = Array.isArray(widgets) ? widgets : Object.values(widgets);

      return React.createElement('div', { key: category, className: 'category' },
        React.createElement('h3', null, category),
        widgetsArray.map(widget =>
          React.createElement('div', {
            className: 'widget-item', // New class for individual widget items
            key: widget.uuid
          },
            React.createElement('span', { className: 'widget-label' }, widget.title), // Label for the widget
            React.createElement('button', {
              className: 'button button--small add-widget-button',
              onClick: (event) => onAddWidget(widget, event) // Pass the event to the handler
            }, 'Add') // Button with label 'Add'
          )
        )
      );
    })
  );
}


// ActiveWidgets Component
function ActiveWidgets({ widgets, onRemove, onSort, onColspanChange }) {
  const handleDragStart = (event, widget) => {
    event.dataTransfer.setData('uuid', widget.uuid);
  };

  const handleDragOver = (event) => {
    event.preventDefault(); // Necessary for onDrop to trigger
  };

  const handleDrop = (event, widget) => {
    event.preventDefault();
    const draggedWidgetUuid = event.dataTransfer.getData('uuid');
    onSort(draggedWidgetUuid, widget.uuid);
  };

  // Check if there are any widgets
  if (widgets.length === 0) {
    // If not, return a message
    return React.createElement('div', { className: 'active-widgets empty' }, "No widgets added.");
  }

  // If there are widgets, render them as before
  return React.createElement('div', { className: 'active-widgets', id: 'content-planner-dashboard' },
    Object.values(widgets).map(widget =>
      React.createElement(Widget, {
        key: widget.uuid,
        data: widget,
        onRemove: onRemove,
        onColspanChange: onColspanChange,
        onDragStart: (event) => handleDragStart(event, widget),
        onDragOver: handleDragOver,
        onDrop: (event) => handleDrop(event, widget),
        draggable: true  // Make the Widget itself draggable
      })
    )
  );
}



// Widget Component
function Widget({ data, onRemove, onColspanChange, onDragStart, onDragOver, onDrop, draggable }) {
  const handleColspanChange = (event) => {
    onColspanChange(data.uuid, event.target.value);
  };

  return React.createElement('div', {
    className: `widget active-widget dashboard-block ${data.colspan}`,
    onDragStart: onDragStart,
    onDragOver: onDragOver,
    onDrop: onDrop,
    draggable: draggable
  },
    React.createElement('div', { className: 'panel__title' },
      React.createElement('span', null, data.title),
      React.createElement('button', {
        onClick: () => onRemove(data.uuid),
        className: 'button button--small remove-button',
        style: { float: 'right' } // Inline style to align the button to the right
      }, 'Remove')
    ),
    React.createElement('div', { className: 'dashboard-block-content' },
      React.createElement('select', {
        value: data.colspan,
        onChange: handleColspanChange,
        className: 'colspan-select'
      },
        ['colspan-1', 'colspan-2', 'colspan-3', 'colspan-4', 'colspan-5', 'colspan-6'].map(colspanValue =>
          React.createElement('option', { value: colspanValue, key: colspanValue }, colspanValue)
        )
      ),
      React.createElement('div', {}, 'Widget: ' + data.widget_name)
    )
  );


}
