<?php

namespace Drupal\content_calendar\Component;

use Drupal\content_calendar\ContentTypeConfigService;
use Drupal\content_calendar\ContentCalendarService;
use Drupal\content_calendar\DateTimeHelper;
use Drupal\content_calendar\FilterHelper;
use Drupal\content_calendar\Plugin\CalendarFilterPluginManager;
use Drupal\content_planner\ContentModerationService;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Theme\ThemeManagerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\content_calendar\Form\SettingsForm;

/**
 * Implements Calendar class.
 */
class Calendar {

  use StringTranslationTrait;

  /**
   * The theme manager.
   *
   * @var \Drupal\Core\Theme\ThemeManagerInterface
   */
  protected $themeManager;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The content moderatiom service.
   *
   * @var \Drupal\content_planner\ContentModerationService
   */
  protected $contentModerationService;

  /**
   * The content type config service.
   *
   * @var \Drupal\content_calendar\ContentTypeConfigService
   */
  protected $contentTypeConfigService;

  /**
   * The content calendar service.
   *
   * @var \Drupal\content_calendar\ContentCalendarService
   */
  protected $contentCalendarService;

  /**
   * Defines the Content Type Config entity.
   *
   * @var \Drupal\content_calendar\Entity\ContentTypeConfig[]
   */
  protected $contentTypeConfigEntities;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $currentRequest;

  /**
   * The filter plugin manager.
   *
   * @var \Drupal\content_calendar\Plugin\CalendarFilterPluginManager
   */
  protected $filterPluginManager;

  /**
   * Desired day to be rendered.
   *
   * @var int
   */
  protected $day;

  /**
   * Desired months to be rendered.
   *
   * @var int
   */
  protected $month;

  /**
   * Desired year to be rendered.
   *
   * @var int
   */
  protected $year;

  /**
   * Calendar constructor.
   *
   * @param \Drupal\Core\Theme\ThemeManagerInterface $themeManager
   *   The theme manager service.
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \Drupal\content_planner\ContentModerationService $contentModerationService
   *   The content moderation service.
   * @param \Drupal\content_calendar\ContentTypeConfigService $contentTypeConfigService
   *   The content type config service.
   * @param \Drupal\content_calendar\ContentCalendarService $contentCalendarService
   *   The content calendar service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\content_calendar\Plugin\CalendarFilterPluginManager $filter_plugin_manager
   *   The filter plugin manager.
   */
  public function __construct(
    ThemeManagerInterface $themeManager,
    AccountProxyInterface $currentUser,
    ConfigFactoryInterface $configFactory,
    ContentModerationService $contentModerationService,
    ContentTypeConfigService $contentTypeConfigService,
    ContentCalendarService $contentCalendarService,
    RequestStack $request_stack,
    CalendarFilterPluginManager $filter_plugin_manager
  ) {
    $this->themeManager = $themeManager;
    $this->currentUser = $currentUser;
    $this->configFactory = $configFactory;
    $this->contentModerationService = $contentModerationService;
    $this->contentTypeConfigService = $contentTypeConfigService;
    $this->contentCalendarService = $contentCalendarService;
    $this->contentTypeConfigEntities = $this->contentTypeConfigService->loadAllEntities();
    $this->currentRequest = $request_stack->getCurrentRequest();
    $this->filterPluginManager = $filter_plugin_manager;
  }

  /**
   * @return static
   */
  public function setDay(int $day): self {
    $this->day = $day;
    return $this;
  }

  /**
   * @return static
   */
  public function setMonth(int $month): self {
    $this->month = $month;
    return $this;
  }

  /**
   * @return static
   */
  public function setYear(int $year): self {
    $this->year = $year;
    return $this;
  }

  /**
   * Generates a calendar id.
   *
   * @return string
   *   The calendar id.
   */
  protected function generateCalendarId() {
    if ($this->day) {
      return $this->year . '-' . $this->month . '-' . $this->day;
    }

    return $this->year . '-' . $this->month;
  }

  /**
   * Creates the render array for the list view.
   *
   * @return array
   *   The render array of the calendar.
   */
  public function buildListViewForMonth() {

    // Build data structure first.
    $list = $this->buildListViewStructure();

    // Get nodes per node type.
    $node_basic_data = $this->getNodesData();

    // Place nodes in Calendars.
    $this->placeNodesInListView($list, $node_basic_data);

    $list['has_nodes'] = FALSE;
    foreach ($list['days'] as $day => $data) {
      if (!empty($data['nodes'])) {
        $list['has_nodes'] = TRUE;
      }
    }

    $build = [
      '#theme' => 'content_calendar_list',
      '#list' => $list,
      '#attached' => [
        'library' => ['content_planner/dashboard'],
      ],
    ];

    return $build;
  }

  /**
   * Creates the render array for the month calendar.
   *
   * @return array
   *   The render array of the calendar.
   */
  public function buildMonthCalendar($show_controls = TRUE) {
    // Build data structure first.
    $calendar = $this->buildMonthCalendarDataStructure();
    $number_of_item_threshold = $this->configFactory
      ->get(SettingsForm::CONFIG_NAME)
      ->get('number_of_item_threshold');

    $settings = $this->configFactory->get(SettingsForm::CONFIG_NAME);

    // Get nodes per node type.
    $node_basic_data = $this->getNodesData();

    // Place nodes in Calendars.
    $this->placeNodesInMonthCalendar($calendar, $node_basic_data, $settings->get('compact_view_threshold'));

    $weekdays = DateTimeHelper::getWeekdays();

    $build = [
      '#theme' => 'content_calendar_month',
      '#calendar' => $calendar,
      '#weekdays' => $weekdays,
      '#show_controls' => $show_controls,
      '#number_of_item_threshold' => $number_of_item_threshold,
      '#current_month_route_slug' => date('Y/m') . "?" . FilterHelper::buildFilterQuerySlug(),
      '#prev_month_route_slug' => date(
        'Y/m',
        strtotime(
          "previous month",
          strtotime($this->year . "-" . $this->month)
        ),
      ) . "?" . FilterHelper::buildFilterQuerySlug(),
      '#next_month_route_slug' => date(
        'Y/m',
        strtotime(
          "next month",
          strtotime($this->year . "-" . $this->month)
        ),
      ) . "?" . FilterHelper::buildFilterQuerySlug(),
      '#node_type_creation_permissions' => $this->getPermittedNodeTypeCreationActions(),
      '#add_content_set_schedule_date' => $this->configFactory
        ->get('content_calendar.settings')
        ->get('add_content_set_schedule_date'),
      '#attached' => [
        'library' => ['content_planner/dashboard'],
      ],
    ];

    return $build;
  }

  /**
   * Creates the render array for the week calendar view.
   *
   * @return array
   *   The render array of the calendar.
   */
  public function buildWeekCalendar() {

    // Build data structure first.
    $calendar = $this->buildWeekCalendarDataStructure();

    $full_date = $this->year . "-" . $this->month . "-" . $this->day;

    $dates = array_keys($calendar['dates']);
    $from_day = reset($dates);
    $to_day = end($dates);

    // Get nodes per node type.
    $node_basic_data = $this->getNodesData(
      [
        'from_date' => strtotime($from_day),
        'to_date' => strtotime($to_day),
      ],
    );

    // Place nodes in Calendars.
    $this->placeNodesInWeekCalendar($calendar, $node_basic_data);

    $build = [
      '#theme' => 'content_calendar_week',
      '#calendar' => $calendar,
      '#current_week_route_slug' => date('Y/m/d') . "?" . FilterHelper::buildFilterQuerySlug(),
      '#prev_week_route_slug' => date(
        'Y/m/d',
        strtotime(
          "last saturday",
          strtotime($full_date)
        ),
      ) . "?" . FilterHelper::buildFilterQuerySlug(),
      '#next_week_route_slug' => date(
        'Y/m/d',
        strtotime(
          "next saturday",
          strtotime($full_date)
        ),
      ) . "?" . FilterHelper::buildFilterQuerySlug(),
      '#node_type_creation_permissions' => $this->getPermittedNodeTypeCreationActions(),
      '#add_content_set_schedule_date' => $this->configFactory
        ->get('content_calendar.settings')
        ->get('add_content_set_schedule_date'),
      '#attached' => [
        'library' => ['content_planner/dashboard'],
      ],
    ];

    return $build;
  }

  /**
   * Creates the render array for the day calendar view.
   *
   * @return array
   *   The render array of the calendar.
   */
  public function buildDayCalendar() {

    // Build data structure first.
    $calendar = $this->buildDayCalendarDataStructure();

    // Get nodes per node type.
    $node_basic_data = $this->getNodesData();

    // Place nodes in Calendars.
    $this->placeNodesInDayCalendar($calendar, $node_basic_data);

    $build = [
      '#theme' => 'content_calendar_day',
      '#calendar' => $calendar,
      '#current_day_route_slug' => date('Y/m/d') . "?" . FilterHelper::buildFilterQuerySlug(),
      '#prev_day_route_slug' => date(
        'Y/m/d',
        strtotime(
          "-1 day",
          strtotime($this->year . "-" . $this->month . "-" . $this->day)
        ),
      ) . "?" . FilterHelper::buildFilterQuerySlug(),
      '#next_day_route_slug' => date(
        'Y/m/d',
        strtotime(
          "+1 day",
          strtotime($this->year . "-" . $this->month . "-" . $this->day)
        ),
      ) . "?" . FilterHelper::buildFilterQuerySlug(),
      '#day' => $this->year . "-" . $this->month . "-" . $this->day,
      '#node_type_creation_permissions' => $this->getPermittedNodeTypeCreationActions(),
      '#add_content_set_schedule_date' => $this->configFactory
        ->get('content_calendar.settings')
        ->get('add_content_set_schedule_date'),
      '#attached' => [
        'library' => ['content_planner/dashboard'],
      ],
    ];

    return $build;
  }

  /**
   * Get all filtered nodes data.
   *
   * @param array $daterange
   *   Daterange to be added to the query.
   *
   * @return array
   *   Returns an array with node data.
   */
  protected function getNodesData(array $daterange = []) {
    $filters = FilterHelper::getActiveFilterValues();

    $node_basic_data = [];

    $node_basic_data[] = $this->contentCalendarService->getNodesByQuery(
      $this->year,
      $this->month,
      $this->day,
      $filters,
      $daterange
    );

    return $node_basic_data;
  }

  /**
   * Get all permitted Node Type Creation actions.
   *
   * @return array
   *   Returns an array with permitted node types.
   */
  protected function getPermittedNodeTypeCreationActions() {

    $permitted_node_types = [];

    foreach ($this->contentTypeConfigEntities as $node_type => $config_entity) {

      if ($this->currentUser->hasPermission("create $node_type content")) {
        $permitted_node_types[$node_type] = $config_entity;
      }

    }

    return $permitted_node_types;
  }

  /**
   * Build data structure for list.
   *
   * @return array
   *   The data for the calendar.
   *
   * @throws \Exception
   */
  protected function buildListViewStructure() {
    $today_datetime = new \DateTime();
    $today_datetime->setTime(0, 0, 0);
    $full_date = strtotime($this->year . "-" . $this->month);

    $one_day_interval = new \DateInterval('P1D');

    // Get the first date of a given month.
    $datetime = DateTimeHelper::getFirstDayOfMonth($this->month, $this->year);

    $scaffold_data = [
      'month' => date("m", $full_date),
      'year' => $this->year,
      'list_id' => $this->generateCalendarId(),
      'label' => DateTimeHelper::getMonthLabelByNumber($this->month) . ' ' . $this->year,
      'days' => [],
    ];

    // Calculate the days in a month.
    $days_in_month = DateTimeHelper::getDayCountInMonth($this->month, $this->year);

    // Build all dates in a month.
    $i = 1;
    while ($i <= $days_in_month) {

      $scaffold_data['days'][] = [
        'date' => $datetime->format('Y-m-d'),
        'day' => $datetime->format('j'),
        'day_with_leading_zero' => $datetime->format('d'),
        'weekday' => DateTimeHelper::getDayOfWeekByDate($datetime),
        'nodes' => [],
        'is_today' => ($today_datetime == $datetime) ? TRUE : FALSE,
      ];

      $i++;
      $datetime->add($one_day_interval);
    }

    return $scaffold_data;
  }

  /**
   * Build data structure for month calendar view.
   *
   * @return array
   *   The data for the calendar.
   *
   * @throws \Exception
   */
  protected function buildMonthCalendarDataStructure() {
    $today_datetime = new \DateTime();
    $today_datetime->setTime(0, 0, 0);
    $full_date = strtotime($this->year . "-" . $this->month);

    $one_day_interval = new \DateInterval('P1D');

    // Get the first date of a given month.
    $datetime = DateTimeHelper::getFirstDayOfMonth($this->month, $this->year);

    $scaffold_data = [
      'calendar_id' => $this->generateCalendarId(),
      'month' => date("m", $full_date),
      'year' => $this->year,
      'label' => DateTimeHelper::getMonthLabelByNumber($this->month) . ' ' . $this->year,
      'first_date_weekday' => DateTimeHelper::getDayOfWeekByDate($datetime),
      'days' => [],
    ];

    // Calculate the days in a month.
    $days_in_month = DateTimeHelper::getDayCountInMonth($this->month, $this->year);

    // Build all dates in a month.
    $i = 1;
    while ($i <= $days_in_month) {

      $scaffold_data['days'][] = [
        'date' => $datetime->format('Y-m-d'),
        'day' => $datetime->format('j'),
        'day_with_leading_zero' => $datetime->format('d'),
        'weekday' => DateTimeHelper::getDayOfWeekByDate($datetime),
        'nodes' => [],
        'is_today' => ($today_datetime == $datetime) ? TRUE : FALSE,
      ];

      $i++;
      $datetime->add($one_day_interval);
    }

    return $scaffold_data;
  }

  /**
   * Build data structure for week calendar view.
   *
   * @return array
   *   The data for the calendar.
   *
   * @throws \Exception
   */
  protected function buildWeekCalendarDataStructure() {

    $datetime = new \DateTime($this->year . "-" . $this->month . "-" . $this->day);
    $datetime->setTime(0, 0, 0);

    $scaffold_data = [
      'calendar_id' => $this->generateCalendarId(),
      'month' => $this->month,
      'year' => $this->year,
      'day' => $this->day,
      'hours' => DateTimeHelper::getHours(),
      'full_date' => $this->year . "-" . $this->month . "-" . $this->day,
    ];

    foreach (DateTimeHelper::getWeekdaysByDate($datetime) as $date) {
      foreach (DateTimeHelper::getHours() as $hour => $label) {
        $scaffold_data['dates'][$date][$hour] = [];
      }
    }

    return $scaffold_data;
  }

  /**
   * Build data structure for day calendar view.
   *
   * @return array
   *   The data for the calendar.
   *
   * @throws \Exception
   */
  protected function buildDayCalendarDataStructure() {

    $datetime = new \DateTime($this->year . "-" . $this->month . "-" . $this->day);
    $datetime->setTime(0, 0, 0);

    $scaffold_data = [
      'calendar_id' => $this->generateCalendarId(),
      'month' => $this->month,
      'year' => $this->year,
      'day' => $this->day,
      'label' => $this->year . "-" . $this->month . "-" . $this->day,
      'full_date' => $this->year . "-" . $this->month . "-" . $this->day,
    ];

    foreach (DateTimeHelper::getHours() as $key => $label) {
      $scaffold_data['hours'][$key] = [];
    }

    return $scaffold_data;
  }

  /**
   * Place Nodes in Calendar.
   *
   * @param array $calendar
   *   Calendar array.
   * @param array $node_basic_data
   *   Array with node basic data.
   * @param int|null $compact_view_threshold
   *   A threshold number at which point we should compact node view.
   */
  protected function placeNodesInMonthCalendar(array &$calendar, array $node_basic_data, ?int $compact_view_threshold = 5) {

    foreach ($node_basic_data as $node_rows) {

      foreach ($node_rows as $node_row) {

        $calendar_entry = new CalendarEntry(
          $this->month,
          $this->year,
          $this->day,
          $this->contentModerationService->getCurrentStateLabel('node', $node_row->type, $node_row->moderation_state),
          $this->getNodeTypeConfig($node_row->type),
          $node_row
        );

        foreach ($calendar['days'] as &$day) {

          // If date of entry is the current date of the calendar day.
          if ($day['date'] == $calendar_entry->formatSchedulingDateAsMySqlDateOnly()) {
            // Generate a unique key within the day for the entry.
            $key = $calendar_entry->getRelevantDate() . '-' . $calendar_entry->getNodeId();

            $day['nodes'][$key] = $calendar_entry->build();
          }
        }
      }
    }

    foreach ($calendar['days'] as $key => $data) {
      $this->sortCalendarEntriesByDateAndTime($calendar['days'][$key]['nodes']);

      // Apply the compact view.
      if ($data['nodes'] && count($data['nodes']) >= $compact_view_threshold) {
        foreach ($calendar['days'][$key]['nodes'] as $nkey => $entry) {
          $calendar['days'][$key]['nodes'][$nkey]["#is_compact"] = TRUE;
        }
      }
    }
  }

  /**
   * Place Nodes in Week Calendar structure.
   *
   * @param array $calendar
   *   Calendar array.
   * @param array $node_basic_data
   *   Array with node basic data.
   */
  protected function placeNodesInWeekCalendar(array &$calendar, array $node_basic_data) {
    foreach ($node_basic_data as $node_rows) {

      foreach ($node_rows as $node_row) {
        // Only show for selected week.
        if ($node_row->publish_on) {
          $node_date = strtotime(date("Y-n-d", $node_row->publish_on));
        }
        else {
          $node_date = strtotime(date("Y-n-d", $node_row->created));
        }

        $dates = array_keys($calendar['dates']);
        $week_start_date = array_shift($dates);
        $week_end_date = end($dates);

        $week_start_date = strtotime($week_start_date);
        $week_end_date = strtotime($week_end_date);

        if ($node_date <= $week_end_date && $node_date >= $week_start_date) {
          $calendar_entry = new CalendarEntry(
            $this->month,
            $this->year,
            $this->day,
            $this->contentModerationService->getCurrentStateLabel('node', $node_row->type, $node_row->moderation_state),
            $this->getNodeTypeConfig($node_row->type),
            $node_row
          );

          foreach ($calendar['dates'] as $date => $hours) {
            $relevent_day = date("Y-m-d", $calendar_entry->getRelevantDate());
            if ($relevent_day != $date) {
              continue;
            }

            foreach ($calendar['dates'][$date] as $hour => $label) {
              $relevant_date_hour = date("h A", $calendar_entry->getRelevantDate());
              if ($relevant_date_hour == $hour) {
                $calendar['dates'][$date][$hour][] = $calendar_entry->build();
              }
            }

          }
        }
      }
    }

    // Sort  all node types.
    foreach ($calendar['dates'] as $date_key => $date_hours) {
      foreach ($calendar['dates'][$date_key] as $hour => $values) {
        $this->sortCalendarEntriesByDateAndTime($calendar['dates'][$date_key][$hour]);
      }
    }
  }

  /**
   * Place Nodes in Day Calendar structure.
   *
   * @param array $calendar
   *   Calendar array.
   * @param array $node_basic_data
   *   Array with node basic data.
   */
  protected function placeNodesInDayCalendar(array &$calendar, array $node_basic_data) {
    foreach ($node_basic_data as $node_rows) {
      foreach ($node_rows as $node_row) {

        // Only show for selected day.
        if ($node_row->publish_on) {
          $node_date = date("Y-n-d", $node_row->publish_on);
        }
        else {
          $node_date = date("Y-n-d", $node_row->created);
        }

        $calendar_day_date = date("Y-n-d", strtotime($this->year . '-' . $this->month . '-' . $this->day));
        if ($node_date != $calendar_day_date) {
          continue;
        }

        $calendar_entry = new CalendarEntry(
          $this->month,
          $this->year,
          $this->day,
          $this->contentModerationService->getCurrentStateLabel('node', $node_row->type, $node_row->moderation_state),
          $this->getNodeTypeConfig($node_row->type),
          $node_row
        );

        foreach ($calendar['hours'] as $key => $label) {
          $relavant_date_hour = date("h A", $calendar_entry->getRelevantDate());
          if ($relavant_date_hour == $key) {
            $calendar['hours'][$key][] = $calendar_entry->build();
          }
        }
      }
    }

    foreach ($calendar['hours'] as $key => $data) {
      $this->sortCalendarEntriesByDateAndTime($calendar['hours'][$key]);
    }
  }

  /**
   * Place Nodes in list structure.
   *
   * @param array $list
   *   List array.
   * @param array $node_basic_data
   *   Array with node basic data.
   */
  protected function placeNodesInListView(array &$list, array $node_basic_data) {
    foreach ($node_basic_data as $node_rows) {

      foreach ($node_rows as $node_row) {

        $calendar_entry = new CalendarEntry(
          $this->month,
          $this->year,
          $this->day,
          $this->contentModerationService->getCurrentStateLabel('node', $node_row->type, $node_row->moderation_state),
          $this->getNodeTypeConfig($node_row->type),
          $node_row
        );

        foreach ($list['days'] as &$day) {

          // If date of entry is the current date of the calendar day.
          if ($day['date'] == $calendar_entry->formatSchedulingDateAsMySqlDateOnly()) {
            // Generate a unique key within the day for the entry.
            $key = $calendar_entry->getRelevantDate() . '-' . $calendar_entry->getNodeId();

            $day['nodes'][$key] = $calendar_entry->build();
          }
        }
      }
    }

    foreach ($list['days'] as $key => $data) {
      $this->sortCalendarEntriesByDateAndTime($list['days'][$key]['nodes']);
    }
  }

  /**
   * Sort array of calendar entries by date and time.
   *
   * @param array $entries
   *   The calendar entries.
   */
  protected function sortCalendarEntriesByDateAndTime(array &$entries) {
    usort($entries, function ($first, $second) {
      // Check publish date otherwise fallback on created.
      $first_node_date = $first['#node']->publish_on;
      if (!$first_node_date) {
        $first_node_date = $first['#node']->created;
      }

      // Check publish date otherwise fallback on created.
      $second_node_date = $second['#node']->publish_on;
      if (!$second_node_date) {
        $second_node_date = $second['#node']->created;
      }

      if ($first_node_date > $second_node_date) {
          return 1;
      }

      if ($first_node_date < $second_node_date) {
          return -1;
      }

      // Sort by title if the time is the same.
      if ($first_node_date == $second_node_date) {
        return strcmp($first['#node']->title, $second['#node']->title);
      }
    });
  }

  /**
   * Get Content Type config entity by Node Type.
   *
   * @param string $node_type
   *   The node type id to get the config.
   *
   * @return bool|\Drupal\content_calendar\Entity\ContentTypeConfig
   *   The content type config.
   */
  protected function getNodeTypeConfig($node_type) {

    if (array_key_exists($node_type, $this->contentTypeConfigEntities)) {
      return $this->contentTypeConfigEntities[$node_type];
    }

    return FALSE;
  }

  /**
   * Determines whether the current theme is Gin or a subtheme of Gin.
   *
   * @return bool
   *   TRUE if the current theme is Gin or a subtheme of Gin
   */
  protected function isGinThemeActive() {
    $theme = $this->themeManager->getActiveTheme();

    return $theme->getName() === 'gin' || isset($theme->getBaseThemeExtensions()['gin']);
  }

}
