<?php

namespace Drupal\content_calendar;

use Drupal\content_calendar\Form\SettingsForm;
use Drupal\content_calendar\Plugin\CalendarFilterPluginManager;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Implements ContentCalendarService class.
 */
class ContentCalendarService {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Drupal\content_calendar\ContentTypeConfigService definition.
   *
   * @var \Drupal\content_calendar\ContentTypeConfigService
   */
  protected $contentTypeConfigService;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * The filter plugin manager.
   *
   * @var \Drupal\content_calendar\Plugin\CalendarFilterPluginManager
   */
  protected $filterPluginManager;

  /**
   * Config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new ContentCalendarService object.
   */
  public function __construct(
    Connection $database,
    ContentTypeConfigService $content_type_config_service,
    EntityTypeManagerInterface $entity_type_manager,
    EntityFieldManagerInterface $entity_field_manager,
    CalendarFilterPluginManager $filter_plugin_manager,
    ConfigFactoryInterface $config_factory
  ) {
    $this->database = $database;
    $this->contentTypeConfigService = $content_type_config_service;
    $this->entityTypeManager = $entity_type_manager;
    $this->entityFieldManager = $entity_field_manager;
    $this->filterPluginManager = $filter_plugin_manager;
    $this->configFactory = $config_factory;
  }

  /**
   * Get Nodes.
   *
   * @param string $year
   *   Year.
   * @param string $month
   *   Month.
   * @param string $day
   *   Day.
   * @param array $filters
   *   An array with filter plugins.
   * @param array $daterange
   *   Daterange to be added to the query.
   *
   * @return array
   *   Return node fields data array.
   */
  public function getNodesByQuery(
    $year = NULL,
    $month = NULL,
    $day = NULL,
    array $filters = [],
    array $daterange = [],
  ) {
    $result = [];

    // Basic table.
    $query = $this->database->select('node_field_data', 'nfd');

    // Joins.
    $query->innerJoin(
      'users_field_data',
      'ufd',
      'nfd.uid = ufd.uid'
    );
    $query->innerJoin(
      'content_moderation_state_field_data',
      'cmfd',
      'nfd.nid = cmfd.content_entity_id AND cmfd.content_entity_type_id = :entity_type_id',
      [':entity_type_id' => 'node']
    );

    // Fields.
    $query->addField('nfd', 'nid');
    $query->addField('nfd', 'type');
    $query->addField('nfd', 'title');
    $query->addField('nfd', 'created');
    $query->addField('nfd', 'status');
    $query->addField('nfd', 'uid');
    $query->addField('ufd', 'name', 'username');
    $query->addField('nfd', 'publish_on');
    $query->addField('cmfd', 'moderation_state');

    // Sort.
    $query->orderBy('nfd.created', 'ASC');

    if ($daterange) {
      $orCondition = $query->orConditionGroup();
      $orCondition->condition("nfd.created", [$daterange['from_date'], $daterange['to_date']], 'BETWEEN');
      $orCondition->condition("nfd.publish_on", [$daterange['from_date'], $daterange['to_date']], 'BETWEEN');
      $query->condition($orCondition);
    }

    // Day filter.
    if ($year && $month && $day && !$daterange) {
      $datetime_from = new \DateTime();
      $datetime_from->setDate($year, $month, $day);
      $datetime_from->setTime(0, 0, 0);

      $datetime_to = new \DateTime();
      $datetime_to->setDate($year, $month, $day);
      $datetime_to->setTime(23, 59, 59);

      $orCondition = $query->orConditionGroup();
      $orCondition->condition("nfd.created", [$datetime_from->getTimestamp(), $datetime_to->getTimestamp()], 'BETWEEN');
      $orCondition->condition("nfd.publish_on", [$datetime_from->getTimestamp(), $datetime_to->getTimestamp()], 'BETWEEN');
      $query->condition($orCondition);
    }
    elseif($year && is_numeric($year) && !$daterange) {
      if ($month && is_numeric($month)) {
        $month_from = $month;
        $month_to = $month;
      }
      else {
        $month_from = 1;
        $month_to = 12;
      }

      // From datetime.
      $datetime_from = DateTimeHelper::getFirstDayOfMonth($month_from, $year);
      // To datetime.
      $datetime_to = DateTimeHelper::getLastDayOfMonth($month_to, $year);

      $orCondition = $query->orConditionGroup();
      $orCondition->condition("nfd.created", [$datetime_from->getTimestamp(), $datetime_to->getTimestamp()], 'BETWEEN');
      $orCondition->condition("nfd.publish_on", [$datetime_from->getTimestamp(), $datetime_to->getTimestamp()], 'BETWEEN');
      $query->condition($orCondition);
    }

    // Type filter.
    $node_type_ids = array_keys($this->contentTypeConfigService->loadAllEntities());
    $query->condition('nfd.type', $node_type_ids, 'IN');

    // Allow each filter to alter the query.
    foreach ($filters as $plugin_id => $value) {
      $plugin_instance = $this->filterPluginManager->createInstance($plugin_id);
      $query = $plugin_instance->alterQuery($query, $value);
    }

    // Add limit.
    $limit = $this->configFactory->get(SettingsForm::CONFIG_NAME)->get("max_results");
    if (!$limit) {
      $limit = 1000;
    }
    $query->range(0, $limit);

    $result = $query->execute()->fetchAll();

    return $result;
  }

}
