<?php

namespace Drupal\content_calendar\Controller;

use Drupal\content_calendar\DateTimeHelper;
use Drupal\content_calendar\FilterHelper;
use Drupal\Core\Controller\ControllerBase;
use Drupal\node\NodeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Implements CalendarController class.
 */
class CalendarController extends ControllerBase {

  /**
   * Drupal\content_calendar\ContentTypeConfigService definition.
   *
   * @var \Drupal\content_calendar\ContentTypeConfigService
   */
  protected $contentTypeConfigService;

  /**
   * Provides an interface for redirect destinations.
   *
   * @var \Drupal\Core\Routing\RedirectDestinationInterface
   */
  protected $redirectDestination;

  /**
   * The content calendar for a certain year and month.
   *
   * @var \Drupal\content_calendar\Component\Calendar
   */
  protected $calendar;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->contentTypeConfigService = $container->get('content_calendar.content_type_config_service');
    $instance->redirectDestination = $container->get('redirect.destination');
    $instance->calendar = $container->get('content_calendar.calendar');

    return $instance;
  }

  /**
   * Show Calendar default view.
   */
  public function showCalendar() {
    $config = $this->config("content_calendar.settings");
    $default_tab = 'month';

    $year = date("Y");
    $month = date("m");
    $day = date("d");

    if ($tab = $config->get('default_tab')) {
      $default_tab = $tab;
    }

    return $this->redirectToCalendar($default_tab, $year, $month, $day);
  }

  /**
   * Show list view.
   */
  public function showListView(?int $year) {
    $lists = [];

    if (!$year) {
      $year = date("o");
    }

    $this->calendar->setYear($year);

    if ($query = FilterHelper::getSavedSessionFilterValues()) {
      return $this->redirectToCalendar('list', $year, NULL, NULL, $query);
    }

    // Get content type config entities.
    $content_type_config_entities = $this->contentTypeConfigService->loadAllEntities();

    // Check if Content Calendar has been configured.
    if (!$content_type_config_entities) {
      $this->messenger()->addMessage($this->t('Content Calendar is not configured yet. Please do this in the settings tab.'), 'error');
      return [];
    }

    // Generate calendar structures.
    foreach (range(1, 12) as $month) {
      $this->calendar->setMonth($month);
      $lists[] = $this->calendar->buildListViewForMonth();
    }

    // Get Filter Form.
    $form_params = [
      'current_year' => date('Y'),
      'selected_year' => $year,
    ];
    $filters_form = $this->formBuilder()->getForm('Drupal\content_calendar\Form\CalenderOverviewFilterForm', $form_params);

    // Year is already included at the top, so it does not make
    // sense to have an additional filter.
    if (isset($filters_form['year'])) {
      unset($filters_form['year']);
    }

    if ($this->currentUser()->hasPermission('administer content calendar settings')) {
      $has_permission = TRUE;
    }
    else {
      $has_permission = FALSE;
    }

    $next_year = date('Y', strtotime("+12 months $year-01-01"));
    $prev_year = date('Y', strtotime("-12 months $year-01-01"));

    $build = [
      '#theme' => 'content_calendar_list_overview',
      '#show_year_switcher' => TRUE,
      '#current_year' => $year,
      '#prev_year_route_slug' => $prev_year,
      '#next_year_route_slug' => $next_year,
      '#lists' => $lists,
      '#filters_form' => $filters_form,
      '#filter_slug' => FilterHelper::buildFilterQuerySlug(),
      '#has_permission' => $has_permission,
    ];

    return $build;
  }

  /**
   * Show Calendar year.
   */
  public function showCalendarYear(?int $year) {

    $calendars = [];

    if (!$year) {
      $year = date("o");
    }

    if ($query = FilterHelper::getSavedSessionFilterValues()) {
      return $this->redirectToCalendar('year', $year, NULL, NULL, $query);
    }

    $this->calendar->setYear($year);

    // Get content type config entities.
    $content_type_config_entities = $this->contentTypeConfigService->loadAllEntities();

    // Check if Content Calendar has been configured.
    if (!$content_type_config_entities) {
      $this->messenger()->addMessage($this->t('Content Calendar is not configured yet. Please do this in the settings tab.'), 'error');
      return [];
    }

    // Generate calendar structures.
    foreach (range(1, 12) as $month) {
      $this->calendar->setMonth($month);
      $calendars[] = $this->calendar->buildMonthCalendar(false);
    }

    // Get Filter Form.
    $form_params = [
      'current_year' => date('Y'),
      'selected_year' => $year,
    ];
    $filters_form = $this->formBuilder()->getForm('Drupal\content_calendar\Form\CalenderOverviewFilterForm', $form_params);

    $months = [
      1 => $this->t('Jan'),
      2 => $this->t('Feb'),
      3 => $this->t('Mar'),
      4 => $this->t('Apr'),
      5 => $this->t('May'),
      6 => $this->t('Jun'),
      7 => $this->t('Jul'),
      8 => $this->t('Aug'),
      9 => $this->t('Sept'),
      10 => $this->t('Oct'),
      11 => $this->t('Nov'),
      12 => $this->t('Dec'),
    ];

    $filters_form["jump_links"] = [
      '#type' => 'item',
      '#title' => "Month",
    ];

    $filters_form['jump_links']['links'] = [
      '#theme' => 'content_calendar_jump_links',
      '#weight' => -1,
      '#months' => $months,
      '#year' => $form_params['selected_year'],
    ];

    if ($this->currentUser()->hasPermission('administer content calendar settings')) {
      $has_permission = TRUE;
    }
    else {
      $has_permission = FALSE;
    }

    $next_year = date('Y', strtotime("+12 months $year-01-01"));
    $prev_year = date('Y', strtotime("-12 months $year-01-01"));

    $build = [
      '#theme' => 'content_calendar_overview',
      '#show_year_switcher' => TRUE,
      '#current_year' => $year,
      '#calendars' => $calendars,
      '#prev_year_route_slug' => $prev_year,
      '#next_year_route_slug' => $next_year,
      '#filters_form' => $filters_form,
      '#filter_slug' => FilterHelper::buildFilterQuerySlug(),
      '#has_permission' => $has_permission,
    ];

    return $build;
  }

  /**
   * Show Calendar month.
   */
  public function showCalendarMonth(?int $year, ?int $month) {
    $build = [];

    if (!$year) {
      $year = date("o");
    }

    if (!$month) {
      $month = date("m");
    }

    $this->calendar->setYear($year);
    $this->calendar->setMonth($month);

    if ($query = FilterHelper::getSavedSessionFilterValues()) {
      return $this->redirectToCalendar('month', $year, $month, NULL, $query);
    }

    // Get content type config entities.
    $content_type_config_entities = $this->contentTypeConfigService->loadAllEntities();

    // Check if Content Calendar has been configured.
    if (!$content_type_config_entities) {
      $this->messenger()->addMessage($this->t('Content Calendar is not configured yet. Please do this in the settings tab.'), 'error');
      return [];
    }

    $form_params = [
      'current_year' => date('Y'),
      'selected_year' => $year,
    ];
    $filters_form = $this->formBuilder()->getForm('Drupal\content_calendar\Form\CalenderOverviewFilterForm', $form_params);

    // Months filter doesn't make sense here.
    unset($filters_form['jump_links']);

    if ($this->currentUser()->hasPermission('administer content calendar settings')) {
      $has_permission = TRUE;
    }
    else {
      $has_permission = FALSE;
    }

    $build = [
      '#theme' => 'content_calendar_overview',
      '#calendars' => [$this->calendar->buildMonthCalendar()],
      '#filters_form' => $filters_form,
      '#has_permission' => $has_permission,
    ];

    return $build;
  }

  /**
   * Show Calendar day.
   */
  public function showCalendarDay(?int $year, ?int $month, ?int $day) {
    $build = [];

    if (!$year) {
      $year = date("o");
    }

    if (!$month) {
      $month = date("m");
    }

    if (!$day) {
      $day = date("d");
    }

    $this->calendar->setYear($year);
    $this->calendar->setMonth($month);
    $this->calendar->setDay($day);

    if ($query = FilterHelper::getSavedSessionFilterValues()) {
      return $this->redirectToCalendar('month', $year, $month, $day, $query);
    }

    // Get content type config entities.
    $content_type_config_entities = $this->contentTypeConfigService->loadAllEntities();

    // Check if Content Calendar has been configured.
    if (!$content_type_config_entities) {
      $this->messenger()->addMessage($this->t('Content Calendar is not configured yet. Please do this in the settings tab.'), 'error');
      return [];
    }

    $form_params = [
      'current_year' => date('Y'),
      'selected_year' => $year,
    ];
    $filters_form = $this->formBuilder()->getForm('Drupal\content_calendar\Form\CalenderOverviewFilterForm', $form_params);

    if ($this->currentUser()->hasPermission('administer content calendar settings')) {
      $has_permission = TRUE;
    }
    else {
      $has_permission = FALSE;
    }

    // Months filter doesn't make sense here.
    unset($filters_form['jump_links']);

    $build = [
      '#theme' => 'content_calendar_overview',
      '#calendars' => [$this->calendar->buildDayCalendar()],
      '#filters_form' => $filters_form,
      '#has_permission' => $has_permission,
    ];

    return $build;
  }

  /**
   * Show Calendar week.
   */
  public function showCalendarWeek(?int $year, ?int $month, ?int $day) {
    $build = [];

    if (!$year) {
      $year = date("o");
    }

    if (!$month) {
      $month = date("m");
    }

    if (!$day) {
      $day = date("d");
    }

    $this->calendar->setYear($year);
    $this->calendar->setMonth($month);
    $this->calendar->setDay($day);

    if ($query = FilterHelper::getSavedSessionFilterValues()) {
      return $this->redirectToCalendar('month', $year, $month, $day, $query);
    }

    // Get content type config entities.
    $content_type_config_entities = $this->contentTypeConfigService->loadAllEntities();

    // Check if Content Calendar has been configured.
    if (!$content_type_config_entities) {
      $this->messenger()->addMessage($this->t('Content Calendar is not configured yet. Please do this in the settings tab.'), 'error');
      return [];
    }

    $form_params = [
      'current_year' => date('Y'),
      'selected_year' => $year,
    ];
    $filters_form = $this->formBuilder()->getForm('Drupal\content_calendar\Form\CalenderOverviewFilterForm', $form_params);

    // Months filter doesn't make sense here.
    unset($filters_form['jump_links']);

    if ($this->currentUser()->hasPermission('administer content calendar settings')) {
      $has_permission = TRUE;
    }
    else {
      $has_permission = FALSE;
    }

    $build = [
      '#theme' => 'content_calendar_overview',
      '#calendars' => [$this->calendar->buildWeekCalendar()],
      '#filters_form' => $filters_form,
      '#has_permission' => $has_permission,
    ];

    return $build;
  }

  /**
   * Update creation date of a given Node.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The node object.
   * @param string $date
   *   The date timestamp.
   * @param string $hour
   *   The date hour, defaults to created at.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   Return a Json Response.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function updateNodePublishDate(NodeInterface $node, $date, $hour = NULL) {

    $data = [
      'success' => FALSE,
      'message' => NULL,
    ];

    // Get content type config entities.
    $content_type_config_entities = $this->contentTypeConfigService->loadAllEntities();

    // Check for allowed types, marked in the Content Calendar settings.
    if (!array_key_exists($node->getType(), $content_type_config_entities)) {

      $data['message'] = $this->t('Action is not allowed for Nodes of type @type', ['@type' => $node->getType()]);
      return new JsonResponse($data);
    }

    // First – Update created on date!
    // Get the Node's "created on" date.
    $created_on_timestamp = $node->get('created')->getValue();
    $created_on_timestamp_value = $created_on_timestamp[0]['value'];
    // Return a date object.
    $original_created_on_datetime = DateTimeHelper::convertUnixTimestampToDatetime($created_on_timestamp_value);

    // Extract hour, minutes and seconds.
    if (!$hour) {
      $new_created_hour = $original_created_on_datetime->format('H');
    }
    else {
      $new_created_hour = date("H", strtotime($hour));
    }

    $minutes = $original_created_on_datetime->format('i');
    $seconds = $original_created_on_datetime->format('s');

    // Create a new datetime object from the given date.
    $new_created_on_datetime = \DateTime::createFromFormat('Y-m-d', $date);

    // Set hour, minutes and seconds.
    $new_created_on_datetime->setTime($new_created_hour, $minutes, $seconds);

    // Set created time.
    $node->set('created', $new_created_on_datetime->getTimestamp());

    // Second - Update publish on date! (only if publish on date is set)
    // Get publish on timestamp.
    $publish_on_timestamp = $node->get('publish_on')->getValue();
    $publish_on_timestamp_value = $publish_on_timestamp[0]['value'];

    // Only change scheduler publish on timestamp, when "publish on" is set.
    if ($publish_on_timestamp_value) {

      // Get the Node's publish ondate and return a datetime object.
      $original_publish_datetime = DateTimeHelper::convertUnixTimestampToDatetime($publish_on_timestamp_value);

      // Extract hour, minutes and seconds.
      if (!$hour) {
        $new_publish_hour = $original_publish_datetime->format('H');
      }
      else {
        $new_publish_hour = date("H", strtotime($hour));
      }
      $minutes = $original_publish_datetime->format('i');
      $seconds = $original_publish_datetime->format('s');

      // Create a new datetime object from the given date.
      $new_publish_datetime = \DateTime::createFromFormat('Y-m-d', $date);

      // Set hour, minutes and seconds.
      $new_publish_datetime->setTime($new_publish_hour, $minutes, $seconds);

      // Set publish on datetime.
      $node->set('publish_on', $new_publish_datetime->getTimestamp());

      // Set created on datetime.
      $node->set('created', $new_publish_datetime->getTimestamp());
    }

    // Save.
    if ($node->save() == SAVED_UPDATED) {
      $data['success'] = TRUE;
      $data['message'] = $this->t('The creation date for Node @id has been updated', ['@id' => $node->id()]);
    }

    return new JsonResponse($data);
  }

  /**
   * Redirect and jump to a given Calendar directly.
   *
   * @param string $view
   *   Calendar view.
   * @param string $year
   *   Calendar year.
   * @param string $month
   *   Calendar month.
   * @param string $day
   *   Calendar day.
   * @param array $query
   *   URL query.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   A redirect response object that may be returned by the controller.
   */
  public function redirectToCalendar($view, $year, $month, $day, array $query = NULL) {
    switch ($view) {
      case "day":
        $month = date("m", strtotime($year . '-' . $month . '-' . $day));
        $day = date("d", strtotime($year . '-' . $month . '-' . $day));

        return $this->redirect(
          'content_calendar.calendar_day',
          [
            'year' => $year,
            'month' => $month,
            'day' => $day,
          ],
          [
            'query' => $query,
          ],
        );

      case "week":
        $month = date("m", strtotime($year . '-' . $month . '-' . $day));
        $day = date("d", strtotime($year . '-' . $month . '-' . $day));

        return $this->redirect(
          'content_calendar.calendar_week',
          [
            'year' => $year,
            'month' => $month,
            'day' => $day,
          ],
          [
            'query' => $query,
          ],
        );

      case "month":
        $month = date("m", strtotime($year . '-' . $month));

        return $this->redirect(
          'content_calendar.calendar_month',
          [
            'year' => $year,
            'month' => $month,
          ],
          [
            'query' => $query,
          ],
        );

      case "list":
        return $this->redirect(
          'content_calendar.calendar_list',
          [
            'year' => $year,
          ],
          [
            'query' => $query,
          ],
        );

      default:
        $month = date("m", strtotime($year . '-' . $month));

        return $this->redirect(
          'content_calendar.calendar_year',
          ['year' => $year],
          [
            'query' => $query,
          ],
        );
    }
  }

  /**
   * Duplicate Node.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The node object.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   A redirect response object that may be returned by the controller.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function duplicateNode(NodeInterface $node) {

    $duplicate = $node->createDuplicate();

    $duplicate->setTitle($duplicate->getTitle() . ' clone');

    $duplicate->save();

    $destination = $this->redirectDestination->get();

    return new RedirectResponse($destination);
  }

}
