<?php

namespace Drupal\content_calendar;

use DateInterval;
use DatePeriod;
use DateTime;
use Drupal\Core\Datetime\DateHelper;

/**
 * Implements DateTimeHelper class.
 */
abstract class DateTimeHelper {

  /**
   * Regex for mysql date only values.
   */
  const FORMAT_MYSQL_DATE_ONLY_REGEX = '\d{4}\-\d{2}\-\d{2}';

  /**
   * Format value for mysql date only values.
   */
  const FORMAT_MYSQL_DATE_ONLY = 'Y-m-d';

  /**
   * Get Month label by its number.
   *
   * @param int $number
   *   Number of a month.
   *
   * @return bool|mixed
   *   Return label of the month.
   */
  public static function getMonthLabelByNumber($number) {

    if (is_numeric($number) && ($number >= 1 && $number <= 12)) {

      $month_labels = [
        1 => t('January'),
        2 => t('February'),
        3 => t('March'),
        4 => t('April'),
        5 => t('May'),
        6 => t('June'),
        7 => t('July'),
        8 => t('August'),
        9 => t('September'),
        10 => t('October'),
        11 => t('November'),
        12 => t('December'),
      ];

      return $month_labels[$number];
    }

    return FALSE;
  }

  /**
   * Get the count of days in a given month of a given year.
   *
   * @param int $month
   *   The month to display in the calendar.
   * @param int $year
   *   The year to display in the calendar.
   *
   * @return int
   *   Return the days count of the month.
   */
  public static function getDayCountInMonth($month, $year): int {
    return (int) date('t', strtotime($year . '-' . $month . '-01'));
  }

  /**
   * Get the first day of a given month and year.
   *
   * @param int $month
   *   The month to display in the calendar.
   * @param int $year
   *   The year to display in the calendar.
   *
   * @return \DateTime
   *   Return the first day of the month.
   */
  public static function getFirstDayOfMonth($month, $year) {
    $datetime = new \DateTime();

    $datetime->setDate($year, $month, 1);
    $datetime->setTime(0, 0, 0);

    return $datetime;
  }

  /**
   * Get the last day of a given month and year.
   *
   * @param int $month
   *   The month to display in the calendar.
   * @param int $year
   *   The year to display in the calendar.
   *
   * @return \DateTime
   *   Return the last day of the month.
   */
  public static function getLastDayOfMonth($month, $year) {

    $datetime = new \DateTime();

    $datetime->setDate($year, $month, 1);
    $datetime->setTime(23, 59, 59);

    $datetime->modify('last day of this month');

    return $datetime;
  }

  /**
   * Convert unix timestamp to Datetime object.
   *
   * @param int $unix_timestamp
   *   Timestamp integer.
   *
   * @return \DateTime
   *   Return datetime.
   */
  public static function convertUnixTimestampToDatetime(int $unix_timestamp) {

    $datetime = new \DateTime();
    $datetime->setTimestamp($unix_timestamp);

    return $datetime;
  }

  /**
   * Check is a given string is a date of the MySQL Date Only format.
   *
   * @param string $value
   *   Value string.
   *
   * @return false|int
   *   Return int or false.
   */
  public static function dateIsMySqlDateOnly($value) {
    return preg_match("/" . self::FORMAT_MYSQL_DATE_ONLY_REGEX . "/", $value);
  }

  /**
   * Gets the weekdays based on the first weekday set in Regional settings.
   *
   * @return array
   *   Returns an array with the translated days of week.
   */
  public static function getWeekdays() {
    $weekdays = DateHelper::weekDays(TRUE);
    // We reset the keys to keep the same order for them.
    return array_values(DateHelper::weekDaysOrdered($weekdays));
  }

  /**
   * Gets the weekdays based on the start date.
   *
   * @return array
   *   Returns an array with the translated days of week.
   */
  public static function getWeekdaysByDate(\Datetime $date) {
    $config = \Drupal::configFactory()->get('system.date');
    $first_day_of_week_number = $config->get("first_day");
    $first_day_of_week = self::getWeekdays()[0]->getUntranslatedString();
    $weekdays = self::getWeekdays();
    $last_day_of_week = end($weekdays);
    $last_day_of_week = $last_day_of_week->__toString();

    $ts = strtotime($date->format('Y-m-d H:i:s'));
    $start = (date('w', $ts) == $first_day_of_week_number) ? $ts : strtotime('last ' . $first_day_of_week, $ts);

    $start_date = date_create(date('Y-m-d', $start));
    $end_date = strtotime("+7 day", $start);
    $end_date = date_create(date('Y-m-d', $end_date));

    $interval = DateInterval::createFromDateString('1 day');
    $daterange = new DatePeriod($start_date, $interval, $end_date);

    $result = [];

    /** @var \DateTime $datetime */
    foreach ($daterange as $datetime) {
      $result[] = $datetime->format("Y-m-d");
    }

    return $result;
  }

  /**
   * Gets the hours in a day.
   *
   * @return array
   *   Returns an array with the translated days of week.
   */
  public static function getHours() {
    return [
      '12 AM' => t("12 AM"),
      '01 AM' => t("1 AM"),
      '02 AM' => t("2 AM"),
      '03 AM' => t("3 AM"),
      '04 AM' => t("4 AM"),
      '05 AM' => t("5 AM"),
      '06 AM' => t("6 AM"),
      '07 AM' => t("7 AM"),
      '08 AM' => t("8 AM"),
      '09 AM' => t("9 AM"),
      '10 AM' => t("10 AM"),
      '11 AM' => t("11 AM"),
      '12 PM' => t("12 PM"),
      '01 PM' => t("1 PM"),
      '02 PM' => t("2 PM"),
      '03 PM' => t("3 PM"),
      '04 PM' => t("4 PM"),
      '05 PM' => t("5 PM"),
      '06 PM' => t("6 PM"),
      '07 PM' => t("7 PM"),
      '08 PM' => t("8 PM"),
      '09 PM' => t("9 PM"),
      '10 PM' => t("10 PM"),
      '11 PM' => t("11 PM"),
    ];
  }

  /**
   * Gets the months in a year.
   *
   * @return array
   *   Returns an array with the translated days of month.
   */
  public static function getMonths() {
    $months = [];

    for ($i = 1; $i <= 12; $i++) {
      $months[$i] = self::getMonthLabelByNumber($i);
    }

    return $months;
  }

  /**
   * Gets the day of week for the given datetime object.
   *
   * This is done by respecting the first day of week configuration from Drupal.
   *
   * @param \DateTime $dateTime
   *   The datetime object.
   *
   * @return false|int|string
   *   Returns the representative number for the weekday for the given date.
   */
  public static function getDayOfWeekByDate(\DateTime $dateTime) {
    $weekdays = self::getWeekdays();
    $dayName = t($dateTime->format('l'));
    return array_search($dayName, $weekdays, FALSE) + 1;
  }

}
