<?php

namespace Drupal\content_calendar\Plugin\CalendarFilter;

use Drupal\content_calendar\Plugin\CalendarFilterPluginBase;
use Drupal\content_calendar\Plugin\CalendarFilterWidgetPluginManager;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Database\Connection;
use Drupal\Core\Database\Query\SelectInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Author calendar filter.
 *
 * @CalendarFilterPlugin(
 *   id = "author",
 *   label = @Translation("Author"),
 *   weight = 4,
 *   query_id = "author"
 * )
 */
class Author extends CalendarFilterPluginBase {

  /**
   * Database service.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    CalendarFilterWidgetPluginManager $filter_widget_plugin_manager,
    ConfigFactory $config_factory,
    EntityTypeManagerInterface $entity_type_manager,
    RouteMatchInterface $route_match,
    Connection $database
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $filter_widget_plugin_manager, $config_factory, $entity_type_manager, $route_match);
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.calendar_filter_widget_plugin.processor'),
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('current_route_match'),
      $container->get('database')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];
    $definition = $this->getPluginDefinition();
    $filter_settings = $this->getSettings();

    if (!$filter_settings['enable']) {
      return;
    }

    $widget = $this->filterWidgetPluginManager()
      ->createInstance(
        $filter_settings['widget']
    );

    $build = $widget->build([
      'id' => $this->getPluginId(),
      'label' => $definition['label'],
      'query_key' => $definition['query_id'],
      'options' => $this->getAuthors(),
    ]);
    $build['#weight'] = $definition['weight'];

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function alterQuery(SelectInterface $query, $value) {
    $uids = explode(",", $value);

    $settings = $this->getSettings();
    $widget = $this->filterWidgetPluginManager->createInstance($settings['widget']);
    $widget_definiton = $widget->getPluginDefinition();

    // Add support for multiple values.
    if (isset($widget_definiton['multiple_values']) && $widget_definiton['multiple_values']) {
      $or_condition_group = $query->orConditionGroup();

      foreach ($uids as $uid) {
        $or_condition_group->condition('nfd.uid', $uid);
      }

      $query->condition($or_condition_group);
      return $query;
    }

    foreach ($uids as $uid) {
      $query->condition('nfd.uid', $uid);
    }

    return $query;
  }

  /**
   * Get all authors.
   *
   * @return array
   *   Array with user id as key and user label as value.
   */
  protected function getAuthors() {
    $query = $this->database->select('node_field_data', 'nfd');
    $query->addField('nfd', 'uid');
    $query->groupBy('uid');
    $result = $query->execute()->fetchAll();

    $authors_ids = [];
    foreach ($result as $item) {
      $authors_ids[] = $item->uid;
    }

    $users = $this->entityTypeManager()
      ->getStorage("user")
      ->loadMultiple($authors_ids);

    $users_options = [];
    foreach ($users as $user) {
      $users_options[$user->id()] = $user->label();
    }

    return $users_options;
  }

  /**
   * {@inheritdoc}
   */
  public function ignoredWidgets() {
    return [
      "textfield",
    ];
  }

}
