<?php

namespace Drupal\content_calendar\Plugin\CalendarFilter;

use Drupal\content_calendar\Plugin\CalendarFilterPluginBase;
use Drupal\content_calendar\Plugin\CalendarFilterWidgetPluginManager;
use Drupal\content_moderation\ModerationInformationInterface;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Database\Query\SelectInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Content type calendar filter.
 *
 * @CalendarFilterPlugin(
 *   id = "content_type",
 *   label = @Translation("Content Type"),
 *   weight = 2,
 *   query_id = "type"
 * )
 */
class ContentType extends CalendarFilterPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The content moderation information service.
   *
   * @var \Drupal\content_moderation\ModerationInformationInterface
   */
  protected $contentModerationInformation;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    CalendarFilterWidgetPluginManager $filter_widget_plugin_manager,
    ConfigFactory $config_factory,
    EntityTypeManagerInterface $entity_type_manager,
    RouteMatchInterface $route_match,
    ModerationInformationInterface $content_moderation_information
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $filter_widget_plugin_manager, $config_factory, $entity_type_manager, $route_match);
    $this->contentModerationInformation = $content_moderation_information;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.calendar_filter_widget_plugin.processor'),
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('current_route_match'),
      $container->get('content_moderation.moderation_information')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];
    $definition = $this->getPluginDefinition();
    $filter_settings = $this->getSettings();

    if (!$filter_settings['enable']) {
      return;
    }

    $widget = $this->filterWidgetPluginManager()
      ->createInstance(
        $filter_settings['widget']
    );


    $build = $widget->build([
      'id' => $this->getPluginId(),
      'label' => $definition['label'],
      'query_key' => $definition['query_id'],
      'options' => $this->getConfiguredContentTypes(),
    ]);
    $build['#weight'] = $definition['weight'];

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function alterQuery(SelectInterface $query, $value) {
    $types = explode(",", $value);

    $settings = $this->getSettings();
    $widget = $this->filterWidgetPluginManager->createInstance($settings['widget']);
    $widget_definiton = $widget->getPluginDefinition();

    // Add support for multiple values.
    if (isset($widget_definiton['multiple_values']) && $widget_definiton['multiple_values']) {
      $or_condition_group = $query->orConditionGroup();
      foreach ($types as $type) {
        $or_condition_group->condition('nfd.type', $type);
      }

      $query->condition($or_condition_group);
      return $query;
    }

    foreach ($types as $type) {
      $query->condition('nfd.type', $type);
    }

    return $query;
  }

  /**
   * Get configured content types.
   *
   * @return array
   *   Returns an array with display options.
   */
  protected function getConfiguredContentTypes() {

    $display_options = [];

    // Load Node Type configurations.
    $definition = $this->entityTypeManager()
      ->getDefinition('node');
    $node_types = $this->entityTypeManager()
      ->getStorage('node_type')
      ->loadMultiple();

    foreach ($node_types as $node_type_key => $node_type) {
      // Exclude node types without scheduler.
      if (!$scheduler = $node_type->getThirdPartySettings('scheduler')) {
        continue;
      }
      if (empty($scheduler['publish_enable'])) {
        continue;
      }

      // Exclude node types without content moderation.
      if (!$this->contentModerationInformation->shouldModerateEntitiesOfBundle($definition, $node_type_key)) {
        continue;
      }

      $display_options[$node_type_key] = $node_type->label();
    }

    return $display_options;
  }

  /**
   * {@inheritdoc}
   */
  public function ignoredWidgets() {
    return [
      "textfield",
    ];
  }

}
