<?php

namespace Drupal\content_calendar\Plugin\CalendarFilter;

use Drupal\content_calendar\Plugin\CalendarFilterPluginBase;
use Drupal\Core\Database\Query\SelectInterface;
use Drupal\content_calendar\Plugin\CalendarFilterWidgetPluginManager;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\RedirectDestinationInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Taxonomy vocabulary calendar filter.
 *
 * @CalendarFilterPlugin(
 *   id = "taxonomy_vocabulary",
 *   label = @Translation("Taxonomy Vocabulary"),
 *   weight = 3,
 *   query_id = "term"
 * )
 */
class TaxonomyVocabulary extends CalendarFilterPluginBase {

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * The redirect destination helper.
   *
   * @var \Drupal\Core\Routing\RedirectDestinationInterface
   */
  protected $destination;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    CalendarFilterWidgetPluginManager $filter_widget_plugin_manager,
    ConfigFactory $config_factory,
    EntityTypeManagerInterface $entity_type_manager,
    RouteMatchInterface $route_match,
    EntityFieldManagerInterface $entity_field_manager,
    RedirectDestinationInterface $destination,
    AccountProxyInterface $current_user,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $filter_widget_plugin_manager, $config_factory, $entity_type_manager, $route_match);
    $this->entityFieldManager = $entity_field_manager;
    $this->destination = $destination;
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.calendar_filter_widget_plugin.processor'),
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('current_route_match'),
      $container->get('entity_field.manager'),
      $container->get('redirect.destination'),
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildSettingsForm() {
    $form = [];

    $definition = $this->getPluginDefinition();
    $settings = $this->getSettings();

    $taxonomy_vocabularies = $this->entityTypeManager()
      ->getStorage('taxonomy_vocabulary')
      ->loadMultiple();

    $taxonomy_vocabularies_options = [];

    foreach ($taxonomy_vocabularies as $vocabulary) {
      $taxonomy_vocabularies_options[$vocabulary->id()] = $vocabulary->label();
    }

    $form = [
      '#type' => 'details',
      '#title' => "Taxonomy",
      '#open' => TRUE,
      '#tree' => TRUE,
      '#weight' => $definition['weight'],
    ];

    foreach ($taxonomy_vocabularies as $vocabulary) {
      $form[$vocabulary->id()] = [
        '#type' => 'fieldset',
        '#title' => $vocabulary->label(),
        '#open' => FALSE,
        '#tree' => TRUE,
      ];

      $form[$vocabulary->id()]['enable'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Enable @label filter.', ['@label' => $vocabulary->label()]),
        '#required' => FALSE,
        '#default_value' => isset($settings[$vocabulary->id()]['enable']) ? $settings[$vocabulary->id()]['enable'] : FALSE,
      ];

      $form[$vocabulary->id()]['widget'] = [
        '#type' => 'select',
        '#title' => $this->t('Which widget to use for displaying?'),
        '#options' => $this->getAvailableWidgets(),
        '#default_value' => isset($settings[$vocabulary->id()]['widget']) ? $settings[$vocabulary->id()]['widget'] : "",
      ];

    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function alterQuery(SelectInterface $query, $value) {
    $taxonomy_reference_fields = [];
    $terms = explode(",", $value);
    $settings = $this->getSettings();

    // Group terms by vocabulary.
    $terms_by_vocabulary = [];
    foreach ($terms as $tid) {
      $term = $this->entityTypeManager()->getStorage("taxonomy_term")->load($tid);
      if (!$term) {
        continue;
      }

      $vocabulary = $this->entityTypeManager()->getStorage("taxonomy_vocabulary")->load($term->vid->target_id);
      $terms_by_vocabulary[$vocabulary->id()][] = $tid;
    }

    $vocabularies = array_keys($terms_by_vocabulary);
    foreach ($vocabularies as $vid) {
      $tids = $terms_by_vocabulary[$vid];
      $taxonomy_reference_fields = $this->getTaxonomyReferenceFields($vid);

      $vocabulary_filter_settings = $settings[$vid];
      $widget = $this->filterWidgetPluginManager->createInstance($vocabulary_filter_settings['widget']);
      $widget_definition = $widget->getPluginDefinition();

      // Add support for OR.
      if (isset($widget_definition['multiple_values']) && $widget_definition['multiple_values']) {
        $query = $this->addFieldCondition($query, $taxonomy_reference_fields, $tids);
        continue;
      }

      foreach ($tids as $tid) {
        $query = $this->addFieldCondition($query, $taxonomy_reference_fields, $tid);
      }

    }

    return $query;
  }

  /**
   * Join field tables and add condition to the query.
   *
   * @return \Drupal\Core\Database\Query\SelectInterface
   *   Modified query.
   */
  protected function addFieldCondition(SelectInterface $query, $fields, $value) {
    foreach ($fields as $field_name => $field_table_name) {
      $query->innerJoin(
        $field_table_name,
        $field_name,
        'nfd.nid = ' . $field_name . '.entity_id'
      );

      $query->condition($field_name . '.' . $field_name . '_target_id', $value, "IN", "OR");
    }

    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];
    $filter_settings = $this->getSettings();
    $definition = $this->getPluginDefinition();

    foreach ($filter_settings as $vid => $vid_settings) {
      $vocabulary = $this->entityTypeManager()
        ->getStorage("taxonomy_vocabulary")
        ->load($vid);

      if (!$vid_settings['enable']) {
        continue;
      }

      $widget = $this->filterWidgetPluginManager()
        ->createInstance(
          $vid_settings['widget']
      );

      $build[$vid] = $widget->build([
        'id' => 'taxonomy',
        'vid' => $vid,
        'query_key' => $definition['query_id'],
        'label' => $vocabulary->label(),
        'options' => $this->getTerms($vid),
      ]);

      $url = Url::fromUserInput('/admin/structure/taxonomy/manage/' . $vid . '/overview');
      $url->setOption('query', ['destination' => $this->destination->get()]);

      $reset = NULL;
      if (isset($build[$vid]['header']['reset'])) {
        $reset = $build[$vid]['header']['reset'];
      }

      $build[$vid]['header']['label'] = [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['label-with-edit'],
        ],
      ];

      $build[$vid]['header']['label']['text'] = [
        '#type' => 'item',
        '#title' => $vocabulary->label(),
      ];

      if ($this->currentUser->hasPermission('access taxonomy overview')) {
        $build[$vid]['header']['label']['edit'] = [
          '#type' => 'link',
          '#title' => "[E]",
          '#url' => $url,
        ];
      }

      if ($reset) {
       $build[$vid]['header']['reset'] = $reset;
      }

      $build['#weight'] = $definition['weight'];
    }

    return $build;
  }

  /**
   * Get terms in a vocabulary.
   *
   * @param string $vid
   *   Taxonomy id.
   *
   * @return array
   *   Array with terms.
   */
  public function getTerms($vid) {
    $terms = $this->entityTypeManager()
      ->getStorage('taxonomy_term')
      ->loadByProperties(['vid' => $vid]);

    $term_options = [];
    foreach ($terms as $term) {
      $term_options[$term->id()] = $term->label();
    }

    return $term_options;
  }

  /**
   * Get all taxonomy references fields for a specific vocabulary.
   *
   * @param string $vid
   *   Vocabulary id.
   *
   * @return array
   *   Array of field names.
   */
  protected function getTaxonomyReferenceFields(string $vid) {
    $taxonomy_reference_fields = [];
    $node_types = $this->entityTypeManager()->getStorage("node_type")->loadMultiple();

    foreach (array_keys($node_types) as $type) {
      $field_definition = $this->entityFieldManager->getFieldDefinitions('node', $type);
      $node_storage = $this->entityTypeManager->getStorage("node");

      /** @var \Drupal\field\Entity\FieldConfig  $definition */
      foreach ($field_definition as $key => $definition) {
        if ($definition->getType() != "entity_reference") {
          continue;
        }

        $handler = $definition->getSetting('handler');
        if ($handler != "default:taxonomy_term") {
          continue;
        }

        $handler_settings = $definition->getSetting('handler_settings');
        if (!$handler_settings || !isset($handler_settings['target_bundles'])) {
          continue;
        }

        $target_bundles = $handler_settings['target_bundles'];
        if (in_array($vid, $target_bundles)) {
          $table_name = $node_storage->getTableMapping()->getAllFieldTableNames($key);
          if ($table_name) {
            $taxonomy_reference_fields[$key] = $table_name[0];
          }
        }
      }
    }

    return $taxonomy_reference_fields;
  }

  /**
   * {@inheritdoc}
   */
  public function ignoredWidgets() {
    return [
      "textfield",
    ];
  }

}
