<?php

namespace Drupal\content_calendar\Plugin;

use Drupal\Component\Plugin\PluginBase;
use Drupal\content_calendar\Form\SettingsForm;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Database\Query\SelectInterface;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for CalendarFilterPluginBase plugins.
 */
abstract class CalendarFilterPluginBase extends PluginBase implements CalendarFilterPluginInterface, ContainerFactoryPluginInterface {

  use StringTranslationTrait, DependencySerializationTrait;

  /**
   * The filter widget plugin manager.
   *
   * @var \Drupal\content_calendar\Plugin\CalendarFilterWidgetPluginManager
   */
  protected $filterWidgetPluginManager;

  /**
   * Configuration object.
   *
   * @var \Drupal\Core\Config\Config|\Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * Provides an interface for entity type managers.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Route match service.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    CalendarFilterWidgetPluginManager $filter_widget_plugin_manager,
    ConfigFactory $config_factory,
    EntityTypeManagerInterface $entity_type_manager,
    RouteMatchInterface $route_match
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->filterWidgetPluginManager = $filter_widget_plugin_manager;
    $this->config = $config_factory->get(SettingsForm::CONFIG_NAME);
    $this->entityTypeManager = $entity_type_manager;
    $this->routeMatch = $route_match;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.calendar_filter_widget_plugin.processor'),
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('current_route_match')
    );
  }

  /**
   * Get entity type manager.
   */
  protected function entityTypeManager() {
    return $this->entityTypeManager;
  }

  /**
   * Get filter widget plugin manager.
   */
  protected function filterWidgetPluginManager() {
    return $this->filterWidgetPluginManager;
  }

  /**
   * Get route match.
   */
  protected function routeMatch() {
    return $this->routeMatch;
  }

  /**
   * Build filter settings form.
   *
   * @return array
   *   Render array.
   */
  public function buildSettingsForm() {
    $form = [];

    $definition = $this->getPluginDefinition();

    $form = [
      '#type' => 'details',
      '#title' => $definition['label'],
      '#open' => TRUE,
      '#tree' => TRUE,
      '#weight' => $definition['weight'],
    ];

    $form['enable'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable @label filter.', ['@label' => $definition['label']]),
      '#required' => FALSE,
      '#default_value' => isset($this->getSettings()['enable']) ? $this->getSettings()['enable'] : FALSE,
    ];

    $widget_options = $this->getAvailableWidgets();
    foreach ($widget_options as $id => $plugin) {
      if (in_array($id, $this->ignoredWidgets())) {
        unset($widget_options[$id]);
      }
    }

    $form['widget'] = [
      '#type' => 'select',
      '#title' => $this->t('Which widget to use for displaying?'),
      '#options' => $widget_options,
      '#default_value' => isset($this->getSettings()['widget']) ? $this->getSettings()['widget'] : "",
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function alterQuery(SelectInterface $query, $value) {
    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function getSettings() {
    $settings = $this->config->get("filter_configuration");

    $plugin_settings = [];

    if (isset($settings[$this->getPluginId()])) {
      $plugin_settings = $settings[$this->getPluginId()];
    }

    return $plugin_settings;
  }

  /**
   * A list of widget ids which are not applicable to this filter.
   *
   * @return array
   *   Array of widgets.
   */
  public function ignoredWidgets() {
    return [];
  }

  /**
   * Get widget id.
   *
   * @return string|null
   *   Widget id.
   */
  protected function getWidgetId() {
    $settings = $this->getSettings();

    if (isset($settings['widget'])) {
      return $settings['widget'];
    }

    return NULL;
  }

  /**
   * Calendar settings.
   *
   * @return \Drupal\Core\Config\ImmutableConfig
   *   Calendar config.
   */
  protected function getCalendarSettings() {
    return $this->config;
  }

  /**
   * Get all available widgets.
   */
  protected function getAvailableWidgets() {
    $widgets = [];

    $plugins = $this->filterWidgetPluginManager->getDefinitions();
    foreach ($plugins as $plugin) {
      $widgets[$plugin['id']] = $plugin['label'];
    }

    return $widgets;
  }

}
