<?php

namespace Drupal\content_calendar\Plugin;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Component\Utility\Html;
use Drupal\content_calendar\Form\SettingsForm;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\RedirectCommand;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Database\Connection;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Path\CurrentPathStack;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Base class for CalendarFilterWidgetPluginBase plugins.
 */
abstract class CalendarFilterWidgetPluginBase extends PluginBase implements CalendarFilterWidgetPluginInterface, ContainerFactoryPluginInterface {

  use StringTranslationTrait, DependencySerializationTrait;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $config;

  /**
   * The route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current requesst.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $currentRequest;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The calendar filter plugin manager.
   *
   * @var \Drupal\content_calendar\Plugin\CalendarFilterPluginManager
   */
  protected $filterPluginManager;

  /**
   * The calendar filter widget plugin manager.
   *
   * @var \Drupal\content_calendar\Plugin\CalendarFilterWidgetPluginManager
   */
  protected $filterWidgetPluginManager;

  /**
   * The tempstore factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The current path stack.
   *
   * @var \Drupal\Core\Path\CurrentPathStack
   */
  protected $currentPathStack;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    ConfigFactory $config_factory,
    RouteMatchInterface $route_match,
    EntityTypeManagerInterface $entity_type_manager,
    RequestStack $request,
    Connection $database,
    CalendarFilterPluginManager $filter_plugin_manager,
    CalendarFilterWidgetPluginManager $filter_widget_plugin_manager,
    PrivateTempStoreFactory $tempstore_factory,
    CurrentPathStack $current_path_stack
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->config = $config_factory->get(SettingsForm::CONFIG_NAME);
    $this->routeMatch = $route_match;
    $this->entityTypeManager = $entity_type_manager;
    $this->currentRequest = $request->getCurrentRequest();
    $this->database = $database;
    $this->filterPluginManager = $filter_plugin_manager;
    $this->filterWidgetPluginManager = $filter_widget_plugin_manager;
    $this->tempStoreFactory = $tempstore_factory;
    $this->currentPathStack = $current_path_stack;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('current_route_match'),
      $container->get('entity_type.manager'),
      $container->get('request_stack'),
      $container->get('database'),
      $container->get('plugin.manager.calendar_filter_plugin.processor'),
      $container->get('plugin.manager.calendar_filter_widget_plugin.processor'),
      $container->get('tempstore.private'),
      $container->get('path.current')
    );
  }

  /**
   * Route match service.
   *
   * @return \Drupal\Core\Routing\RouteMatchInterface
   *   Route match service.
   */
  protected function routeMatch() {
    return $this->routeMatch;
  }

  /**
   * Entity type manager.
   *
   * @return \Drupal\Core\Entity\EntityTypeManagerInterface
   *   Entity type manager.
   */
  protected function entityTypeManager() {
    return $this->entityTypeManager;
  }

  /**
   * Current request.
   *
   * @return \\Symfony\Component\HttpFoundation\Request
   *   Current request.
   */
  protected function currentRequest() {
    return $this->currentRequest;
  }

  /**
   * Database connection.
   *
   * @return \Drupal\Core\Database\Connection
   *   Database connection.
   */
  protected function database() {
    return $this->database;
  }

  /**
   * Config factory.
   *
   * @return \Drupal\Core\Config\ConfigFactory
   *   Database connection.
   */
  protected function configFactory() {
    return $this->config;
  }

  /**
   * Get current route.
   *
   * @return null|string
   *   The route name or NULL if not found.
   */
  protected function getCurrentRouteName() {
    return $this->routeMatch()->getRouteName();
  }

  /**
   * Get filter widget manager.
   *
   * @return \Drupal\content_calendar\Plugin\CalendarFilterWidgetPluginManager
   *   The filter widget manager.
   */
  protected function getCalendarFilterWidgetManager() {
    return $this->filterWidgetPluginManager;
  }

  /**
   * {@inheritdoc}
   */
  protected function buildFilterForm($header_title, $filter_query_id) {
    $build = [];

    $build['header'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ["header"],
      ],
    ];

    $build['header']['label'] = [
      '#type' => 'item',
      '#title' => $header_title,
    ];

    $is_content_type_filter_active = $this->currentRequest()->query->get($filter_query_id);
    if ($is_content_type_filter_active) {
      $build['header']['reset'] = [
        '#prefix' => "<div class='reset-btn'>",
        '#suffix' => "</div>",
        '#type' => 'button',
        '#name' => Html::getUniqueId('reset-btn'),
        '#id' => Html::getUniqueId('reset-btn'),
        '#value' => '(' . $this->t("Reset") . ')',
        '#ajax' => [
          'callback' => [$this, 'resetFilterAjaxCallback'],
        ],
      ];
    }

    $build['filter'] = [
      '#type' => 'container',
    ];

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function build(array $settings) {
    return [];
  }

  /**
   * Applies the filter via AJAX.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   Ajax redirect response.
   */
  public function applyFilterAjaxCallback(array &$form, FormStateInterface $form_state) {
    $route_name = $this->routeMatch()->getRouteName();
    $route_params = $this->routeMatch()->getParameters()->all();
    $form_values = $form_state->getValues();

    $filter_plugins = $this->filterPluginManager->getDefinitions();

    $query = [];
    $saved_active_filters = [];

    foreach ($filter_plugins as $type => $plugin) {
      $value = $form_state->getValue($type);
      if (!$value || $value == "none") {
        continue;
      }

      if ($value == "_all") {
        continue;
      }

      if (is_string($value)) {
        $query[$plugin['query_id']] = $value;
        $saved_active_filters[$plugin['id']] = $value;
      }
      else {
        $query[$plugin['query_id']] = implode(",", array_filter($value));
        $saved_active_filters[$plugin['id']] = implode(",", array_filter($value));
      }

    }

    // Grab all taxonomy vocabularies.
    $terms = [];
    foreach ($form_values as $id => $value) {
      if (!str_contains($id, "_term_value")) {
        continue;
      }

      if ($value == "_all") {
        continue;
      }

      if (is_string($value)) {
        $tid = $value;
        $value = [$tid];
      }

      $terms = array_merge($terms, $value);
    }
    $terms = array_filter($terms);

    $query = array_filter($query);
    if ($terms) {
      $query["term"] = implode(",", $terms);
      $saved_active_filters["taxonomy_vocabulary"] = implode(",", $terms);
    }

    if (!$value) {
      $redirect_url = Url::fromRoute(
        $route_name,
        $route_params,
      );
    }
    else {
      $redirect_url = Url::fromRoute(
        $route_name,
        $route_params,
        [
          "query" => $query,
        ]
      );
    }

    $tempstore = $this->tempStoreFactory->get('content_calendar');
    $tempstore->set('active_filters', $saved_active_filters);

    $redirect_command = new RedirectCommand($redirect_url->setAbsolute()->toString());
    $response = new AjaxResponse();
    return $response->addCommand($redirect_command);
  }

  /**
   * Reset the filter via AJAX.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   Ajax redirect response.
   */
  public function resetFilterAjaxCallback(array &$form, FormStateInterface $form_state) {
    $tempstore = $this->tempStoreFactory->get('content_calendar');
    $tempstore->set('active_filters', []);

    $redirect_url = Url::fromUri('base:' . $this->currentPathStack->getPath());
    $redirect_command = new RedirectCommand($redirect_url->setAbsolute()->toString());
    $response = new AjaxResponse();
    return $response->addCommand($redirect_command);
  }

}
