<?php

namespace Drupal\content_kanban\Component;

use Drupal\content_calendar\FilterHelper;
use Drupal\content_calendar\Plugin\CalendarFilterPluginManager;
use Drupal\content_kanban\KanbanService;
use Drupal\content_planner\ContentModerationService;
use Drupal\Core\Config\Config;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\workflows\Entity\Workflow;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * The main Kanban class.
 */
class Kanban {

  use StringTranslationTrait;

  /**
   * The request service.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The current user service.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The Kanban service.
   *
   * @var \Drupal\content_kanban\KanbanService
   */
  protected $kanbanService;

  /**
   * The Content Moderation service.
   *
   * @var \Drupal\content_planner\ContentModerationService
   */
  protected $contentModerationService;

  /**
   * The workflow service.
   *
   * @var \Drupal\workflows\Entity\Workflow
   */
  protected $workflow;

  /**
   * The config service.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * The workflow ID.
   *
   * @var string
   */
  protected $workflowID;

  /**
   * The type settings.
   *
   * @var array
   */
  protected $typeSettings = [];

  /**
   * The entity types.
   *
   * @var array
   */
  protected $entityTypes = [];

  /**
   * The states.
   *
   * @var array
   */
  protected $states = [];

  /**
   * The filter plugin manager.
   *
   * @var \Drupal\content_calendar\Plugin\CalendarFilterPluginManager
   */
  protected $filterPluginManager;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The form builder.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * Constructor for the Kanban class.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user service.
   * @param \Drupal\content_kanban\KanbanService $kanban_service
   *   The Kanban service.
   * @param \Drupal\content_planner\ContentModerationService $content_moderation_service
   *   The Content Moderation service.
   * @param \Drupal\workflows\Entity\Workflow $workflow
   *   The workflow service.
   * @param \Drupal\Core\Config\Config $config
   *   The config.
   * @param \Drupal\content_calendar\Plugin\CalendarFilterPluginManager $filter_plugin_manager
   *   The filter plugin manager.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   *
   * @throws \Exception
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, AccountInterface $current_user, KanbanService $kanban_service, ContentModerationService $content_moderation_service, Workflow $workflow, Config $config, CalendarFilterPluginManager $filter_plugin_manager, RequestStack $request_stack, FormBuilderInterface $form_builder) {

    if (!$content_moderation_service->isValidContentModerationWorkflow($workflow)) {
      throw new \Exception('The given workflow is no valid Content Moderation Workflow');
    }

    // Store request object.
    $this->request = \Drupal::request();

    $this->entityTypeManager = $entity_type_manager;

    // Store current user.
    $this->currentUser = $current_user;

    // Store Kanban service.
    $this->kanbanService = $kanban_service;

    // Store Content Moderation service.
    $this->contentModerationService = $content_moderation_service;

    // Store Workflow.
    $this->workflow = $workflow;

    // Store config.
    $this->config = $config;

    // Store calendar filter plugin.
    $this->filterPluginManager = $filter_plugin_manager;

    // Store request stack.
    $this->requestStack = $request_stack;

    // Store form builder.
    $this->formBuilder = $form_builder;

    // Store Workflow ID.
    $this->workflowID = $workflow->get('id');

    // Store Type settings.
    $this->typeSettings = $workflow->get('type_settings');
    // Store Entity types this workflow applies to.
    $this->entityTypes = $this->typeSettings['entity_types'];
    // Store states.
    $this->states = $this->sortStates($this->typeSettings['states']);
  }

  /**
   * Sorts the given states.
   *
   * @param array $states
   *   An array with the states to sort.
   *
   * @return array
   *   Returns the sorted array of states.
   */
  protected function sortStates(array $states) {

    // Make a copy of the states.
    $sorted_states = $states;

    // Add the state id, so it does not get lost during the custom sort
    // function.
    foreach ($sorted_states as $state_id => &$state) {
      $state['state_id'] = $state_id;
    }

    // Sort for weight.
    usort($sorted_states, function ($a, $b) {
      if ($a['weight'] == $b['weight']) {
        return 0;
      }
      elseif ($a['weight'] < $b['weight']) {
        return -1;
      }
      else {
        return 1;
      }
    });

    // Build a new return array.
    $return = [];

    foreach ($sorted_states as $sorted_state) {
      $return[$sorted_state['state_id']] = $sorted_state;
    }

    return $return;
  }

  /**
   * Build.
   *
   * @return array
   *   Returns a renderable array with the build of the Kanban board.
   */
  public function build($year = NULL) {

    $columns = [];
    // Get all Entity Type configs.
    $entityTypeConfigs = $this->kanbanService->getEntityTypeConfigs($this->entityTypes);

    foreach ($this->states as $state_id => $state) {

      // Prepare filter for the Kanban service.
      $filters = [
        'moderation_state' => $state_id,
      ];

      $filters = array_merge($filters, FilterHelper::getActiveFilterValues());

      // Get Entity IDs.
      $multipleEntities = [];
      if ($entityIds = $this->contentModerationService->getEntityIdsFromContentModerationEntities($this->workflowID, $filters, $this->entityTypes)) {
        $multipleEntities = $this->kanbanService->getEntitiesByEntityIds($entityIds, $filters, $this->entityTypes, $year);
      }

      $columnEntities = [];
      foreach ($multipleEntities as $entities) {
        $columnEntities = array_merge($columnEntities, $entities);
      }

      $max_number_of_entities = $this->config->get('maximum_number_of_items_per_state');
      if (!$max_number_of_entities) {
        $max_number_of_entities = 50;
      }

      $total_entities = count($columnEntities);
      $columnEntities = array_slice($columnEntities, 0, $max_number_of_entities);

      // Create Kanban object.
      $kanban_column = new KanbanColumn(
        $this->workflowID,
        $state_id,
        $state,
        $columnEntities,
        $entityTypeConfigs
      );

      $kanban_column = $kanban_column->build();

      $kanban_column['#total_entities'] = $total_entities;
      $kanban_column['#max_number_of_entities'] = $max_number_of_entities;

      // Build render array for Kanban.
      $columns[] = $kanban_column;
    }

    // Permissions.
    $permissions = [
      'create_entity' => $this->getCreateEntityPermissions($entityTypeConfigs),
    ];


    $next_year = date('Y', strtotime("+12 months $year-01-01"));
    $prev_year = date('Y', strtotime("-12 months $year-01-01"));

    // Build render array for Kanban.
    $build = [
      '#theme' => 'content_kanban',
      '#kanban_id' => $this->workflowID,
      '#kanban_label' => $this->workflow->label(),
      '#filter_form' => $this->buildFilterForm(),
      '#permissions' => $permissions,
      '#current_year' => $year,
      '#filter_slug' => FilterHelper::buildFilterQuerySlug(),
      '#prev_year_route_slug' => $prev_year,
      '#next_year_route_slug' => $next_year,
      '#headers' => $this->buildHeaders(),
      '#columns' => $columns,
      '#attached' => [
        'library' => ['content_planner/dashboard'],
      ],
    ];

    return $build;

  }

  /**
   * Builds headers for table.
   *
   * @return array
   *   Returns an array with the table headers.
   */
  protected function buildHeaders() {

    $headers = [];

    foreach ($this->states as $state) {
      $headers[] = $state['label'];
    }

    return $headers;
  }

  /**
   * Gets the list of permissions the current user may create a Entity type.
   *
   * @param \Drupal\content_kanban\EntityTypeConfig[] $entity_type_configs
   *   An array with the entity type configs.
   *
   * @return array
   *   Returns an array with the permissions.
   */
  protected function getCreateEntityPermissions(array $entity_type_configs) {

    $permissions = [];
    foreach ($entity_type_configs as $bundle => $entity_type_config) {
      // Check if the current user has the permisson to create a certain Entity
      // type.
      if ($this->entityTypeManager->getAccessControlHandler($entity_type_config->getEntityType())->createAccess($bundle)) {
        $permissions[$bundle] = $this->t("Add @type", ['@type' => $entity_type_config->getLabel()]);
      }

    }

    return $permissions;
  }

  /**
   * Builds the Filter form.
   *
   * @return array
   *   Returns an array with the filter form.
   */
  protected function buildFilterForm() {
    // If the user cannot edit any content, hide the Filter form.
    if (!$this->currentUser->hasPermission('manage any content with content kanban')) {
      return [];
    }

    $filter_form = $this->formBuilder->getForm('Drupal\content_kanban\Form\KanbanFilterForm');

    return $filter_form;
  }

}
