<?php

namespace Drupal\content_planner\Controller;

use Drupal\content_planner\DashboardSettingsService;
use Drupal\content_planner\DashboardTypePluginManager;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Routing\RouteMatchInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Implements DashboardController class.
 */
class DashboardController extends ControllerBase {

  /**
   * Route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * Dashboard plugin type manager.
   *
   * @var \Drupal\content_planner\DashboardTypePluginManager
   */
  protected $dashboardTypePluginManager;

  /**
   * Dashboard type settings.
   *
   * @var \Drupal\content_planner\DashboardSettingsService
   */
  protected $dashboardSettings;

  /**
   * Constructs a DashboardController object.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   * @param \Drupal\content_planner\DashboardTypePluginManager $dashboard_type_plugin_manager
   *   The dashboard type plugin manager.
   * @param \Drupal\content_planner\DashboardSettingsService $dashboard_settings
   *   The dashboard settings service.
   */
  public function __construct(
    RouteMatchInterface $route_match,
    DashboardTypePluginManager $dashboard_type_plugin_manager,
    DashboardSettingsService $dashboard_settings,
  ) {
    $this->routeMatch = $route_match;
    $this->dashboardTypePluginManager = $dashboard_type_plugin_manager;
    $this->dashboardSettings = $dashboard_settings;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_route_match'),
      $container->get('content_planner.dashboard_type_plugin_manager'),
      $container->get('content_planner.dashboard_settings_service'),
    );
  }

  /**
   * Show dashboard.
   *
   * @param string $dashboard_id
   *   Dashboard id.
   *
   * @return array
   *   The content planner dashboard render array.
   */
  public function showDashboard($dashboard_id = NULL): array {
    $build = [];

    // Grab the type out of the route.
    $route = $this->routeMatch->getRouteName();
    $route = explode(".", $route);
    $type = $route[1];

    // Build the plugin.
    /** @var \Drupal\content_planner\DashboardTypeInterface $dashboard_type */
    $dashboard_type = $this->dashboardTypePluginManager->createInstance($type);

    if (!$dashboard_id) {
      $dashboard_id = $dashboard_type->getBaseDashboardId();
    }

    // Show 404 if the dashboard is disabled.
    $settings = $this->dashboardSettings->getDashboardConfigurations($dashboard_id);
    if (!isset($settings['enabled']) || !$settings['enabled']) {
      throw new NotFoundHttpException();
    }

    $build['#title'] = $dashboard_type->getPageTitle($dashboard_id);
    $build['content'] = $dashboard_type->buildDashboardContent(
      $dashboard_id,
      $dashboard_type
    );

    return $build;
  }

}
