<?php

namespace Drupal\content_planner;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Session\AccountProxyInterface;

/**
 * Implements DashboardSettingsService class.
 */
class DashboardSettingsService {

  /**
   * Config name.
   */
  const CONFIG_NAME = 'content_planner.dashboard_settings';

  /**
   * Drupal\Core\Config\ConfigFactoryInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Dashboard type plugin manager.
   *
   * @var \Drupal\content_planner\DashboardTypePluginManager
   */
  protected $dashboardTypePluginManager;

  /**
   * Current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * Constructs a new DashboardSettingsService object.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    DashboardTypePluginManager $dashboard_type_plugin_manager,
    AccountProxyInterface $current_user,
  ) {
    $this->configFactory = $config_factory;
    $this->dashboardTypePluginManager = $dashboard_type_plugin_manager;
    $this->currentUser = $current_user;
  }

  /**
   * Get dashboard configurations.
   *
   * @param string $dashboard_id
   *   The dashboard id to retrieve the config for.
   *
   * @return array
   *   The dashboard configuration.
   */
  public function getDashboardConfigurations($dashboard_id = 'master') {
    $config = $this->configFactory->getEditable(
      'content_planner.dashboards.' . $dashboard_id
    );

    return $config->get('configuration') ?: [];
  }

  /**
   * Save dashboard configurations.
   *
   * @param string $dashboard_id
   *   The dashboard id to save the config for.
   * @param array $configurations
   *   The dashboard configurations to save.
   */
  public function saveDashboardConfigurations($dashboard_id, array $configurations) {
    $config = $this->configFactory->getEditable(
      'content_planner.dashboards.' . $dashboard_id
    );

    // Save configurations for the specified dashboard.
    $config->set('configuration', $configurations)->save();
  }

  /**
   * Delete dashboard configurations.
   *
   * @param string $dashboard_id
   *   The dashboard id to save the config for.
   */
  public function deleteDashboardConfigurations($dashboard_id) {
    $config = $this->configFactory->getEditable(
      'content_planner.dashboards.' . $dashboard_id
    );

    $config->delete();
  }

  /**
   * Gets the complete settings config of the module.
   *
   * @return \Drupal\Core\Config\ImmutableConfig
   *   The complete settings config of the module.
   */
  public function getSettings() {
    return $this->configFactory->get(self::CONFIG_NAME);
  }

  /**
   * Gets the widget configuration for a dashboard.
   *
   * @param string $dashboard_id
   *   The dashboard id.
   * @param string $uuid
   *   The widget uuid.
   *
   * @return array
   *   The plugin settings.
   */
  public function getWidgetConfigurations($dashboard_id, $uuid) {
    $settings = $this->getDashboardConfigurations($dashboard_id);
    if (!isset($settings['widgets'])) {
      return [];
    }

    $widgets = $settings['widgets'];
    if (!isset($widgets[$uuid])) {
      return [];
    }

    return $widgets[$uuid];
  }

  /**
   * Sets a widget configuration.
   *
   * @param string $dashboard_id
   *   The dashboard id.
   * @param string $uuid
   *   The widget uuid.
   * @param array $data
   *   The array of data.
   *
   * @return array
   *   The plugin settings.
   */
  public function setWidgetConfigurations($dashboard_id, $uuid, $data) {
    $config = $this->configFactory->getEditable(
      'content_planner.dashboards.' . $dashboard_id
    );

    $configuration = $config->get('configuration');
    $configuration['widgets'][$uuid] = $data;

    $config->set('configuration', $configuration)->save();

    return $configuration['widgets'][$uuid];
  }

  /**
   * Deletes a widget from a dashboard.
   *
   * @param string $dashboard_id
   *   The dashboard id.
   * @param string $uuid
   *   The widget uuid.
   *
   * @return array
   *   The plugin settings.
   */
  public function deleteWidget($dashboard_id, $uuid) {
    // Delete the config.
    $config = $this->configFactory->getEditable(
      'content_planner.dashboards.' . $dashboard_id
    );

    $configuration = $config->get('configuration');
    unset($configuration['widgets'][$uuid]);

    $config->set('configuration', $configuration)->save();

    return $configuration['widgets'][$uuid];
  }

  /**
   * Get Block configurations.
   *
   * @return array
   *   All block configurations.
   */
  public function getBlockConfigurations() {
    if ($settings = $this->getSettings()) {
      if ($block_configurations = $settings->get('blocks')) {
        return $block_configurations;
      }
    }

    return [];
  }

  /**
   * Get the configuration of a specific block.
   *
   * @param string $block_id
   *   The block id to retrieve the config for.
   *
   * @return array|mixed
   *   The block configuration.
   */
  public function getBlockConfiguration($block_id) {
    if ($block_configurations = $this->getBlockConfigurations()) {
      if (array_key_exists($block_id, $block_configurations)) {
        return $block_configurations[$block_id];
      }
    }

    return [];
  }

  /**
   * Save configuration of a specific block.
   *
   * @param string $block_id
   *   The block id to save the config for.
   * @param array $configuration
   *   The config to save.
   *
   * @return bool
   *   TRUE if config was found and saved, FALSE otherwise.
   */
  public function saveBlockConfiguration($block_id, array $configuration) {
    if ($block_configurations = $this->getBlockConfigurations()) {
      if (array_key_exists($block_id, $block_configurations)) {
        $block_configurations[$block_id] = $configuration;
        $this->saveBlockConfigurations($block_configurations);
        return TRUE;
      }
    }

    return FALSE;
  }

  /**
   * Save all block configurations.
   *
   * @param array $configuration
   *   The config to save.
   */
  public function saveBlockConfigurations(array $configuration) {
    $this->configFactory->getEditable(self::CONFIG_NAME)
      ->set('blocks', $configuration)
      ->save();
  }

  /**
   * Gets all available dashboards.
   *
   * @return array
   *   An associative array of all dashboard IDs.
   */
  public function getAllDashboards() {
    $allConfigNames = $this->configFactory->listAll('content_planner.dashboards.');
    $dashboards = [];

    foreach ($allConfigNames as $configName) {
      // Extract the dashboard ID from the config name.
      $dashboardId = str_replace('content_planner.dashboards.', '', $configName);

      // Optionally, load dashboard name or other properties here.
      $dashboardConfig = $this->configFactory->get($configName);

      // Store the dashboard ID and any other details in the result array.
      $dashboards[$dashboardId] = $dashboardConfig->get('configuration');
    }

    return $dashboards;
  }

  /**
   * Gets all available dashboards for a specific type.
   *
   * @return array
   *   An associative array of all dashboard IDs.
   */
  public function getDashboardsByType(string $type) {
    $all_config_names = $this->configFactory->listAll('content_planner.dashboards.');
    $dashboards = [];

    foreach ($all_config_names as $config_name) {
      // Extract the dashboard ID from the config name.
      $dashboard_id = str_replace('content_planner.dashboards.', '', $config_name);

      // Optionally, load dashboard name or other properties here.
      $dashboard_config = $this->configFactory->get($config_name);

      // Check if its enabled for the entity type.
      $dashboard_configuration = $dashboard_config->get('configuration');

      if (!isset($dashboard_configuration['types'])) {
        continue;
      }

      if (!in_array($type, $dashboard_configuration['types'])) {
        continue;
      }

      if (!isset($dashboard_configuration['enabled'])) {
        continue;
      }

      if (!$dashboard_configuration['enabled']) {
        continue;
      }

      // Check if current user has acess.
      $enabled_for_user_role = FALSE;
      foreach ($dashboard_configuration['roles'] as $rid) {
        if (in_array($rid, $this->currentUser->getRoles())) {
          $enabled_for_user_role = TRUE;
        }
      }

      // Always enable for super admins.
      if (in_array("administrator", $this->currentUser->getRoles()) || $this->currentUser->id() == 1) {
        $enabled_for_user_role = TRUE;
      }

      if (!$enabled_for_user_role) {
        continue;
      }

      // Store the dashboard ID and any other details in the result array.
      $dashboards[$dashboard_id] = $dashboard_configuration;
    }

    return $dashboards;
  }

  /**
   * Gets all available dashboards for a entity.
   *
   * @return array
   *   An associative array of all dashboard IDs.
   */
  public function getDashboardsByEntity(string $entity_type, int $id) {
    $all_config_names = $this->configFactory->listAll('content_planner.dashboards.');
    $dashboards = [];

    foreach ($all_config_names as $config_name) {
      // Extract the dashboard ID from the config name.
      $dashboard_id = str_replace('content_planner.dashboards.', '', $config_name);

      // Optionally, load dashboard name or other properties here.
      $dashboard_config = $this->configFactory->get($config_name);

      // Check if its enabled for the entity type.
      $dashboard_configuration = $dashboard_config->get('configuration');

      if (!isset($dashboard_configuration['enabled'])) {
        continue;
      }

      if (!$dashboard_configuration['enabled']) {
        continue;
      }

      if (!isset($dashboard_configuration['enabled_on_entity'])) {
        continue;
      }

      if (!isset($dashboard_configuration['enabled_on_entity'][$entity_type])) {
        continue;
      }

      if (!isset($dashboard_configuration['enabled_on_entity'][$entity_type][$id])) {
        continue;
      }

      if (!$dashboard_configuration['enabled_on_entity'][$entity_type][$id]) {
        continue;
      }

      // Check if current user has acess.
      $enabled_for_user_role = FALSE;
      foreach ($dashboard_configuration['roles'] as $rid) {
        if (in_array($rid, $this->currentUser->getRoles())) {
          $enabled_for_user_role = TRUE;
        }
      }

      // Always enable for super admins.
      if (in_array("administrator", $this->currentUser->getRoles()) || $this->currentUser->id() == 1) {
        $enabled_for_user_role = TRUE;
      }

      if (!$enabled_for_user_role) {
        continue;
      }

      // Store the dashboard ID and any other details in the result array.
      $dashboards[$dashboard_id] = $dashboard_configuration;
    }

    return $dashboards;
  }

  /**
   * Gets all available dashboard types.
   *
   * @return array
   *   An associative array of all dashboard types.
   */
  public function getDashboardTypes() {
    $types = [];

    foreach ($this->dashboardTypePluginManager->getDefinitions() as $id => $definition) {
      $plugin = $this->dashboardTypePluginManager->createInstance($id);

      $types[$plugin->getId()] = $plugin->getLabel();
    }

    return $types;
  }

  /**
   * Gets dashboard type instance.
   *
   * @return \Drupal\content_planner\DashboardTypeInterface
   *   Dashboard type.
   */
  public function getDashboardType($id) {
    $plugin = $this->dashboardTypePluginManager->createInstance($id);
    return $plugin;
  }

}
