<?php

namespace Drupal\content_planner;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\Config\Config;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Link;
use Drupal\Core\Menu\LocalTaskManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Routing\RedirectDestinationInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Routing\Route;

/**
 * Implements DashboardTypeBase class.
 */
class DashboardTypeBase extends PluginBase implements DashboardTypeInterface, ContainerFactoryPluginInterface {

  use StringTranslationTrait;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The dashboard block plugin manager.
   *
   * @var \Drupal\content_planner\DashboardBlockPluginManager
   */
  protected $dashboardBlockPluginManager;

  /**
   * Current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * Local task manager.
   *
   * @var \Drupal\Core\Menu\LocalTaskManagerInterface
   */
  protected $localTaskManager;

  /**
   * Route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * Destination service.
   *
   * @var \Drupal\Core\Routing\RedirectDestinationInterface
   */
  protected $destination;

  /**
   * Constructs a DashboardTypeBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\content_planner\DashboardBlockPluginManager $dashboard_block_plugin_manager
   *   The dashboard block plugin manager.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Menu\LocalTaskManagerInterface $local_task_manager
   *   The local task manager.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   * @param \Drupal\Core\Routing\RedirectDestinationInterface $destination
   *   The destination service.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    ConfigFactoryInterface $config_factory,
    DashboardBlockPluginManager $dashboard_block_plugin_manager,
    AccountProxyInterface $current_user,
    LocalTaskManagerInterface $local_task_manager,
    RouteMatchInterface $route_match,
    RedirectDestinationInterface $destination,
  ) {
    parent::__construct(
      $configuration,
      $plugin_id,
      $plugin_definition
    );
    $this->configFactory = $config_factory;
    $this->dashboardBlockPluginManager = $dashboard_block_plugin_manager;
    $this->currentUser = $current_user;
    $this->localTaskManager = $local_task_manager;
    $this->routeMatch = $route_match;
    $this->destination = $destination;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('content_planner.dashboard_block_plugin_manager'),
      $container->get('current_user'),
      $container->get('plugin.manager.menu.local_task'),
      $container->get('current_route_match'),
      $container->get('redirect.destination')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getLabel(): string {
    return $this->pluginDefinition['label'];
  }

  /**
   * {@inheritdoc}
   */
  public function getId(): string {
    return $this->pluginDefinition['id'];
  }

  /**
   * {@inheritdoc}
   */
  public function getCanonicalRouteName(): string {
    return $this->pluginDefinition['canonical_route'] ?? "";
  }

  /**
   * {@inheritdoc}
   */
  public function getBaseRouteName(): string {
    return $this->pluginDefinition['base_route'];
  }

  /**
   * {@inheritdoc}
   */
  public function getBaseRoute(): ?Route {
    return new Route(
      $this->getBaseRouteName(),
      [
        '_controller' => '\Drupal\content_planner\Controller\DashboardController::showDashboard',
        '_title' => 'Dashboard',
      ],
      [
        '_custom_access'  => 'Drupal\content_planner\Access\DashboardAccessCheck::checkAccess',
      ]
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getBaseRouteParameters(): array {
    return $this->pluginDefinition['base_route_parameters'] ?? [];
  }

  /**
   * {@inheritdoc}
   */
  public function getBaseDashboardId(): string {
    return $this->pluginDefinition['base_route_parameters'] ?? "master";
  }

  /**
   * {@inheritdoc}
   */
  public function isEntity(): bool {
    return isset($this->pluginDefinition['entity_type']) ? TRUE : FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function getEntityType(): ?string {
    return $this->pluginDefinition['entity_type'] ?: NULL;
  }

  /**
   * Get the local tasks to build on the dashboard pages.
   *
   * @return array
   *   Array of local tasks.
   */
  public function getLocalTasks(): array {
    $tasks = $this->localTaskManager->getLocalTasksForRoute(
      $this->getCanonicalRouteName()
    );

    $top_level_tasks = $tasks[0] ?? [];

    /** @var \Drupal\Core\Menu\LocalTaskDefault $task */
    $local_tasks = [];
    foreach ($top_level_tasks as $task) {
      $task_definition = $task->getPluginDefinition();
      $route_params = $task_definition['route_parameters'];

      // Add route params.
      foreach ($this->getBaseRouteParameters() as $parameter) {
        $route_params[$parameter] = $this->routeMatch->getParameter($parameter);
      }

      $local_tasks[$task->getPluginId()] = [
        "link" => [
          'title' => $task_definition['title'],
          'url' => Url::fromRoute(
            $task_definition['route_name'],
            $route_params,
          ),
        ],
        'weight' => $task_definition['weight'],
      ];
    }

    return $local_tasks;
  }

  /**
   * {@inheritdoc}
   */
  public function getPageTitle(string $dashboard_id): string {
    $dashboard_config = $this->getDashboardConfiguration(
      $dashboard_id
    );

    $dashboard_config = $dashboard_config->get('configuration');
    return $dashboard_config['name'] ?? "";
  }

  /**
   * {@inheritdoc}
   */
  public function buildDashboardContent(string $dashboard_id, DashboardTypeInterface $dashboard_type): array {
    $build = [];

    $dashboard_config = $this->getDashboardConfiguration($dashboard_id);

    if ($dashboard_id === 'master' && !$dashboard_config->get('configuration')) {
      $default_master_dashboard_config = [
        'name' => 'Master Dashboard',
        'widgets' => [],
        'roles' => ['authenticated'],
      ];
      $dashboard_config->set('configuration', $default_master_dashboard_config)->save();
    }

    // Load the updated or existing dashboard configuration.
    $dashboard_configuration = $dashboard_config->get('configuration');

    // Get registered Plugins.
    $plugins = $this->dashboardBlockPluginManager->getDefinitions();

    // Check if the current user has permission to configure dashboards.
    if ($this->currentUser->hasPermission('administer content planner dashboard settings')) {
      // Create a link to the dashboard settings form.
      $config_link = Url::fromRoute(
        'content_planner.dashboards.edit',
        [
          'dashboard_id' => $dashboard_id,
        ]
      );

      $config_link = [
        '#type' => 'link',
        '#title' => $this->t("Configure Dashboard"),
        '#url' => $this->ensureDestination($config_link),
        '#attributes' => [
          'class' => [
            'button',
            'button--secondary',
            'button--small',
          ],
        ],
      ];

      $build['config_link'] = $config_link;
    }

    if (!empty($dashboard_configuration['widgets']) && $dashboard_configuration['widgets']) {
      $block_builds = $this->buildBlocks(
        $dashboard_id,
        $dashboard_type,
        $dashboard_configuration,
        $plugins
      );

      $build['widgets'] = [
        '#theme' => 'content_planner_dashboard',
        '#full_width' => $dashboard_configuration['width'] ?? FALSE,
        '#blocks' => $block_builds,
        '#attached' => [
          'library' => ['content_planner/dashboard'],
        ],
      ];
    }

    return $build;
  }

  /**
   * Get config factory.
   *
   * @return \Drupal\Core\Config\ConfigFactoryInterface
   *   Config factory.
   */
  protected function getConfigFactory(): ConfigFactoryInterface {
    return $this->configFactory;
  }

  /**
   * Get config factory.
   *
   * @return \Drupal\Core\Routing\RouteMatchInterface
   *   Config factory.
   */
  protected function getRouteMatch(): RouteMatchInterface {
    return $this->routeMatch;
  }

  /**
   * Get the dashboard configuration.
   *
   * @param string $dashboard_id
   *   The dashboard id.
   *
   * @return \Drupal\Core\Config\Config
   *   Config object.
   */
  protected function getDashboardConfiguration($dashboard_id): Config {
    return $this->configFactory
      ->getEditable('content_planner.dashboards.' . $dashboard_id);
  }

  /**
   * Build dashboard blocks.
   *
   * @param string $dashboard_id
   *   The dashboard id.
   * @param \Drupal\content_planner\DashboardTypeInterface $dashboard_type
   *   The dashboard type.
   * @param array $dashboard_configuration
   *   The dashboard configuration.
   * @param array $plugins
   *   The dashboard plugins.
   *
   * @return array
   *   Array of content block render arrays.
   */
  protected function buildBlocks(
    string $dashboard_id,
    DashboardTypeInterface $dashboard_type,
    array &$dashboard_configuration,
    array &$plugins,
  ): array {
    $block_builds = [];

    foreach ($dashboard_configuration['widgets'] as $block) {
      if (array_key_exists($block['plugin_id'], $plugins)) {

        $route_params = [
          'dashboard_id' => $dashboard_id,
        ];

        foreach ($dashboard_type->getBaseRouteParameters() as $param) {
          $route_params[$param] = $this->routeMatch->getParameter($param);
        }

        $dashboard_destination = Url::fromRoute(
          'content_planner.' . $dashboard_type->getId() . '.dashboard',
          $route_params,
        );

        /** @var \Drupal\content_planner\DashboardBlockInterface $instance */
        try {
          $instance = $this->dashboardBlockPluginManager->createInstance($block['plugin_id'], $block);
          if ($this->currentUser->hasPermission('administer content planner dashboard settings')) {
            $has_permission = TRUE;
          }
          else {
            $has_permission = FALSE;
          }

          // Build block render array.
          if ($block_build = $instance->build()) {
            $block_builds[$block['uuid']] = [
              '#theme' => 'content_planner_dashboard_block',
              '#uuid' => $block['uuid'],
              '#dashboard_id' => $dashboard_id,
              '#destination_url' => $dashboard_destination->toString(),
              '#css_id' => str_replace('_', '-', $block['plugin_id']),
              '#block_id' => $block['plugin_id'],
              '#name' => (isset($block['title']) && $block['title']) ? $block['title'] : $instance->getName(),
              '#has_permission' => $has_permission,
              '#block' => $block_build,
              '#weight' => $block['weight'],
              '#colspan_class' => !empty($block['colspan']) ? $block['colspan'] : '',
            ];
          }
        }
        catch (\Exception $e) {
          $block_builds[$block['uuid']] = [
            '#theme' => 'content_planner_dashboard_block',
            '#uuid' => $block['uuid'],
            '#dashboard_id' => $dashboard_id,
            '#destination_url' => $dashboard_destination->toString(),
            '#css_id' => str_replace('_', '-', $block['plugin_id']),
            '#block_id' => $block['plugin_id'],
            '#name' => (isset($block['title']) && $block['title']) ? $block['title'] : $instance->getName(),
            '#has_permission' => $has_permission,
            '#block' => '(!) Widget cannot be rendered, because it not longer exist or the providing module is missing. Remove it in the dashboard configuration.',
            '#weight' => $block['weight'],
            '#colspan_class' => !empty($block['colspan']) ? $block['colspan'] : '',
          ];
        }
      }
    }

    return $block_builds;
  }

  /**
   * Ensures that a destination is present on the given URL.
   *
   * @param \Drupal\Core\Url $url
   *   The URL object to which the destination should be added.
   *
   * @return \Drupal\Core\Url
   *   The updated URL object.
   */
  protected function ensureDestination(Url $url) {
    return $url->mergeOptions([
      'query' => $this->destination->getAsArray(),
    ]);
  }

}
