<?php

namespace Drupal\content_planner;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\RedirectDestinationInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;

/**
 * Implements DashboardTypeLocalTaskManager class.
 */
class DashboardTypeLocalTaskManager {

  use StringTranslationTrait;

  /**
   * Drupal\Core\Config\ConfigFactoryInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Dashboard type plugin manager.
   *
   * @var \Drupal\content_planner\DashboardTypePluginManager
   */
  protected $dashboardTypePluginManager;

  /**
   * Current route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * Redirect destination.
   *
   * @var \Drupal\Core\Routing\RedirectDestinationInterface
   */
  protected $destination;

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * Constructs a new DashboardTypeLocalTaskManager object.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    DashboardTypePluginManager $dashboard_type_plugin_manager,
    RouteMatchInterface $route_match,
    RedirectDestinationInterface $destination,
    EntityTypeManagerInterface $entity_type_manager,
    AccountProxyInterface $current_user,
  ) {
    $this->configFactory = $config_factory;
    $this->dashboardTypePluginManager = $dashboard_type_plugin_manager;
    $this->routeMatch = $route_match;
    $this->destination = $destination;
    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public function buildMainLocalTasksForDashboardRoute(string $route_name): array {
    // Is dashboard route.
    $route = explode(".", $route_name);
    if (!isset($route[1])) {
      return [];
    }

    // Try to get the plugin.
    $type = $route[1];

    if (!$this->dashboardTypePluginManager->hasDefinition($type)) {
      return [];
    }

    /** @var \Drupal\content_planner\DashboardTypeInterface $type */
    $type = $this->dashboardTypePluginManager->createInstance($type);

    $local_tasks = [];
    foreach ($type->getLocalTasks() as $task) {
      $local_tasks[] = [
        '#theme' => 'menu_local_task',
        '#link' => $task['link'],
        '#weight' => $task['weight'],
      ];
    }

    return $local_tasks;
  }

  /**
   * {@inheritdoc}
   */
  public function buildSecondLocalTasksForDashboardRoute(string $route_name): array {
    // Get all canonical routes for all types.
    $dashboard_types_canonical_routes = [];

    foreach ($this->dashboardTypePluginManager->getDefinitions() as $id => $type) {
      if (isset($type['canonical_route'])) {
        $dashboard_types_canonical_routes[$id] = $type['canonical_route'];
      }
    }

    // Check if we are on canonical route, if so we need to render all
    // dashboards.
    $local_tasks = [];
    if (in_array($route_name, $dashboard_types_canonical_routes)) {
      $type_id = NULL;

      // Grab the type id for this route.
      foreach ($dashboard_types_canonical_routes as $id => $name) {
        if ($name == $route_name) {
          $type_id = $id;
        }
      }

      /** @var \Drupal\content_planner\DashboardTypeInterface $type */
      $type = $this->dashboardTypePluginManager->createInstance($type_id);
    }
    else {
      // Dashboard route.
      $route = explode(".", $route_name);
      if (!isset($route[1])) {
        return [];
      }

      // Try to get the plugin.
      $type_id = $route[1];
      if (!$this->dashboardTypePluginManager->hasDefinition($type_id)) {
        return [];
      }
      $type = $this->dashboardTypePluginManager->createInstance($type_id);
    }

    if ($route_name == 'content_planner.' . $type->getId() . '.dashboards.create') {
      return [];
    }

    if ($route_name == 'content_planner.' . $type->getId() . '.dashboards.list') {
      return [];
    }

    // Get all dashboards by this type.
    $type_dashboards = $this->getDashboardsByType($type_id);
    $local_task_route_params = [];
    foreach ($type->getBaseRouteParameters() as $param) {
      $param_value = $this->routeMatch->getParameter($param);
      if ($param_value instanceof EntityInterface) {
        $param_value = $param_value->id();
      }

      if (!$param_value) {
        continue;
      }

      $local_task_route_params[$param] = $param_value;
    }

    $local_tasks = [];

    // The first task is always a canonical route.
    if ($type->getCanonicalRouteName()) {
      /** @var \Drupal\content_planner\DashboardTypeInterface $type */
      $local_tasks["content_planner." . $type->getId() . ".dashboard"] = [
        '#theme' => 'menu_local_task',
        '#link' => [
          'title' => $type->getLabel(),
          'url' => Url::fromRoute(
            $type->getCanonicalRouteName(),
            $local_task_route_params,
          ),
        ],
        '#weight' => -999,
      ];
    }

    // Build tasks for all the dashboards of this type.
    foreach ($type_dashboards as $id => $dashboard_data_details) {
      // Check if this is an entity.
      if ($type->isEntity()) {
        $enabled_on_entity = isset($dashboard_data_details['enabled_on_entity']) ? $dashboard_data_details['enabled_on_entity'] : [];

        if ($enabled_on_entity && isset($enabled_on_entity[$type->getEntityType()])) {
          $entity = $this->routeMatch->getParameter($type->getEntityType());

          // Try to load if int.
          if (is_string($entity)) {
            $entity = $this->entityTypeManager
              ->getStorage($type->getEntityType())
              ->load($entity);
          }

          if ($entity && $entity instanceof EntityInterface && !isset($enabled_on_entity[$type->getEntityType()][$entity->id()])) {
            continue;
          }

          if ($entity && $entity instanceof EntityInterface && !$enabled_on_entity[$type->getEntityType()][$entity->id()]) {
            continue;
          }
        }
      }

      // Don't render if not enabled.
      if (!isset($dashboard_data_details['enabled']) || !$dashboard_data_details['enabled']) {
        continue;
      }

      // Check roles.
      $enabled_for_user_role = FALSE;
      foreach ($dashboard_data_details['roles'] as $rid) {
        if (in_array($rid, $this->currentUser->getRoles())) {
          $enabled_for_user_role = TRUE;
        }
      }

      // Always enable for super admins.
      if (in_array("administrator", $this->currentUser->getRoles()) || $this->currentUser->id() == 1) {
        $enabled_for_user_role = TRUE;
      }

      if (!$enabled_for_user_role) {
        continue;
      }

      // Add the id.
      $local_task_route_params['dashboard_id'] = $dashboard_data_details['id'];

      $local_tasks[$dashboard_data_details['id']] = [
        '#theme' => 'menu_local_task',
        '#link' => [
          'title' => $dashboard_data_details['name'],
          'url' => Url::fromRoute(
            "content_planner." . $type->getId() . ".dashboard",
            $local_task_route_params,
          ),
        ],
        '#weight' => $dashboard_data_details['weight'] ?? 0,
      ];
    }

    if ($local_tasks && $this->currentUser->hasPermission("administer content planner dashboard settings")) {
      // Add config task.
      $local_tasks['content_planner.dashboard_tabs_settings'] = [
        '#theme' => 'menu_local_task',
        '#link' => [
          'title' => $this->t("Configure Dashboards"),
          'url' => Url::fromRoute(
            'content_planner.' . $type->getId() . '.dashboards.list',
            $local_task_route_params,
            ['query' => $this->destination->getAsArray()]
          ),
        ],
        '#weight' => 999,
      ];
    }

    return $local_tasks;
  }

  /**
   * Get all dashboards of specific type.
   *
   * @param string $type
   *   Dashboard type.
   *
   * @return array
   *   Array of dashboards.
   */
  protected function getDashboardsByType($type): array {
    $type_dashboards = [];
    foreach ($this->configFactory->listAll('content_planner.dashboards.') as $dashboard_config_name) {
      $dashboard_config = $this->configFactory->get($dashboard_config_name);
      $dashboard_id = explode('.', $dashboard_config_name);
      $dashboard_id = end($dashboard_id);

      if (!$dashboard_config) {
        continue;
      }
      $dashboard_details = $dashboard_config->get('configuration');
      $dashboard_details['id'] = $dashboard_id;

      if (isset($dashboard_details['types']) && is_array($dashboard_details['types']) && in_array($type, $dashboard_details['types'])) {
        $type_dashboards[$dashboard_id] = $dashboard_details;
      }
    }

    return $type_dashboards;
  }

}
