<?php

namespace Drupal\content_planner\Form;

use Drupal\content_planner\DashboardSettingsService;
use Drupal\content_planner\DashboardTypePluginManager;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Create a new dashboard.
 */
class DashboardCreateForm extends FormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The dashboard settings service.
   *
   * @var \Drupal\content_planner\DashboardSettingsService
   */
  protected $dashboardSettingsService;

  /**
   * Dashboard type plugin manager.
   *
   * @var \Drupal\content_planner\DashboardTypePluginManager
   */
  protected $dashboardTypePluginManager;

  /**
   * DashboardCreateForm constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager.
   * @param \Drupal\content_planner\DashboardSettingsService $dashboard_settings_service
   *   Implements DashboardSettingsService class.
   * @param \Drupal\content_planner\DashboardTypePluginManager $dashboard_type_plugin_manager
   *   Implements dashboard type manager.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    DashboardSettingsService $dashboard_settings_service,
    DashboardTypePluginManager $dashboard_type_plugin_manager,
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->dashboardSettingsService = $dashboard_settings_service;
    $this->dashboardTypePluginManager = $dashboard_type_plugin_manager;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('content_planner.dashboard_settings_service'),
      $container->get('content_planner.dashboard_type_plugin_manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'content_planner_dashboard_create_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $type = $this->getRequest()->query->get('dashboard_type');
    $entity_type = $this->getRequest()->query->get('entity_type');
    $entity_id = $this->getRequest()->query->get('entity_id');

    $form['dashboard_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Dashboard Name'),
      '#required' => TRUE,
      '#description' => $this->t('Enter the name of the dashboard.'),
      '#maxlength' => 255,
    ];

    $form['dashboard_id'] = [
      '#type' => 'machine_name',
      '#title' => $this->t('Dashboard machine name'),
      '#maxlength' => 255,
      '#description' => $this->t('A unique machine-readable name for this dashboard. It must only contain lowercase letters, numbers, and underscores.'),
      '#machine_name' => [
        'exists' => [$this, 'dashboardExists'],
        'source' => ['dashboard_name'],
        'replace_pattern' => '[^a-z0-9_]+',
        'replace' => '_',
      ],
      '#required' => TRUE,
    ];

    $types = $this->dashboardSettingsService->getDashboardTypes();

    $form['enable_on'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Enable on'),
      '#options' => $types,
      '#required' => TRUE,
      '#description' => $this->t('Select where the dashboard will display.'),
    ];

    if ($type) {
      $form['enable_on']['#default_value'] = [$type];
      $form['enable_on']['#attributes']['class'] = ['hidden'];
    }

    // Render the types settings.
    $form['enable_on_entity'] = [
      '#type' => 'container',
      '#tree' => TRUE,
    ];

    foreach ($types as $id => $label) {
      $type = $this->dashboardSettingsService->getDashboardType($id);
      if ($type->isEntity()) {
        $entity_type = $type->getEntityType();

        $form['enable_on_entity'][$entity_type] = [
          '#type' => 'details',
          '#title' => $label,
          '#states' => [
            'visible' => [
              ":input[name='enable_on[" . $id . "]']" => ['checked' => TRUE],
            ],
          ],
        ];

        if ($entity_id) {
          $form['enable_on_entity'][$entity_type]['#attributes']['class'] = ['visually-hidden'];
        }

        // Get all entities.
        $entities = $this->entityTypeManager
          ->getStorage($type->getEntityType())
          ->loadMultiple();

        foreach ($entities as $entity) {
          $form['enable_on_entity'][$entity_type][$entity->id()] = [
            '#type' => 'checkbox',
            '#title' => $entity->label(),
          ];

          if ($entity_id == $entity->id()) {
            $form['enable_on_entity'][$entity_type][$entity->id()]['#default_value'] = TRUE;
          }
        }
      }
    }

    $role_options = [];
    $default_roles = [];
    $roles = $this->entityTypeManager->getStorage('user_role')->loadMultiple();
    foreach ($roles as $role_id => $role) {
      if ($role_id != 'anonymous') {
        $role_options[$role_id] = $role->label();
        $default_roles[] = $role_id;
      }
    }

    $form['roles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Roles'),
      '#options' => $role_options,
      '#default_value' => $default_roles,
      '#description' => $this->t('Select the roles that can view this dashboard.'),
    ];

    $form['advanced_settings']['enable'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#default_value' => TRUE,
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Create Dashboard'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * Check if the dashboard already exists.
   *
   * @param string $dashboard_id
   *   The dashboard machine name.
   *
   * @return bool
   *   TRUE if the dashboard exists, FALSE otherwise.
   */
  public function dashboardExists($dashboard_id) {
    $dashboard_configurations = $this->dashboardSettingsService
      ->getDashboardConfigurations($dashboard_id);
    return !empty($dashboard_configurations);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $dashboard_name = $form_state->getValue('dashboard_name');
    $dashboard_types = $form_state->getValue('enable_on');
    $dashboard_id = $form_state->getValue('dashboard_id');
    $selected_roles = array_filter($form_state->getValue('roles'));

    $dashboard_enabled = $form_state->getValue('enable');
    $dashboard_enabled_on_entity = $form_state->getValue(['enable_on_entity']);

    // Initialize default configurations for the new dashboard.
    $dashboard_configurations = [
      'id' => $dashboard_id,
      'name' => $dashboard_name,
      'types' => array_filter($dashboard_types),
      'roles' => array_keys($selected_roles),
      'enabled' => $dashboard_enabled,
      'enabled_on_entity' => $dashboard_enabled_on_entity,
    ];

    // Save the dashboard configurations using the DashboardSettingsService.
    $this->dashboardSettingsService->saveDashboardConfigurations(
      $dashboard_id,
      $dashboard_configurations
    );

    if ($type = $this->getRequest()->query->get('dashboard_type')) {
      $type = $this->dashboardTypePluginManager->createInstance($type);

      $route_params = [];
      foreach ($type->getBaseRouteParameters() as $param) {
        $param_value = $this->getRouteMatch()->getParameter($param);
        if ($param_value instanceof EntityInterface) {
          $param_value = $param_value->id();
        }

        if (!$param_value) {
          continue;
        }

        $route_params[$param] = $param_value;
      }
      $route_params['dashboard_id'] = $dashboard_id;

      $edit_url = Url::fromRoute(
        'content_planner.' . $type->getId() . '.dashboard',
        $route_params,
      );
      $form_state->setRedirectUrl($edit_url);
    }
    else {
      $edit_url = Url::fromRoute('content_planner.dashboards.list');
      $form_state->setRedirectUrl($edit_url);
    }

    // Optionally, add a confirmation message.
    $this->messenger()
      ->addMessage(
        $this->t(
          'Dashboard %name has been created.',
          [
            '%name' => $dashboard_name,
          ]
        )
      );

    drupal_flush_all_caches();
  }

}
