<?php

namespace Drupal\content_planner\Form;

use Drupal\content_planner\DashboardSettingsService;
use Drupal\content_planner\DashboardTypePluginManager;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a list builder for dashboards.
 */
class DashboardListBuilderForm extends FormBase {

  /**
   * The dashboard settings service.
   *
   * @var \Drupal\content_planner\DashboardSettingsService
   */
  protected $dashboardSettingsService;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The module_handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The manager to be used for instantiating plugins.
   *
   * @var \Drupal\content_planner\DashboardTypePluginManager
   */
  protected $dashboardTypePluginManager;

  /**
   * Constructs a new DashboardListBuilderForm .
   *
   * @param \Drupal\content_planner\DashboardSettingsService $dashboard_settings_service
   *   The dashboard list service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module_handler service.
   * @param \Drupal\content_planner\DashboardSettingsService $dashboard_type_plugin_manager
   *   The dashboard type plugin manager.
   */
  public function __construct(
    DashboardSettingsService $dashboard_settings_service,
    EntityTypeManagerInterface $entity_type_manager,
    RouteMatchInterface $route_match,
    ModuleHandlerInterface $module_handler,
    DashboardTypePluginManager $dashboard_type_plugin_manager,
  ) {
    $this->dashboardSettingsService = $dashboard_settings_service;
    $this->entityTypeManager = $entity_type_manager;
    $this->routeMatch = $route_match;
    $this->moduleHandler = $module_handler;
    $this->dashboardTypePluginManager = $dashboard_type_plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('content_planner.dashboard_settings_service'),
      $container->get('entity_type.manager'),
      $container->get('current_route_match'),
      $container->get('module_handler'),
      $container->get('content_planner.dashboard_type_plugin_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'content_planner_dashboard_list';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = [];

    // List builder.
    $header = [
      $this->t("Enabled"),
      $this->t("Dashboard Title"),
      $this->t("Dashboard ID"),
      $this->t("Enabled on"),
      $this->t("Roles"),
      $this->t("Operations"),
    ];

    $dashboards = $this->dashboardSettingsService->getAllDashboards();

    $form['number_of_dashboards'] = [
      '#type' => 'label',
      '#title' => $this->t("Number of dashboards: @number", ['@number' => count($dashboards)]),
      '#attributes' => [
        'class' => [
          'number-of-dashboards',
        ],
      ],
    ];

    $form['dashboards'] = [
      '#type' => 'table',
      '#header' => $header,
      '#empty' => $this->t('No dashboards available.'),
    ];

    // Sort dashboards by weight.
    uasort($dashboards, function ($a, $b) {
      if (!isset($a['weight'])) {
        $a['weight'] = 0;
      }

      if (!isset($b['weight'])) {
        $b['weight'] = 0;
      }

      return $a['weight'] <=> $b['weight'];
    });

    // Build table rows.
    foreach ($dashboards as $id => $dashboard) {
      if (!isset($dashboard['enabled'])) {
        $dashboard['enabled'] = 0;
      }

      $enabled_on_links = [];
      foreach ($dashboard['types'] as $type_id) {
        /** @var \Drupal\content_planner\DashboardTypeInterface $type_plugin */
        $type_plugin = $this->dashboardTypePluginManager->createInstance($type_id);

        if (!$type_plugin->isEntity()) {
          $enabled_on_links[] = "<a href='" . $type_plugin->getBaseRouteName() . "/" . $id . "'>" . $type_plugin->getLabel() . "</a>";
        }
      }

      if (isset($dashboard['enabled_on_entity'])) {
        foreach ($dashboard['enabled_on_entity'] as $entity_type => $entity_ids) {
          $entity_ids = array_filter($entity_ids);
          if ($entity_ids) {
            foreach ($entity_ids as $eid => $value) {
              $entity = $this->entityTypeManager->getStorage($entity_type)
                ->load($eid);

              $enabled_on_links[] = "<a href='" . $entity->toUrl()->toString() . "/dashboard/" . $id . "'>" . $entity->label() . "</a>";
            }
          }
        }
      }

      $form['dashboards'][$id] = [
        'enabled' => [
          '#markup' => $dashboard['enabled'] ? $this->t('Yes') : $this->t('No'),
        ],
        'name' => [
          '#markup' => $dashboard['name'],
        ],
        'id' => [
          '#markup' => $id,
        ],
        'enabled_on' => [
          '#markup' => implode(", ", $enabled_on_links),
        ],
        'roles' => [
          '#markup' => implode(", ", $dashboard['roles']),
        ],
        'operations' => [
          '#type' => 'operations',
          '#links' => $this->getDefaultOperations($id),
          '#attached' => [
            'library' => ['core/drupal.dialog.ajax'],
          ],
        ],
      ];
    }

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $dashboards = $form_state->getValue('dashboards');

    foreach ($dashboards as $id => $dashboard_data) {
      // Load the config of the dashboard.
      $config = $this->configFactory()
        ->getEditable('content_planner.dashboards.' . $id);

      if (!$config) {
        continue;
      }

      $configuration = $config->get('configuration');
      $configuration['weight'] = $dashboard_data['weight'];
      $config->set('configuration', $configuration);
      $config->save();

      $this->messenger()->addStatus("Configuration saved successfully.");
    }
  }

  /**
   * Gets this list's default operations.
   *
   * @param string $dashboard_id
   *   The dashboard id.
   *
   * @return array
   *   Array of operations.
   */
  protected function getDefaultOperations($dashboard_id) {
    $operations = [];

    $edit_url = Url::fromRoute(
      'content_planner.dashboards.edit',
      [
        'dashboard_id' => $dashboard_id,
      ]
    );

    $operations['edit'] = [
      'title' => $this->t('Edit'),
      'weight' => 1,
      'url' => $this->ensureDestination($edit_url),
    ];

    $delete_url = Url::fromRoute(
      'content_planner.dashboards.delete',
      [
        'dashboard_id' => $dashboard_id,
      ]
    );

    $operations['delete'] = [
      'title' => $this->t('Delete'),
      'weight' => 10,
      'url' => $this->ensureDestination($delete_url),
    ];

    return $operations;
  }

  /**
   * Ensures that a destination is present on the given URL.
   *
   * @param \Drupal\Core\Url $url
   *   The URL object to which the destination should be added.
   *
   * @return \Drupal\Core\Url
   *   The updated URL object.
   */
  protected function ensureDestination(Url $url) {
    return $url->mergeOptions([
      'query' => $this->getRedirectDestination()->getAsArray(),
    ]);
  }

}
