<?php

namespace Drupal\content_planner\Form;

use Drupal\content_planner\DashboardSettingsService;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\Role;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Defines a form that configures dashboard settings.
 */
class DashboardSettingsForm extends ConfigFormBase {

  /**
   * The dashboard settings service.
   *
   * @var \Drupal\content_planner\DashboardSettingsService
   */
  protected $dashboardSettingsService;

  /**
   * The dashboard block plugin manager.
   *
   * @var \Drupal\content_planner\DashboardBlockPluginManager
   */
  protected $dashboardBlockPluginManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->dashboardSettingsService = $container->get('content_planner.dashboard_settings_service');
    $instance->dashboardBlockPluginManager = $container->get('content_planner.dashboard_block_plugin_manager');
    $instance->entityTypeManager = $container->get('entity_type.manager');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'content_planner_dashboard_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      DashboardSettingsService::CONFIG_NAME,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, Request $request = NULL, $dashboard_id = 'master') {
    $form['actions'] = [
      '#type' => 'actions',
      '#weight' => -10,
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save dashboard configuration'),
      '#button_type' => 'primary',
    ];

    $dashboard_configurations = $this->dashboardSettingsService
      ->getDashboardConfigurations($dashboard_id);

    $role_options = [];
    $roles = Role::loadMultiple();
    foreach ($roles as $role_id => $role) {
      if ($role_id != 'anonymous') {
        $role_options[$role_id] = $role->label();
      }
    }

    // Add a fieldset for the dashboard settings.
    $form['advanced_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Settings'),
      '#tree' => TRUE,
    ];

    $form['advanced_settings']['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Dashboard Title'),
      '#default_value' => $dashboard_configurations['name'] ?: "",
      '#required' => TRUE,
    ];

    $types = $this->dashboardSettingsService->getDashboardTypes();
    $form['advanced_settings']['enable_on'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Enable on'),
      '#options' => $types,
      '#default_value' => $dashboard_configurations['types'] ?: [],
      '#required' => TRUE,
      '#description' => $this->t('Select where the dashboard will display.'),
    ];

    // Render the types settings.
    $form['advanced_settings']['enable_on_entity'] = [
      '#type' => 'container',
    ];

    $enabled_on_entity_settings = $dashboard_configurations['enabled_on_entity'] ?? [];
    foreach ($types as $id => $label) {
      $type = $this->dashboardSettingsService->getDashboardType($id);
      if ($type->isEntity()) {
        $entity_type = $type->getEntityType();

        $form['advanced_settings']['enable_on_entity'][$entity_type] = [
          '#type' => 'details',
          '#title' => $label,
          '#tree' => TRUE,
          '#states' => [
            'visible' => [
              ":input[name='advanced_settings[enable_on][" . $id . "]']" => ['checked' => TRUE],
            ],
          ],
        ];

        // Get all entities.
        $entities = $this->entityTypeManager
          ->getStorage($type->getEntityType())
          ->loadMultiple();

        foreach ($entities as $entity) {
          $form['advanced_settings']['enable_on_entity'][$entity_type][$entity->id()] = [
            '#type' => 'checkbox',
            '#title' => $entity->label(),
            '#default_value' => $enabled_on_entity_settings[$entity_type][$entity->id()] ?? FALSE,
          ];
        }
      }
    }

    $form['advanced_settings']['roles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Roles'),
      '#options' => $role_options,
      '#default_value' => $dashboard_configurations['roles'] ?: [],
      '#description' => $this->t('Select the roles that can view this dashboard.'),
    ];

    $form['advanced_settings']['enable'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#default_value' => $dashboard_configurations['enabled'] ?? FALSE,
    ];

    $form['advanced_settings']['width'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show dashboard in full width.'),
      '#default_value' => $dashboard_configurations['width'] ?? FALSE,
    ];

    $form['advanced_settings']['dashboard_id'] = [
      '#type' => 'hidden',
      '#title' => $this->t('Dashboard ID'),
      '#default_value' => $dashboard_id,
      '#required' => TRUE,
    ];

    if (!empty($dashboard_configurations['widgets']) && $dashboard_configurations['widgets'] != NULL) {
      $active_widgets = json_encode(
        $dashboard_configurations['widgets'],
        JSON_FORCE_OBJECT,
      );
    }
    else {
      $active_widgets = [];
    }

    $form['widgets'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Widgets configuration'),
      '#tree' => TRUE,
    ];

    $form['active_widgets'] = [
      '#type' => 'textarea',
      '#id' => 'active-widgets-field',
      '#default_value' => $active_widgets,
    ];

    $categorized_plugins = $this->getCategorizedPlugins();

    $widgets_data = [];
    foreach ($categorized_plugins as $category => $plugins) {
      foreach ($plugins as $plugin_id => $plugin_name) {
        $widgets_data[$category][$plugin_id] = [
          'plugin_id' => $plugin_id,
          'title' => $plugin_name,
        ];
      }
    }

    // Prepare the data for JS.
    $settings = [
      'dashboard' => [
        'widgets' => $widgets_data,
      ],
    ];
    $form['#attached']['drupalSettings'] = $settings;

    $form['#attached']['library'][] = 'content_planner/content_planner_react_app';
    $form['widgets']['react_app'] = [
      '#type' => 'markup',
      '#markup' => '<div id="root"></div>',
    ];

    $form['#attached']['library'][] = 'content_planner/dashboard';

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Retrieve the whole submitted values.
    $values = $form_state->getValues();

    // Access advanced settings directly from the nested structure.
    $dashboard_id = $values['advanced_settings']['dashboard_id'];
    $dashboard_name = $values['advanced_settings']['name'];
    $dashboard_width = $values['advanced_settings']['width'];
    $dashboard_enabled = $values['advanced_settings']['enable'];
    $dashboard_enabled_on_entity = isset($values['advanced_settings']['enable_on_entity']) ? $values['advanced_settings']['enable_on_entity'] : [];

    // Fetch the configurations for the selected dashboard.
    $dashboard_configurations['id'] = $dashboard_id;
    $dashboard_configurations['name'] = $dashboard_name;
    $dashboard_configurations['width'] = $dashboard_width;
    $dashboard_configurations['enabled'] = $dashboard_enabled;
    $dashboard_configurations['enabled_on_entity'] = $dashboard_enabled_on_entity;

    // Process roles.
    $selected_roles = array_filter($values['advanced_settings']['roles']);
    $selected_types = array_filter($values['advanced_settings']['enable_on']);

    // Since roles are checkboxes, use array_keys to get selected roles.
    $dashboard_configurations['roles'] = array_keys($selected_roles);
    $dashboard_configurations['types'] = array_keys($selected_types);

    // Save configurations for the selected dashboard.
    $this->dashboardSettingsService->saveDashboardConfigurations(
      $dashboard_id,
      $dashboard_configurations
    );

    // Widgets configurations are stored in a different part of the form.
    // Assuming 'active_widgets' is directly under $form and not nested within 'widgets'.
    $activeWidgetsJson = $values['active_widgets'];

    // Decode the JSON string to an associative array.
    $activeWidgets = json_decode($activeWidgetsJson, TRUE);
    if ($activeWidgets) {
      foreach ($activeWidgets as $widget_config) {
        $widget_config['dashboard_id'] = $dashboard_id;
        $this->dashboardSettingsService->setWidgetConfigurations(
          $dashboard_id,
          $widget_config['uuid'],
          $widget_config
        );
      }
    }

    // Display a message and redirect.
    $this->messenger()->addMessage(
      $this->t('Dashboard %dashboard has been updated.', ['%dashboard' => $dashboard_name])
    );

    $form_state->setRedirect('content_planner.content_planner.dashboard', ['dashboard_id' => $dashboard_id]);
    drupal_flush_all_caches();
  }

  /**
   * Get active widgets for a dashboard.
   *
   * @param string $dashboard_id
   *   Id of the dashboard.
   *
   * @return array
   *   Active widgets.
   */
  protected function getActiveWidgets($dashboard_id) {
    $blocks = $this->dashboardSettingsService->getDashboardConfigurations($dashboard_id);
    $plugins = $this->dashboardBlockPluginManager->getDefinitions();

    $active_widgets = [];

    foreach ($blocks as $block_id => $block) {
      if (array_key_exists($block_id, $plugins)) {
        $active_widgets[] = [
          'id' => $block_id,
          'title' => (isset($block['title']) && $block['title']) ? $block['title'] : $plugins[$block_id]['name'],
          'plugin_name' => $plugins[$block_id]['name'],
          'weight' => $block['weight'],
          'colspan' => !empty($block['colspan']) ? $block['colspan'] : '',
        ];
      }
    }

    usort($active_widgets, function ($a, $b) {
      return $a['weight'] - $b['weight'];
    });

    return $active_widgets;
  }

  /**
   * Get categorized plugins.
   *
   * @return array
   */
  protected function getCategorizedPlugins() {
    // Get registered Plugins.
    $plugins = $this->dashboardBlockPluginManager->getDefinitions();
    uksort($plugins, function ($a, $b) {
      return strnatcmp($a, $b);
    });

    // Initialize categories array
    $categorized_plugins = [];

    foreach ($plugins as $plugin_id => $plugin) {
      // Get the category from the plugin definition
      if (isset($plugin['category'])) {
        $id_category = $plugin['category'];
      } else {
        $id_category = "other";
      }

      if (!isset($categorized_plugins[$id_category])) {
        $categorized_plugins[$id_category] = [];
      }

      $categorized_plugins[$id_category][$plugin_id] = $plugin['name'];
    }

    return $categorized_plugins;
  }

}
