<?php

namespace Drupal\content_planner\Form;

use Drupal\content_planner\DashboardSettingsService;
use Drupal\content_planner\DashboardTypeInterface;
use Drupal\content_planner\DashboardTypePluginManager;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a list builder for a specific dashboard type.
 */
class DashboardTypeListBuilderForm extends FormBase {

  /**
   * The dashboard settings service.
   *
   * @var \Drupal\content_planner\DashboardSettingsService
   */
  protected $dashboardSettingsService;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The module_handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The manager to be used for instantiating plugins.
   *
   * @var \Drupal\content_planner\DashboardTypePluginManager
   */
  protected $dashboardTypePluginManager;

  /**
   * Constructs a new DashboardListBuilderForm .
   *
   * @param \Drupal\content_planner\DashboardSettingsService $dashboard_settings_service
   *   The dashboard list service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module_handler service.
   * @param \Drupal\content_planner\DashboardSettingsService $dashboard_type_plugin_manager
   *   The dashboard type plugin manager.
   */
  public function __construct(
    DashboardSettingsService $dashboard_settings_service,
    EntityTypeManagerInterface $entity_type_manager,
    RouteMatchInterface $route_match,
    ModuleHandlerInterface $module_handler,
    DashboardTypePluginManager $dashboard_type_plugin_manager,
  ) {
    $this->dashboardSettingsService = $dashboard_settings_service;
    $this->entityTypeManager = $entity_type_manager;
    $this->routeMatch = $route_match;
    $this->moduleHandler = $module_handler;
    $this->dashboardTypePluginManager = $dashboard_type_plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('content_planner.dashboard_settings_service'),
      $container->get('entity_type.manager'),
      $container->get('current_route_match'),
      $container->get('module_handler'),
      $container->get('content_planner.dashboard_type_plugin_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'content_planner_dashboard_type_list';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = [];

    $dashboard_type_id = $this->routeMatch
      ->getRouteObject()
      ->getDefault('_dashboard_type');

    if (!$dashboard_type_id) {
      return $form;
    }

    /** @var \Drupal\content_planner\DashboardTypeInterface $dashboard_type */
    $dashboard_type = $this->dashboardTypePluginManager
      ->createInstance($dashboard_type_id);

    $route_params = [];
    foreach ($dashboard_type->getBaseRouteParameters() as $param) {
      $param_value = $this->getRouteMatch()->getParameter($param);
      if ($param_value instanceof EntityInterface) {
        $param_value = $param_value->id();
      }

      if (!$param_value) {
        continue;
      }

      $route_params[$param] = $param_value;
    }

    $create_link = Url::fromRoute(
      'content_planner.' . $dashboard_type_id . '.dashboards.create',
      $route_params,
    );

    $create_link_query = $create_link->getOption('query');

    if ($dashboard_type->isEntity()) {
      $dashboard_type_entity = $dashboard_type->getEntityType();
      $entity_id = $this->routeMatch->getParameter($dashboard_type_entity);
      if ($dashboard_type_entity && $entity_id) {
        $dashboards = $this->dashboardSettingsService->getDashboardsByEntity(
          $dashboard_type_entity,
          $entity_id
        );
      }

      $create_link_query = $create_link->getOption('query');
      $create_link_query['dashboard_type'] = $dashboard_type_id;
      $create_link_query['entity_type'] = $entity_id;
      $create_link_query['entity_id'] = $entity_id;
    }
    else {
      $dashboards = $this->dashboardSettingsService->getDashboardsByType(
        $dashboard_type_id
      );

      $create_link = $this->ensureDestination(
        Url::fromRoute('content_planner.dashboards.create')
      );

      $create_link_query['dashboard_type'] = $dashboard_type_id;
    }

    $create_link->setOption('query', $create_link_query);
    $form['create'] = [
      '#type' => 'link',
      '#prefix' => "<div class='create-dashboard-btn'>",
      '#suffix' => "</div>",
      '#title' => $this->t('Create dashboard'),
      '#url' => $create_link,
      '#attributes' => [
        'class' => [
          'button',
          'button--action',
          'button--primary',
        ],
      ],
    ];

    // List builder.
    $header = [
      $this->t("Enabled"),
      $this->t("Dashboard Title"),
      $this->t("Dashboard ID"),
      $this->t("Enabled on"),
      $this->t("Roles"),
      $this->t("Weight"),
      $this->t("Operations"),
    ];

    $dashboards = [];

    if ($dashboard_type->isEntity()) {
      $dashboard_type_entity = $dashboard_type->getEntityType();
      $entity_id = $this->routeMatch->getParameter($dashboard_type_entity);
      if ($dashboard_type_entity && $entity_id) {
        $dashboards = $this->dashboardSettingsService->getDashboardsByEntity(
          $dashboard_type_entity,
          $entity_id
        );
      }
    }
    else {
      $dashboards = $this->dashboardSettingsService->getDashboardsByType(
        $dashboard_type_id
      );
    }

    $form['number_of_dashboards'] = [
      '#type' => 'label',
      '#title' => $this->t("Number of dashboards: @number", ['@number' => count($dashboards)]),
      '#attributes' => [
        'class' => [
          'number-of-dashboards',
        ],
      ],
    ];

    $form['dashboards'] = [
      '#type' => 'table',
      '#header' => $header,
      '#empty' => $this->t('No dashboards available.'),
      '#tabledrag' => [
        [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => 'dashboards-order-weight',
        ],
      ],
    ];

    // Sort dashboards by weight.
    uasort($dashboards, function ($a, $b) {
      if (!isset($a['weight'])) {
        $a['weight'] = 0;
      }

      if (!isset($b['weight'])) {
        $b['weight'] = 0;
      }

      return $a['weight'] <=> $b['weight'];
    });

    // Build table rows.
    foreach ($dashboards as $id => $dashboard) {
      $weight = $dashboard['weight'] ?? 0;

      if (!isset($dashboard['enabled'])) {
        $dashboard['enabled'] = 0;
      }

      $form['dashboards'][$id] = [
        'enabled' => [
          '#markup' => $dashboard['enabled'] ? $this->t('Yes') : $this->t('No'),
        ],
        'name' => [
          '#markup' => $dashboard['name'],
        ],
        'id' => [
          '#markup' => $id,
        ],
        'enabled_on' => [
          '#markup' => is_array($dashboard['types']) ? implode(", ", $dashboard['types']) : '',
        ],
        'roles' => [
          '#markup' => implode(", ", $dashboard['roles']),
        ],
        'weight' => [
          '#type' => 'weight',
          '#title' => $this->t('Weight for @title', ['@title' => $id]),
          '#title_display' => 'invisible',
          '#default_value' => $weight,
          '#attributes' => ['class' => ['dashboards-order-weight']],
        ],
        'operations' => [
          '#type' => 'operations',
          '#links' => $this->getDefaultOperations($id, $dashboard_type),
          '#attached' => [
            'library' => ['core/drupal.dialog.ajax'],
          ],
        ],
      ];

      $form['dashboards'][$id]['#attributes']['class'][] = 'draggable';
    }

    $form['#attached']['library'][] = "content_planner/dashboard-type-list-builder";
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $dashboards = $form_state->getValue('dashboards');

    foreach ($dashboards as $id => $dashboard_data) {
      // Load the config of the dashboard.
      $config = $this->configFactory()
        ->getEditable('content_planner.dashboards.' . $id);

      if (!$config) {
        continue;
      }

      $configuration = $config->get('configuration');
      $configuration['weight'] = $dashboard_data['weight'];
      $config->set('configuration', $configuration);
      $config->save();

      $this->messenger()->addStatus("Configuration saved successfully.");
      drupal_flush_all_caches();
    }
  }

  /**
   * Gets this list's default operations.
   *
   * @param string $dashboard_id
   *   The dashboard id.
   * @param Drupal\content\_planner\DashboardTypeInterface $type
   *   The dashboard type.
   *
   * @return array
   *   Array of operations.
   */
  protected function getDefaultOperations($dashboard_id, DashboardTypeInterface $type) {
    $operations = [];

    $edit_url = Url::fromRoute(
      'content_planner.dashboards.edit',
      [
        'dashboard_id' => $dashboard_id,
      ]
    );

    $operations['edit'] = [
      'title' => $this->t('Edit'),
      'weight' => 1,
      'url' => $this->ensureDestination($edit_url),
    ];

    // Get route params.
    $route_params = [];
    foreach ($type->getBaseRouteParameters() as $param) {
      $param_value = $this->getRouteMatch()->getParameter($param);
      if ($param_value instanceof EntityInterface) {
        $param_value = $param_value->id();
      }

      if (!$param_value) {
        continue;
      }

      $route_params[$param] = $param_value;
    }

    $dashboard_list_url = Url::fromRoute(
      'content_planner.' . $type->getId() . '.dashboards.list',
      $route_params,
    );

    $delete_url = Url::fromRoute(
      'content_planner.dashboards.delete',
      [
        'dashboard_id' => $dashboard_id,
      ],
      [
        'query' => [
          'destination' => $dashboard_list_url->toString(),
        ],
      ]
    );

    $operations['delete'] = [
      'title' => $this->t('Delete'),
      'weight' => 10,
      'url' => $delete_url,
    ];

    return $operations;
  }

  /**
   * Ensures that a destination is present on the given URL.
   *
   * @param \Drupal\Core\Url $url
   *   The URL object to which the destination should be added.
   *
   * @return \Drupal\Core\Url
   *   The updated URL object.
   */
  protected function ensureDestination(Url $url) {
    return $url->mergeOptions([
      'query' => $this->getRedirectDestination()->getAsArray(),
    ]);
  }

}
