<?php

namespace Drupal\content_planner\Form;

use Drupal\content_planner\DashboardSettingsService;
use Drupal\Core\Form\ConfigFormBase;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a form that configures dashboard widget settings.
 */
class DashboardWidgetConfigForm extends ConfigFormBase {

  /**
   * The dashboard settings service.
   *
   * @var \Drupal\content_planner\DashboardSettingsService
   */
  protected $dashboardSettingsService;

  /**
   * The Dashboard block plugin manager.
   *
   * @var \Drupal\content_planner\DashboardBlockPluginManager
   */
  protected $dashboardBlockPluginManager;

  /**
   * The widget instance.
   *
   * @var \Drupal\content_planner\DashboardBlockInterface
   */
  protected $widget;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->dashboardSettingsService = $container->get('content_planner.dashboard_settings_service');
    $instance->dashboardBlockPluginManager = $container->get('content_planner.dashboard_block_plugin_manager');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'content_planner_dashboard_widget_config_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      DashboardSettingsService::CONFIG_NAME,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, Request $request = NULL, $dashboard_id = NULL, $uuid = NULL) {
    $widget_config = $this->dashboardSettingsService->getWidgetConfigurations(
      $dashboard_id,
      $uuid,
    );

    /** @var \Drupal\content_planner\DashboardBlockInterface $instance */
    $widget_instance = $this->dashboardBlockPluginManager
      ->createInstance($widget_config['plugin_id'], $widget_config);

    $this->widget = $widget_instance;

    $form['title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Title'),
      '#maxlength' => 255,
      '#default_value' => (!empty($widget_config['title'])) ? $widget_config['title'] : ''
    ];



    $form['weight'] = [
      '#type' => 'weight',
      '#title' => $this->t('Weight'),
      '#default_value' => (isset($widget_config['weight'])) ? $widget_config['weight'] : 0,
      '#delta' => 10,
    ];

    $form['colspan'] = [
      '#type' => 'select',
      '#title' => $this->t('Width'),
      '#default_value' => (isset($widget_config['colspan'])) ? $widget_config['colspan'] : 'colspan-2',
      '#options' => [
        'colspan-1' => $this->t('1 Column'),
        'colspan-2' => $this->t('2 Columns'),
        'colspan-3' => $this->t('3 Columns'),
        'colspan-4' => $this->t('4 Columns'),
        'colspan-5' => $this->t('5 Columns'),
        'colspan-6' => $this->t('6 Columns'),
      ],
    ];

    $form['dashboard_id'] = [
      '#type' => 'hidden',
      '#value' => $dashboard_id,
    ];

    $form['widget_uuid'] = [
      '#type' => 'hidden',
      '#value' => $uuid,
    ];

    // Load plugin specific config.
    $widget_settings = $this->widget
      ->getConfigSpecificFormFields($form_state, $request, $widget_config);

    $form['plugin_specific_config'] = [
      '#type' => 'fieldset',
      '#tree' => TRUE,
    ];

    foreach ($widget_settings as $key => $setting) {
      $form['plugin_specific_config'][$key] = $setting;
    }

    $form = parent::buildForm($form, $form_state);

    $form['actions']['delete'] = [
      '#type' => 'submit',
      '#value' => $this->t('Delete'),
      '#submit' => [[get_class($this), "deleteWidget"]],
      '#attributes' => [
        'class' => [
          "action-link",
          "action-link--danger",
          "action-link--icon-trash",
        ],
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $widget_configuration = [];
    $widget_configuration['plugin_id'] = $this->widget->getId();
    $widget_configuration['widget_name'] = $this->widget->getName();
    $widget_configuration['dashboard_id'] = $this->widget->getDashboardId();
    $widget_configuration['uuid'] = $this->widget->getUuid();
    $widget_configuration['title'] = $form_state->getValue('title');
    $widget_configuration['weight'] = $form_state->getValue('weight');
    $widget_configuration['colspan'] = $form_state->getValue('colspan');
    $widget_configuration['configured'] = TRUE;
    $widget_configuration['plugin_specific_config'] = $form_state->getValue('plugin_specific_config');

    $this->dashboardSettingsService->setWidgetConfigurations(
      $this->widget->getDashboardId(),
      $this->widget->getUuid(),
      $widget_configuration
    );
  }

  /**
   * Submit handler for deleting a widget.
   *
   * @param array $form
   *   Form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Current form state.
   */
  public static function deleteWidget(&$form, FormStateInterface $form_state) {
    $dashboard_settings_service = \Drupal::service('content_planner.dashboard_settings_service');
    $dashboard_settings_service->deleteWidget(
      $form_state->getValue('dashboard_id'),
      $form_state->getValue('widget_uuid'),
    );
  }

}
