<?php

namespace Drupal\content_planner\Plugin\DashboardBlock;

use Drupal\content_planner\DashboardBlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Provides a 'ContentTypeListBlock' block.
 *
 * @DashboardBlock(
 *   id = "content_type_list_block",
 *   name = @Translation("Create Content"),
 *   category = "Content Management",
 *   id_category = "content_management"
 * )
 */
class ContentTypeListBlock extends DashboardBlockBase implements ContainerFactoryPluginInterface
{

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function build()
  {
    $build = [];
    $config = $this->getConfiguration();

    // Get selected content types from configuration or default to all if none selected.
    $enabled_content_types = !empty($config['plugin_specific_config']['enabled_content_types']) ? array_filter($config['plugin_specific_config']['enabled_content_types']) : [];

    // Load content types based on configuration.
    $content_types = \Drupal\node\Entity\NodeType::loadMultiple($enabled_content_types);

    // If no content types are specifically enabled, load all.
    if (empty($enabled_content_types)) {
      $content_types = \Drupal\node\Entity\NodeType::loadMultiple();
    }

    $build[] = ['#markup' => '<div class="content-type-list-block-wrapper">',];

    $current_path = \Drupal::service('path.current')->getPath();

    // Loop through each content type.
    foreach ($content_types as $content_type) {
      // Get the node count for this content type.
      $node_count = \Drupal::entityQuery('node')
        ->condition('type', $content_type->id())
        ->accessCheck(FALSE) // Add this line
        ->count()
        ->execute();

      // Create an "Add" link for this content type.
      $url = Url::fromRoute('node.add', ['node_type' => $content_type->id()], ['query' => ['destination' => $current_path]]);



      $add_link_renderable = [
        '#type' => 'link',
        '#title' => $this->t('Create'),
        '#url' => $url,
        '#attributes' => [
          'class' => [],
        ],
      ];

      // Convert the render array to a string.
      $renderer = \Drupal::service('renderer');
      $add_link = $renderer->render($add_link_renderable);



      // Add the content type information to the block's content.
      $build[] = [
        '#markup' => '
            <div class="item">
              <div class="first-row">
                <b>' . $content_type->label() . '</b>
                <h2>' . $node_count . '</h2>

              </div>
              <div>' . $add_link . '</div>
            </div>',
      ];
    }
    $build[] = [
      '#markup' => '</div>',
    ];

    return $build;
  }

  public function getConfigSpecificFormFields(FormStateInterface &$form_state, Request &$request, array $block_configuration)
  {
    $form = [];

    // Load all content types.
    $content_types = \Drupal\node\Entity\NodeType::loadMultiple();
    $options = [];
    foreach ($content_types as $content_type) {
      $options[$content_type->id()] = $content_type->label();
    }

    // Create a checkboxes form element to select enabled content types.
    $form['enabled_content_types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Enabled Content Types'),
      '#description' => $this->t('Select the content types that should be displayed in the widget. If none are selected, all will be shown.'),
      '#options' => $options,
      '#default_value' => !empty($block_configuration['plugin_specific_config']['enabled_content_types']) ? $block_configuration['plugin_specific_config']['enabled_content_types'] : [],
    ];

    return $form;
  }

}
