<?php

namespace Drupal\content_planner\Plugin\DashboardBlock;

use Drupal\Core\Database\Database;
use Drupal\content_planner\DashboardBlockBase;

/**
 * Provides a 'Database Table Report' block.
 *
 * @DashboardBlock(
 *   id = "db_table_report_block",
 *   name = @Translation("Database Table Report"),
 *   category = "System",
 *   id_category = "system"
 * )
 */
class DBTableReportBlock extends DashboardBlockBase
{

    /**
     * {@inheritdoc}
     */
    public function build()
    {
        // Fetch the database connection info.
        $database = Database::getConnectionInfo('default')['default'];

        // Fetch sizes and row counts for all tables in the database.
        $query = \Drupal::database()->select('information_schema.tables', 'ist');
        $query->fields('ist', ['TABLE_NAME', 'DATA_LENGTH', 'INDEX_LENGTH', 'TABLE_ROWS']);
        $query->condition('ist.table_schema', $database['database']);
        $tables_info = $query->execute()->fetchAllAssoc('TABLE_NAME');


        $table_sizes = [];
        $total_database_size = 0;

        foreach ($tables_info as $table_name => $table_info) {
            $total_size = $table_info->DATA_LENGTH + $table_info->INDEX_LENGTH;
            $total_database_size += $total_size;
            $table_sizes[$table_name] = [
                'rows' => $table_info->TABLE_ROWS,
                'size_formatted' => $this->formatSize($total_size),
                'size_bytes' => $total_size,
            ];
        }

        // Sort tables by size in descending order.
        uasort($table_sizes, function ($a, $b) {
            return $b['size_bytes'] - $a['size_bytes'];
        });

        $build = [
            'total_size' => [
                '#type' => 'item',
                '#markup' => '<strong>' . $this->t('Total Database Size: @size', ['@size' => $this->formatSize($total_database_size)]) . '</strong>',
                '#prefix' => '<div>',
                '#suffix' => '</div>',
            ],
            'table' => [
                '#type' => 'table',
                '#header' => [
                    $this->t('Table Name'),
                    $this->t('Row Count'),
                    $this->t('Table Size'),
                ],
                '#attributes' => [
                    'class' => ['db-table-report-table'],
                ],
            ],
        ];

        $build['#cache'] = [
            'max-age' => 300, // Cache for 5 minutes
            'contexts' => ['url'], // Cache per URL (optional, if you want caching to be per URL)
        ];

        // Add table sizes to the table section of the build array.
        foreach ($table_sizes as $table => $info) {
            $build['table']['#rows'][] = [$table, $info['rows'], $info['size_formatted']];
        }
        return $build;
    }

    /**
     * Formats a size in bytes into a human-readable format.
     *
     * @param int $size
     *   The size in bytes.
     *
     * @return string
     *   The formatted size.
     */
    protected function formatSize($size)
    {
        if ($size < 1024) {
            return $size . ' B';
        }

        $unit = ['B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
        $exp = floor(log($size, 1024));

        return sprintf('%.2f %s', ($size / pow(1024, $exp)), $unit[$exp]);
    }
}
