<?php

namespace Drupal\content_planner\Plugin\DashboardBlock;

use Drupal\content_planner\DashboardBlockBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'Data Statistics' block.
 *
 * @DashboardBlock(
 *   id = "data_statistics_block",
 *   name = @Translation("Content Statistics"),
 *   description = @Translation("Show number of nodes, taxonomy terms, media items, content types, paragraph types and menus."),
 *   category = "Content",
 *   id_category = "content"
 * )
 */
class DataStatisticsBlock extends DashboardBlockBase implements ContainerFactoryPluginInterface
{

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Constructs a DataStatisticsBlock object.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entityTypeManager, ModuleHandlerInterface $moduleHandler)
  {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entityTypeManager;
    $this->moduleHandler = $moduleHandler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition)
  {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('module_handler')
    );
  }

  public function build()
  {
    $currentUser = \Drupal::currentUser();

    // Fetch total nodes.
    $node_count = $this->entityTypeManager->getStorage('node')->getQuery()->accessCheck(TRUE)->count()->execute();

    // Node overview link
    $node_link = '';
    if ($currentUser->hasPermission('access content overview')) {
      $node_url = \Drupal\Core\Url::fromRoute('view.content.page_1');
      $node_link = \Drupal\Core\Link::fromTextAndUrl($this->t('View'), $node_url)->toString();
    }

    // Fetch total taxonomy terms.
    $term_count = $this->entityTypeManager->getStorage('taxonomy_term')->getQuery()->accessCheck(TRUE)->count()->execute();

    // Taxonomy overview link
    $taxonomy_link = '';
    if ($currentUser->hasPermission('administer taxonomy')) {
      $taxonomy_url = \Drupal\Core\Url::fromRoute('entity.taxonomy_vocabulary.collection');
      $taxonomy_link = \Drupal\Core\Link::fromTextAndUrl($this->t('View'), $taxonomy_url)->toString();
    }

    // Fetch total media items.
    $media_count = $this->entityTypeManager->getStorage('media')->getQuery()->accessCheck(TRUE)->count()->execute();

    // Media overview link
    $media_link = '';
    if ($currentUser->hasPermission('view media')) {
      $media_url = \Drupal\Core\Url::fromRoute('view.media.media_page_list');
      $media_link = \Drupal\Core\Link::fromTextAndUrl($this->t('View'), $media_url)->toString();
    }

    $content_types = $this->entityTypeManager->getStorage('node_type')->getQuery()->execute();
    $content_type_count = count($content_types);

    // Content type overview link
    $content_type_link = '';
    if ($currentUser->hasPermission('administer content types')) {
      $content_type_url = \Drupal\Core\Url::fromRoute('entity.node_type.collection');
      $content_type_link = \Drupal\Core\Link::fromTextAndUrl($this->t('View'), $content_type_url)->toString();
    }

    $rows = [
      [$node_count, $this->t('Nodes'), $node_link],
      [$term_count, $this->t('Taxonomy Terms'), $taxonomy_link],
      [$media_count, $this->t('Media Items'), $media_link],
      [$content_type_count, $this->t('Content Types'), $content_type_link],
    ];

    if ($this->moduleHandler->moduleExists('paragraphs')) {
      $paragraph_types = $this->entityTypeManager->getStorage('paragraphs_type')->getQuery()->execute();
      $paragraph_type_count = count($paragraph_types);

      // Paragraph type overview link
      $paragraph_type_link = '';
      if ($currentUser->hasPermission('administer paragraphs types')) {
        $paragraph_type_url = \Drupal\Core\Url::fromRoute('entity.paragraphs_type.collection');
        $paragraph_type_link = \Drupal\Core\Link::fromTextAndUrl($this->t('View'), $paragraph_type_url)->toString();
      }

      $rows[] = [$paragraph_type_count, $this->t('Paragraph Types'), $paragraph_type_link];
    }

    $menus = $this->entityTypeManager->getStorage('menu')->getQuery()->execute();
    $menu_count = count($menus);

    // Menu overview link
    $menu_link = '';
    if ($currentUser->hasPermission('administer menus')) {
      $menu_url = \Drupal\Core\Url::fromRoute('entity.menu.collection');
      $menu_link = \Drupal\Core\Link::fromTextAndUrl($this->t('View'), $menu_url)->toString();
    }

    $rows[] = [$menu_count, $this->t('Menus'), $menu_link];

    $build = [
      '#type' => 'table',
      '#rows' => $rows,
      '#attributes' => [
        'class' => ['data-statistics-table'],
      ],
    ];

    $build['#cache'] = [
      'max-age' => 300, // Cache for 5 minutes
      'contexts' => ['url'], // Cache per URL (optional, if you want caching to be per URL)
    ];

    return $build;
  }
}
