<?php

namespace Drupal\content_planner\Plugin\DashboardBlock;

use Drupal\content_planner\DashboardBlockBase;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Provides a 'Drupal Housekeeping' block.
 *
 * @DashboardBlock(
 *   id = "drupal_housekeeping_block",
 *   name = @Translation("Cache and Cron Housekeeping"),
 *   category = "System",
 *   id_category = "system"
 * )
 */
class DrupalHouseKeepingBlock extends DashboardBlockBase
{

    /**
     * {@inheritdoc}
     */
    public function build()
    {
        $build = [];

        // Cache size.
        $cache_size = $this->getCacheSize();
        $flush_cache_link = Link::fromTextAndUrl($this->t('Flush cache'), Url::fromRoute('system.performance_settings', [], [
            'query' => ['destination' => \Drupal::request()->getRequestUri()],
            'attributes' => ['class' => ['button', 'button--secondary', 'button--small']],
        ]))->toString();

        $build['#prefix'] = '<div class="housekeeping">';
        $build['#suffix'] = '</div>';
        $build['cache'] = [
            '#type' => 'div',
            '#prefix' => '<div class="housekeeping-item">',
            '#suffix' => '</div>',
        ];
        $build['cache']['cache_size'] = [
            '#markup' => $this->t('Cache size<div>@size</div>', ['@size' => $cache_size]),
        ];
        $build['cache']['link'] = [
            '#type' => 'html_tag',
            '#tag' => 'div',
            '#value' => $flush_cache_link
        ];

        // Last cron run.
        $last_cron_run = $this->getLastCronRun();
        $run_cron_link = Link::fromTextAndUrl($this->t('Run cron'), Url::fromRoute('system.cron_settings', [], [
            'query' => ['destination' => \Drupal::request()->getRequestUri()],
            'attributes' => ['class' => ['button', 'button--secondary', 'button--small']],
        ]))->toString();


        $build['cron'] = [
            '#type' => 'div',
            '#prefix' => '<div class="housekeeping-item">',
            '#suffix' => '</div>',
        ];
        $build['cron']['last_cron_run'] = [
            '#markup' => $this->t('Last cron run<div>@time</div>', ['@time' => $last_cron_run]),
        ];
        $build['cron']['link'] = [
            '#type' => 'html_tag',
            '#tag' => 'div',
            '#value' => $run_cron_link
        ];

        return $build;
    }

    /**
     * Calculate the current size of the cache.
     *
     * @return string
     *   The formatted size of the cache.
     */
    protected function getCacheSize()
    {
        $database = \Drupal::database();
        // Get the current database name.
        $dbName = $database->getConnectionOptions()['database'];

        // Query the information_schema to get sizes of tables with names starting with 'cache_'.
        $query = $database->select('information_schema.TABLES', 'ist');
        $query->fields('ist', ['TABLE_NAME', 'DATA_LENGTH', 'INDEX_LENGTH']);
        $query->condition('ist.TABLE_SCHEMA', $dbName, '=');
        $query->condition('ist.TABLE_NAME', 'cache_%', 'LIKE');

        $result = $query->execute();

        $totalSize = 0;
        foreach ($result as $row) {
            // Sum up the data length and index length to get the total table size.
            $totalSize += $row->DATA_LENGTH + $row->INDEX_LENGTH;
        }

        // Convert bytes to a more readable format.
        $size_in_mb = \Drupal\Core\StringTranslation\ByteSizeMarkup::create($totalSize);

        return $size_in_mb ?: $this->t('Not available');
    }


    /**
     * Get the last time cron was run.
     *
     * @return string
     *   The formatted date/time of the last cron run.
     */
    protected function getLastCronRun()
    {
        $last_run = \Drupal::state()->get('system.cron_last');
        if ($last_run) {
            return \Drupal::service('date.formatter')->formatInterval(\Drupal::time()->getRequestTime() - $last_run) . ' ago';
        } else {
            return $this->t('Never');
        }
    }
}
