<?php

namespace Drupal\content_planner\Plugin\DashboardBlock;

use Drupal\content_planner\DashboardBlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\views\Views;
use Drupal\views\ViewExecutable;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides a custom html block for Content Planner Dashboard.
 *
 * @DashboardBlock(
 *   id = "drupal_view_block",
 *   name = @Translation("Drupal View Widget"),
 *   category = "General",
 *   id_category = "general"
 * )
 */
class DrupalViewBlock extends DashboardBlockBase implements ContainerFactoryPluginInterface
{

  use StringTranslationTrait;

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition)
  {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition
    );
  }

  public function __construct(array $configuration, $plugin_id, $plugin_definition)
  {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  public function build()
  {
    $content = [];
    $config = $this->getConfiguration();

    // Split the view_name into view_id and display_id if it's in the format "view_id.display_id".
    list($view_id, $view_display_id) = explode('.', $config['plugin_specific_config']['view_name'] ?? 'missing.missing');

    if ($view_id !== 'missing' && $view_display_id !== 'missing') {
      $view = Views::getView($view_id);
      if ($view && $view instanceof ViewExecutable && $view->access($view_display_id)) {
        $content = views_embed_view($view_id, $view_display_id);
      }
    } else {
      $content['#markup'] = $this->t('Please configure the widget properly.');
    }

    return $content;
  }


  public function getConfigSpecificFormFields(FormStateInterface &$form_state, Request &$request, array $block_configuration)
  {
    $form = [];
    $view_options = $this->prepareViewOptions();

    // Assuming 'plugin_specific_config' is the correct level where 'view_name' and 'display_id' are stored.
    $selected_view = isset($block_configuration['plugin_specific_config']['view_name']) ? $block_configuration['plugin_specific_config']['view_name'] : '';

    // Debugging: Check if the selected_view matches an option.
    \Drupal::logger('my_module')->notice('Selected view for default value: @view', ['@view' => $selected_view]);

    $form['view_name'] = [
      '#type' => 'select',
      '#title' => $this->t('Select View'),
      '#options' => $view_options,
      '#default_value' => $selected_view,
      '#required' => TRUE,
    ];

    return $form;
  }


  protected function prepareViewOptions()
  {
    $options = [];
    $views = Views::getEnabledViews();
    foreach ($views as $view_id => $view) {
      foreach ($view->get('display') as $display_id => $display) {
        if (isset($display['display_plugin']) && $display['display_plugin'] != 'default') {
          $options["{$view_id}.{$display_id}"] = $view->label() . ' (' . $display['display_title'] . ')';
        }
      }
    }
    return $options;
  }
}
