<?php

namespace Drupal\content_planner\Plugin\DashboardBlock;

use Drupal\content_planner\DashboardBlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides a link block.
 *
 * @DashboardBlock(
 *   id = "link_block",
 *   name = @Translation("Link Block"),
 *   category = "General",
 *   id_category = "general"
 * )
 */
class LinkBlock extends DashboardBlockBase implements ContainerFactoryPluginInterface
{

  // Use the StringTranslationTrait to support translation of strings.
  use StringTranslationTrait;

  /**
   * Factory method for dependency injection.
   *
   * This method is used by Drupal to instantiate the plugin and inject
   * necessary dependencies.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The container to pull out services used in the plugin.
   * @param array $configuration
   *   An array of configuration values.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   *
   * @return static
   *   Returns an instance of this plugin.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition)
  {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition
    );
  }

  /**
   * Constructor.
   *
   * Calls the parent constructor with provided configuration.
   *
   * @param array $configuration
   *   An array of configuration values.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition)
  {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * Build the renderable array for the link block.
   *
   * This function processes the configured links, sorts them by weight, and
   * creates the necessary render array elements to display the links along with
   * their favicons.
   *
   * @return array
   *   A render array for the block content.
   */
  public function build()
  {
    $content = [];

    // Retrieve block configuration.
    $config = $this->getConfiguration();
    if (isset($config['plugin_specific_config']['links']['add_item'])) {
      // Remove any add_item entries from the configuration.
      unset($config['plugin_specific_config']['links']['add_item']);
    }

    // Check if there are any links configured.
    if (!$config['plugin_specific_config'] || !$config['plugin_specific_config']['links']) {
      return [
        '#markup' => $this->t("No links configured."),
      ];
    }

    // Sort the links by their weight.
    usort($config['plugin_specific_config']['links'], function ($a, $b) {
      return $a['weight'] <=> $b['weight'];
    });

    // Create a container for all the links.
    $content['links'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => [
          'links-container',
        ],
      ],
    ];

    // Iterate over each configured link.
    $i = 0;
    foreach ($config['plugin_specific_config']['links'] as $link) {
      // Create a wrapper for each link.
      $content['links']['link'][$i] = [
        '#type' => 'container',
        '#attributes' => [
          'class' => [
            'link-wrapper',
          ],
        ],
      ];

      // Add the favicon image using Google's favicon service.
      $content['links']['link'][$i]['favicon'] = [
        '#markup' => "<img src='https://www.google.com/s2/favicons?domain=" . $link['url'] . "&sz=32' />",
      ];

      // Check if a title is provided, otherwise use the URL as link text.
      if ($link['title']) {
        $content['links']['link'][$i]['link'] = [
          '#markup' => "<a href='" . $link['url'] . "' target='_blank' >" . $link['title'] . " ↗</a>",
        ];
      } else {
        $content['links']['link'][$i]['link'] = [
          '#markup' => "<a href='" . $link['url'] . "' target='_blank'>" . $link['url'] . " ↗</a>",
        ];
      }

      $i++;
    }

    // Attach the library for additional widget behaviors.
    $content['#attached']['library'][] = "content_planner/link-block-widget";

    return $content;
  }

  /**
   * Provide configuration form fields specific to this block plugin.
   *
   * This method defines a form for managing the links, including adding,
   * editing, sorting, and removing link items.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request object.
   * @param array $block_configuration
   *   Existing configuration for the block.
   *
   * @return array
   *   A renderable array representing the configuration form fields.
   */
  public function getConfigSpecificFormFields(FormStateInterface &$form_state, Request &$request, array $block_configuration)
  {
    $form = [];

    // Create a table to display the links with table drag functionality.
    $form['links'] = [
      '#type' => 'table',
      '#prefix' => "<div id='links-items-wrapper'>",
      '#suffix' => "</div>",
      '#header' => [
        $this->t('Url'),
        $this->t('Title'),
        $this->t('Weight'),
        $this->t('Actions'),
      ],
      '#empty' => $this->t('No links available.'),
      '#tabledrag' => [
        [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => 'links-order-weight',
        ],
      ],
    ];

    // Retrieve existing links configuration.
    $default_value = isset($block_configuration['plugin_specific_config']['links']) ? $block_configuration['plugin_specific_config']['links'] : [];
    if (isset($default_value['add_item'])) {
      // Remove any temporary add_item entries.
      unset($default_value['add_item']);
    }

    // Retrieve the current items from the form state or initialize them.
    $items = $form_state->get('items');
    if ($items === NULL) {
      $items = $default_value;
      $form_state->set('items', $items);
    } else {
      $default_value = $items;
    }

    // Build form rows for each link item.
    foreach ($items as $delta => $value) {
      // Mark each row as draggable.
      $form['links'][$delta]['#attributes']['class'][] = 'draggable';

      // URL field for the link.
      $form['links'][$delta]['url'] = [
        '#type' => 'url',
        '#title' => $this->t('URL'),
        '#description' => $this->t('Enter the URL.'),
        '#required' => TRUE,
        '#default_value' => $default_value[$delta]['url'] ?: NULL,
      ];

      // Textfield for the link title.
      $form['links'][$delta]['title'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Title'),
        '#description' => $this->t('Enter the link title.'),
        '#default_value' => $default_value[$delta]['title'] ?: NULL,
      ];

      // Weight field for ordering.
      $form['links'][$delta]['weight'] = [
        '#type' => 'weight',
        '#title' => $this->t('Weight'),
        '#title_display' => 'invisible',
        '#attributes' => ['class' => ['links-order-weight']],
      ];

      // Remove action button for each link item.
      $form['links'][$delta]['actions'] = [
        '#type' => 'submit',
        '#name' => 'remove_item_' . $delta,
        '#value' => $this->t('Remove'),
        '#submit' => [get_class($this) . '::removeItem'],
        '#ajax' => [
          'callback' => [$this, 'ajaxCallback'],
          'wrapper' => 'links-items-wrapper',
        ],
        '#limit_validation_errors' => [],
      ];
    }

    // Button to add another link item.
    $form['add_item'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add another link'),
      '#submit' => [get_class($this) . '::addItem'],
      '#ajax' => [
        'callback' => [$this, 'ajaxCallback'],
        'wrapper' => 'links-items-wrapper',
      ],
      '#limit_validation_errors' => [],
    ];

    return $form;
  }

  /**
   * Ajax callback for the configuration form.
   *
   * Returns the links part of the form to be refreshed via Ajax.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return array
   *   The portion of the form that should be re-rendered.
   */
  public function ajaxCallback(array &$form, FormStateInterface $form_state)
  {
    return $form['plugin_specific_config']['links'];
  }

  /**
   * Adds an item to the list of link items.
   *
   * This static method is triggered when the "Add another link" button is
   * pressed.
   *
   * @param array $form
   *   The current form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   */
  public static function addItem(array &$form, FormStateInterface $form_state)
  {
    $items = $form_state->get('items');
    if (!$items) {
      $items = [];
    }

    // Append a new item by adding a new entry to the items array.
    $items[] = count($items);
    $form_state->set('items', $items);
    $form_state->setRebuild();
  }

  /**
   * Removes an item from the list of link items.
   *
   * This static method is triggered when a remove button is clicked for a
   * specific link item.
   *
   * @param array $form
   *   The current form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   */
  public static function removeItem(array &$form, FormStateInterface $form_state)
  {
    // Get the triggering element to determine which item to remove.
    $triggering_element = $form_state->getTriggeringElement();
    $delta = $triggering_element['#name'];
    $items = $form_state->get('items');
    // Extract the delta key from the button name.
    $delta_key = str_replace("remove_item_", "", $delta);
    unset($items[$delta_key]);
    $form_state->set('items', $items);
    $form_state->setRebuild();
  }

}
