<?php

namespace Drupal\content_planner\Plugin\DashboardBlock;

use Drupal\content_planner\DashboardBlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\UserInterface;
use Drupal\workflows\WorkflowInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\workflows\Entity\Workflow;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Provides a MatomoMonthlyGraphStats block for Content Planner Dashboard.
 *
 * @DashboardBlock(
 *   id = "matomo_monthly_graph_stats",
 *   name = @Translation("Matomo Monthly Graph Stats"),
 *   category = "Site Analytics",
 *   id_category = "site_analytics"
 * )
 */
class MatomoMonthlyGraphStats extends DashboardBlockBase
{
  use StringTranslationTrait;

  /**
   * The HTTP client to fetch the stats.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition)
  {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->httpClient = $container->get('http_client');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function build()
  {
    $config = $this->getConfiguration();

    if (!isset(
      $config['plugin_specific_config']['matomo_url'],
      $config['plugin_specific_config']['site_id'],
      $config['plugin_specific_config']['api_key']
    )) {
      return [
        '#markup' => $this->t('Please configure the Matomo settings in the dashboard block configuration.'),
      ];
    }

    // Fetch the monthly stats.
    $stats = $this->getMonthlyStats(
      $config['plugin_specific_config']['matomo_url'],
      $config['plugin_specific_config']['site_id'],
      $config['plugin_specific_config']['api_key']
    );

    $svg = $this->generateSVGChart($stats);

    $build['svg_chart'] = [
      '#markup' => '<div class="responsive-svg-container">' . $svg . '</div>',
      '#allowed_tags' => ['svg', 'rect', 'text', 'line', 'div', 'circle'],
      '#attached' => [
          'library' => [
              'nodehive_core/node',  // Assuming you have your CSS here
          ],
      ],
    ];

    if (!empty($config['plugin_specific_config']['matomo_link'])) {
      $build['matomo_link'] = [
        '#type' => 'link',
        '#title' => $this->t('Open Matomo Dashboard'),
        '#url' => \Drupal\Core\Url::fromUri($config['plugin_specific_config']['matomo_link']),
        '#attributes' => [
          'class' => ['button', 'button--small'],
          'target' => '_blank'
        ],
      ];
    }

    $build['#cache'] = [
      'max-age' => 300, // Cache for 5 minutes
      'contexts' => ['url'], // Cache per URL (optional, if you want caching to be per URL)
    ];


    return $build;
  }

  protected function generateSVGChart(array $stats)
  {
    $maxValue = max(max(array_column($stats, 'visits')), max(array_column($stats, 'page_views')));

    // Round the max value up to the nearest 100
    $roundedMaxValue = ceil($maxValue / 100) * 100;

    $barWidth = 16;
    $gap = 1;
    $heightFactor = 200 / $roundedMaxValue;
    $chartWidth = count($stats) * ($barWidth + $gap);

    $svg = '<svg viewBox="0 0 ' . ($chartWidth + 40) . ' 250" preserveAspectRatio="xMidYMid meet">';


    // Y-Axis line
    $svg .= '<line x1="0" y1="10" x2="0" y2="210" stroke="#9EBEBD" stroke-width="1" />';
    // Y-Axis labels
    $svg .= '<text x="2" y="220" font-family="Arial" font-size="10" fill="black">0</text>';
    $svg .= '<text x="2" y="8" font-family="Arial" font-size="10" fill="black">' . $roundedMaxValue . '</text>';

    // Horizontal grid lines
    $svg .= '<line x1="0" y1="10" x2="' . ($chartWidth + 30) . '" y2="10" stroke="#9EBEBD" stroke-width="0.5" />';
    $svg .= '<line x1="0" y1="60" x2="' . ($chartWidth + 30) . '" y2="60" stroke="#9EBEBD" stroke-width="0.5" />';
    $svg .= '<line x1="0" y1="110" x2="' . ($chartWidth + 30) . '" y2="110" stroke="#9EBEBD" stroke-width="0.5" />';
    $svg .= '<line x1="0" y1="160" x2="' . ($chartWidth + 30) . '" y2="160" stroke="#9EBEBD" stroke-width="0.5" />';
    $svg .= '<line x1="0" y1="210" x2="' . ($chartWidth + 30) . '" y2="210" stroke="#9EBEBD" stroke-width="0.5" />'; // Line for 0


    $textX = 20;
    $textY = 250;
    $svg .= '<text x="' . $textX . '" y="' . $textY . '" font-family="Arial" font-size="14" fill="black">' . date('F Y') . '</text>';

    $x = $textX;
    foreach ($stats as $date => $values) {
      $height_visits = $values['visits'] == 0 ? 1 : $values['visits'] * $heightFactor;
      $height_pageviews = $values['page_views'] == 0 ? 1 : $values['page_views'] * $heightFactor;

      // Drawing the entire bar for pageviews
      $svg .= '<rect x="' . $x . '" y="' . (210 - $height_pageviews) . '" width="' . $barWidth . '" height="' . $height_pageviews . '" fill="#00726E" />';
      // Overlaying the bar for visits on top of the pageviews bar
      $svg .= '<rect x="' . $x . '" y="' . (210 - $height_visits) . '" width="' . $barWidth . '" height="' . $height_visits . '" fill="#00A39F" />';

      // Displaying the date for each column below the bars
      $day = substr($date, -2);
      $dayOfWeek = date('w', strtotime($date));

      if ($dayOfWeek == 6 || $dayOfWeek == 0) { // 6 = Saturday, 0 = Sunday
        // Drawing a circle for Saturday and Sunday
        $circleRadius = 8;
        $svg .= '<circle cx="' . ($x + 9) . '" cy="226" r="' . $circleRadius . '" fill="none" stroke="grey" stroke-width="1" />';
        $svg .= '<text x="' . ($x + 4) . '" y="230" font-family="Arial" font-size="10" fill="grey">' . $day . '</text>';
      } else {
        $svg .= '<text x="' . ($x + 4) . '" y="230" font-family="Arial" font-size="10" fill="black">' . $day . '</text>';
      }


      $x += $barWidth + $gap;
    }

    $svg .= '</svg>';
    return $svg;
  }







  /**
   * Fetch monthly stats from Matomo.
   */
  public function getMonthlyStats($matomoUrl, $siteId, $apiKey)
  {
    // Define the start and end dates for the current month
    $start_date = date('Y-m-01');
    $end_date = date('Y-m-t');

    $url = $matomoUrl . "/index.php?module=API&method=API.get&idSite=$siteId&period=day&date=$start_date,$end_date&format=JSON&token_auth=$apiKey";

    try {
      $response = $this->httpClient->get($url);
      $data = json_decode($response->getBody(), TRUE);
      $stats = [];

      foreach ($data as $date => $values) {
        $stats[$date] = [
          'visits' => $values['nb_visits'] ?? 0,
          'page_views' => $values['nb_pageviews'] ?? 0,
        ];
      }

      // Ensure all days of the current month are represented
      $current_month_days = range(1, date('t'));
      foreach ($current_month_days as $day) {
        $full_date = date('Y-m-') . str_pad($day, 2, '0', STR_PAD_LEFT);
        if (!isset($stats[$full_date])) {
          $stats[$full_date] = [
            'visits' => 0,
            'page_views' => 0,
          ];
        }
      }

      ksort($stats); // Ensure stats are sorted by date

      return $stats;
    } catch (\Exception $e) {
      // Handle exception or log error based on your needs.
      return []; // return an empty array in case of an error.
    }
  }


  /**
   * {@inheritdoc}
   */
  public function getConfigSpecificFormFields(
    FormStateInterface &$form_state,
    Request &$request,
    array $block_configuration
  ) {
    $form = [];

    $form['matomo_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Matomo URL'),
      '#description' => $this->t('The URL of your Matomo instance.'),
      '#default_value' => $block_configuration['plugin_specific_config']['matomo_url'] ?? '',
      '#required' => TRUE,
    ];

    $form['site_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Site ID'),
      '#description' => $this->t('The ID of the site in Matomo you want to fetch data for.'),
      '#default_value' => $block_configuration['plugin_specific_config']['site_id'] ?? '',
      '#required' => TRUE,
    ];

    $form['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Key'),
      '#description' => $this->t('The API key for authentication with Matomo.'),
      '#default_value' => $block_configuration['plugin_specific_config']['api_key'] ?? '',
      '#required' => TRUE,
    ];

    $form['matomo_link'] = [
      '#type' => 'url',
      '#title' => $this->t('Link to Matomo'),
      '#description' => $this->t('The direct link to your Matomo dashboard.'),
      '#default_value' => $block_configuration['plugin_specific_config']['matomo_link'] ?? '',
      '#required' => FALSE,
    ];

    return $form;
  }
}
