<?php

namespace Drupal\content_planner\Plugin\DashboardBlock;

use Drupal\content_planner\DashboardBlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\UserInterface;
use Drupal\workflows\WorkflowInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\workflows\Entity\Workflow;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Provides a MatomoWeeklyStats block for Content Planner Dashboard.
 *
 * @DashboardBlock(
 *   id = "matomo_weekly_stats",
 *   name = @Translation("Matomo Weekly Stats"),
 *   category = "Site Analytics",
 *   id_category = "site_analytics"
 * )
 */
class MatomoWeeklyStats extends DashboardBlockBase
{
  use StringTranslationTrait;

  /**
   * The HTTP client to fetch the stats.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition)
  {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->httpClient = $container->get('http_client');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function build()
  {
    $config = $this->getConfiguration();

    if (!isset(
      $config['plugin_specific_config']['matomo_url'],
      $config['plugin_specific_config']['site_id'],
      $config['plugin_specific_config']['api_key']
    )) {
      return [
        '#markup' => $this->t('Please configure the Matomo settings in the dashboard block configuration.'),
      ];
    }

    $stats = $this->getWeeklyStats(
      $config['plugin_specific_config']['matomo_url'],
      $config['plugin_specific_config']['site_id'],
      $config['plugin_specific_config']['api_key']
    );

    $header = array_merge(['Metrics'], array_keys($stats));
    $rows = [];

    // Define the metrics you want in the table
    $metrics = ['unique_visitors', 'visits', 'page_views', 'bounce_rate', 'avg_time_on_site', 'actions_per_visit', 'from_search_engines', 'from_social_networks', 'direct_entry', 'conversions', 'conversion_rate'];

    foreach ($metrics as $metric) {
      $row = [$metric];
      foreach ($stats as $date => $values) {
        $row[] = $values[$metric];
      }
      $rows[] = $row;
    }


    $build['table'] = [
      '#theme' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No data available'),
    ];

    $build['#cache'] = [
      'max-age' => 300, // Cache for 5 minutes
      'contexts' => ['url'], // Cache per URL (optional, if you want caching to be per URL)
    ];

    if (!empty($config['plugin_specific_config']['matomo_link'])) {
      $build['matomo_link'] = [
        '#type' => 'link',
        '#title' => $this->t('Open Matomo Dashboard'),
        '#url' => \Drupal\Core\Url::fromUri($config['plugin_specific_config']['matomo_link']),
        '#attributes' => [
          'class' => ['button', 'button--small'],
          'target' => '_blank'
        ],
      ];
    }

    return $build;
  }

  /**
   * Fetch weekly stats from Matomo.
   */
  public function getWeeklyStats($matomoUrl, $siteId, $apiKey)
  {
    $url = $matomoUrl . "/index.php?module=API&method=API.get&idSite=$siteId&period=day&date=last7&format=JSON&token_auth=$apiKey";

    try {
      $response = $this->httpClient->get($url);
      $data = json_decode($response->getBody(), TRUE);
      $stats = [];

      foreach ($data as $date => $values) {
        $stats[$date] = [
          'visits' => $values['nb_visits'] ?? 0,
          'page_views' => $values['nb_pageviews'] ?? 0,
          'unique_visitors' => $values['nb_uniq_visitors'] ?? 0,
          'bounce_rate' => $values['bounce_rate'] ?? 0,
          'avg_time_on_site' => $values['avg_time_on_site'] ?? 0,
          'actions_per_visit' => $values['nb_actions_per_visit'] ?? 0,
          'from_search_engines' => $values['Referrers_visitorsFromSearchEngines_percent'] ?? 0,
          'from_social_networks' => $values['Referrers_visitorsFromSocialNetworks_percent'] ?? 0,
          'direct_entry' => $values['Referrers_visitorsFromDirectEntry_percent'] ?? 0,
          'conversions' => $values['nb_conversions'] ?? 0,
          'conversion_rate' => $values['conversion_rate'] ?? 0,
        ];
      }
      return $stats;
    } catch (\Exception $e) {
      // Handle exception or log error based on your needs.
      return []; // return an empty array in case of an error.
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getConfigSpecificFormFields(
    FormStateInterface &$form_state,
    Request &$request,
    array $block_configuration
  ) {
    $form = [];

    $form['matomo_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Matomo URL'),
      '#description' => $this->t('The URL of your Matomo instance.'),
      '#default_value' => $block_configuration['plugin_specific_config']['matomo_url'] ?? '',
      '#required' => TRUE,
    ];

    $form['site_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Site ID'),
      '#description' => $this->t('The ID of the site in Matomo you want to fetch data for.'),
      '#default_value' => $block_configuration['plugin_specific_config']['site_id'] ?? '',
      '#required' => TRUE,
    ];

    $form['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Key'),
      '#description' => $this->t('The API key for authentication with Matomo.'),
      '#default_value' => $block_configuration['plugin_specific_config']['api_key'] ?? '',
      '#required' => TRUE,
    ];

    $form['matomo_link'] = [
      '#type' => 'url',
      '#title' => $this->t('Link to Matomo'),
      '#description' => $this->t('The direct link to your Matomo dashboard.'),
      '#default_value' => $block_configuration['plugin_specific_config']['matomo_link'] ?? '',
      '#required' => FALSE,
    ];

    return $form;
  }
}
