<?php

namespace Drupal\content_planner\Plugin\DashboardBlock;

use Drupal\content_planner\DashboardBlockBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Provides a MatomoYearlyTrafficStats block for Content Planner Dashboard.
 *
 * @DashboardBlock(
 *   id = "matomo_yearly_traffic_stats",
 *   name = @Translation("Matomo Yearly Traffic Stats"),
 *   category = "Site Analytics",
 *   id_category = "site_analytics"
 * )
 */
class MatomoYearlyTrafficStats extends DashboardBlockBase
{

  use StringTranslationTrait;

  /**
   * The HTTP client to fetch the stats.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition)
  {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->httpClient = $container->get('http_client');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function build()
  {
    $config = $this->getConfiguration();

    if (!isset(
      $config['plugin_specific_config']['matomo_url'],
      $config['plugin_specific_config']['site_id'],
      $config['plugin_specific_config']['api_key']
    )) {
      return [
        '#markup' => $this->t('Please configure the Matomo settings in the dashboard block configuration.'),
      ];
    }

    $stats = $this->getYearlyStats(
      $config['plugin_specific_config']['matomo_url'],
      $config['plugin_specific_config']['site_id'],
      $config['plugin_specific_config']['api_key']
    );

    $svg = $this->generateContributionGraph($stats);

    $build['svg_chart'] = [
      '#markup' => '<div class="responsive-svg-container">' . $svg . '</div>',
      '#allowed_tags' => ['svg', 'rect', 'div', 'text', 'line', 'circle', 'g', 'title'],
      '#attached' => [
        'library' => [
          'nodehive_core/node',  // Assuming you have your CSS here
        ],
      ],
    ];

    if (!empty($config['plugin_specific_config']['matomo_link'])) {
      $build['matomo_link'] = [
        '#type' => 'link',
        '#title' => $this->t('Open Matomo Dashboard'),
        '#url' => \Drupal\Core\Url::fromUri($config['plugin_specific_config']['matomo_link']),
        '#attributes' => [
          'class' => ['button', 'button--small'],
          'target' => '_blank'
        ],
      ];
    }

    $build['#cache'] = [
      'max-age' => 300, // Cache for 5 minutes
      'contexts' => ['url'], // Cache per URL (optional, if you want caching to be per URL)
    ];

    return $build;
  }

  /**
   * Fetch yearly stats from Matomo.
   */
  public function getYearlyStats($matomoUrl, $siteId, $apiKey)
  {
    // Define the start and end dates for the current year
    $lastYear = date('Y') - 1;
    $lastMondayOfLastYear = date('Y-m-d', strtotime("last monday of December $lastYear"));
    $start_date = $lastMondayOfLastYear;
    $end_date = date('Y-12-31');

    $url = $matomoUrl . "/index.php?module=API&method=API.get&idSite=$siteId&period=day&date=$start_date,$end_date&format=JSON&token_auth=$apiKey";

    try {
      $response = $this->httpClient->get($url);
      $data = json_decode($response->getBody(), TRUE);

      $stats = [];
      foreach ($data as $date => $values) {
        $stats[$date] = [
          'row' => (date('w', strtotime($date)) + 6) % 7,
          'weekday' => date('D', strtotime($date)), // Mon, Tue, Wed, etc.
          'col' => date('W', strtotime($date)), // Week number
          'visits' => $values['nb_visits'] ?? 0,
          'page_views' => $values['nb_pageviews'] ?? 0,
        ];
      }

      return $stats;
    } catch (\Exception $e) {
      // Handle exception or log error based on your needs.
      return []; // return an empty array in case of an error.
    }
  }






  /**
   * Visualize yearly stats in a GitHub-style contribution graph.
   */
  protected function generateContributionGraph(array $stats)
  {
    $daysOfWeek = ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun'];
    $rectSize = 10;
    $gap = 2;
    $width = (52 * ($rectSize + $gap)) + 30; // 52 weeks + space for week numbers
    $height = (7 * ($rectSize + $gap)) + 20; // 7 days of the week + space for day names

    $svg = '<svg viewBox="0 0 ' . $width . ' ' . $height . '" preserveAspectRatio="xMidYMid meet">';

    // Add the labels for days of the week
    foreach ($daysOfWeek as $i => $day) {
      $y = ($i * ($rectSize + $gap)) + 28;
      $svg .= '<text x="0" y="' . $y . '" font-size="10" fill="grey">' . $day . '</text>';
    }

    $currentWeek = (int)date('W', strtotime(key($stats)));
    $col = 0;
    $row = 0;

    // Calculate the maximum visits from stats
    $maxVisits = max(array_column($stats, 'visits'));

    // Define thresholds based on percentages of maxVisits
    $threshold25 = 0.25 * $maxVisits;
    $threshold50 = 0.50 * $maxVisits;
    $threshold75 = 0.75 * $maxVisits;
    $threshold90 = 0.90 * $maxVisits;
    $threshold100 = $maxVisits; // This is just the maximum itself


    $today = date('Y-m-d');

    foreach ($stats as $date => $values) {
      $color = "#eee"; // Default color for no data

      if ($values['visits'] > 5) {
        $color = "#d6e685"; // base color for any traffic > 5
      }
      if ($values['visits'] > $threshold25) {
        $color = "#8cc665";
      }
      if ($values['visits'] > $threshold50) {
        $color = "#44a340";
      }
      if ($values['visits'] > $threshold75) {
        $color = "#1e6823";
      }
      if ($values['visits'] > $threshold90) {
        $color = "#116611";
      }
      if ($values['visits'] == $threshold100) {
        $color = "#ADD8E6";
      }

      $x = ($values['col'] * ($rectSize + $gap)) + 30;
      $y = ($values['row'] * ($rectSize + $gap)) + 20;

      // Generate the tooltip content for the rectangle
      $tooltipContent = "Date: $date\nVisits: " . $values['visits'] . "\nPageviews: " . $values['page_views'];

      // Add the tooltip to the SVG
      $svg .= '<g class="tooltip-group">'; // Start a group for the rectangle and its title
      $svg .= '<title>' . htmlspecialchars($tooltipContent) . '</title>';

      if ($date == $today) {
        $svg .= '<rect x="' . $x . '" y="' . $y . '" width="' . $rectSize . '" height="' . $rectSize . '" fill="' . $color . '" stroke="black" stroke-width="1" />';
      } else {
        $svg .= '<rect x="' . $x . '" y="' . $y . '" width="' . $rectSize . '" height="' . $rectSize . '" fill="' . $color . '" />';
      }

      // Determine the text position. Adjust the offsets to position the text correctly.
      $textX = $x + ($rectSize / 2);
      $textY = $y + ($rectSize / 2) + 3;

      // Add the text element for the number of visits. Adjust the font-size and fill as needed.
      //$svg .= '<text x="' . $textX . '" y="' . $textY . '" class="svg-tooltip"  visibility="hidden" font-size="6" fill="black" text-anchor="middle">' . htmlspecialchars($tooltipContent) . '</text>';

      $svg .= '</g>'; // End the group

      if ((int)date('W', strtotime($date)) != $currentWeek) {
        $svg .= '<text x="' . $x . '" y="15" font-size="8" fill="grey">' . $currentWeek . '</text>';
        $currentWeek = (int)date('W', strtotime($date));
        $col++;
        $row = 0;
      } else {
        $row = ($row + 1) % 7;  // Ensure the row wraps around to Monday after Sunday.
      }
    }


    $svg .= '</svg>';
    return $svg;
  }


  /**
   * {@inheritdoc}
   */
  public function getConfigSpecificFormFields(
    FormStateInterface &$form_state,
    Request &$request,
    array $block_configuration
  ) {
    $form = [];

    $form['matomo_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Matomo URL'),
      '#description' => $this->t('The URL of your Matomo instance.'),
      '#default_value' => $block_configuration['plugin_specific_config']['matomo_url'] ?? '',
      '#required' => TRUE,
    ];

    $form['site_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Site ID'),
      '#description' => $this->t('The ID of the site in Matomo you want to fetch data for.'),
      '#default_value' => $block_configuration['plugin_specific_config']['site_id'] ?? '',
      '#required' => TRUE,
    ];

    $form['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Key'),
      '#description' => $this->t('The API key for authentication with Matomo.'),
      '#default_value' => $block_configuration['plugin_specific_config']['api_key'] ?? '',
      '#required' => TRUE,
    ];

    $form['matomo_link'] = [
      '#type' => 'url',
      '#title' => $this->t('Link to Matomo'),
      '#description' => $this->t('The direct link to your Matomo dashboard.'),
      '#default_value' => $block_configuration['plugin_specific_config']['matomo_link'] ?? '',
      '#required' => FALSE,
    ];

    return $form;
  }
}
