<?php

namespace Drupal\content_planner\Plugin\DashboardBlock;

use Drupal\content_planner\DashboardBlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Datetime\DateFormatterInterface;

/**
 * Provides a Dashboard block for Content Planner Dashboard.
 *
 * @DashboardBlock(
 *   id = "drupal_blog_rss_reader_block",
 *   name = @Translation("RSS Feed Widget"),
 *   category = "General",
 *   id_category = "general"
 * )
 */
class RSSReaderBlock extends DashboardBlockBase implements ContainerFactoryPluginInterface {

  use StringTranslationTrait;

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The Kanban Statistic service.
   *
   * @var \Drupal\content_kanban\KanbanStatisticService
   */
  protected $kanbanStatisticService;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * RSS Feed URL.
   */
  protected $rssUrl = 'https://www.drupal.org/section-blog/2603760/feed';

  /**
   * Number of items to show.
   */
  protected $numItemsToShow = 5;

  /**
   * Constructs a new RSSReaderBlock.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, DateFormatterInterface $date_formatter) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->dateFormatter = $date_formatter;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('date.formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];
    $config = $this->getConfiguration();
    $rss_url = $config['plugin_specific_config']['rss_url'] ?? $this->rssUrl;
    $num_items_to_show = $config['plugin_specific_config']['num_items_to_show'] ?? $this->numItemsToShow;

    // Load the RSS feed.
    if ($rss = simplexml_load_file($rss_url)) {
      // Get the 'item' elements.
      $items = $rss->channel->item;

      // Keep track of how many items we've displayed.
      $count = 0;

      $build['rss_feed'] = [
        '#type' => 'html_tag',
        '#tag' => 'ul',
        '#attributes' => ['class' => ['rss-feed-list']],
      ];

      foreach ($items as $item) {
        if ($count >= $num_items_to_show) {
          break;
        }

        // Format the date according to the Drupal system format.
        $timestamp = strtotime((string) $item->pubDate);
        $date = $this->dateFormatter->format($timestamp, 'custom', 'm/d/Y H:i');

        // Check for image in <media:content> or <enclosure>.
        $image_url = null;
        if (isset($item->enclosure)) {
          $image_url = (string) $item->enclosure['url'];
        } elseif ($item->children('media', true)->content) {
          $image_url = (string) $item->children('media', true)->content->attributes()->url;
        }

        // Construct the markup for the list item.
        $item_markup = '<div class="rss-feed-item-date-title"><div>' . $date . '</div><b><a href="' . $item->link . '" target="_blank">' . $item->title . '</a></b></div>';

        $build['rss_feed'][] = [
          '#type' => 'html_tag',
          '#tag' => 'li',
          '#attributes' => ['class' => ['rss-feed-item']],
          'image' => [
            '#type' => 'html_tag',
            '#tag' => 'div',
            '#attributes' => [
              'class' => ['rss-feed-item-image'],
              'style' => $image_url ? 'background-image: url(' . $image_url . ');' : 'background-color: var(--gin-bg-header);',
            ],
          ],
          'content' => [
            '#markup' => $item_markup,
          ],
        ];

        $count++;
      }
    }

    $build['#attached']['library'][] = 'content_planner/rss_feed_widget';

    $build['#cache'] = [
      'max-age' => 5 * 60, // Cache for 5 minutes
      'contexts' => ['url'], // Cache per URL (optional, if you want caching to be per URL)
    ];

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function getConfigSpecificFormFields(FormStateInterface &$form_state, Request &$request, array $block_configuration) {
    $form = [];

    $form['rss_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('RSS Feed URL'),
      '#default_value' => $block_configuration['plugin_specific_config']['rss_url'] ?? $this->rssUrl,
      '#description' => $this->t('Enter the URL of the RSS feed you want to display.'),
    ];

    $form['num_items_to_show'] = [
      '#type' => 'number',
      '#title' => $this->t('Number of items to show'),
      '#default_value' => $block_configuration['plugin_specific_config']['num_items_to_show'] ?? $this->numItemsToShow,
      '#description' => $this->t('Enter the number of RSS feed items to display.'),
      '#min' => 1,
    ];

    return $form;
  }
}
