<?php

namespace Drupal\content_planner\Plugin\DashboardBlock;

use Drupal\Core\Database\Database;
use Drupal\content_planner\DashboardBlockBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;

/**
 * Provides an 'System Information' block.
 *
 * @DashboardBlock(
 *   id = "system_info_block",
 *   name = @Translation("System Information"),
 *   category = "System",
 *   id_category = "system"
 * )
 */
class SystemInfoBlock extends DashboardBlockBase implements ContainerFactoryPluginInterface
{

  public function build()
  {
    // Fetch the database connection info.
    $database = Database::getConnectionInfo('default')['default'];

    // Your new database info code
    $query = \Drupal::database()->select('information_schema.tables', 'ist');
    $query->fields('ist', ['TABLE_NAME', 'DATA_LENGTH', 'INDEX_LENGTH']);
    $query->condition('ist.table_schema', $database['database']);
    $result = $query->execute()->fetchAllAssoc('TABLE_NAME');
    $database_size = array_sum(array_map(function ($item) {
      return $item->DATA_LENGTH + $item->INDEX_LENGTH;
    }, $result));
    $table_count = count($result);



    // Active users in last hour
    $active_users = \Drupal::database()->query("SELECT COUNT(uid) FROM {sessions} WHERE timestamp >= :time", [':time' => \Drupal::time()->getRequestTime() - 3600])->fetchField();

    // Get the last update time.
    $last_update = \Drupal::state()->get('system.last_updated');

    // If last_update is numeric, format it. Else, set it to "Never".
    $last_update_formatted = is_numeric($last_update) ? \Drupal::service('date.formatter')->format($last_update) : $this->t('Never');


    // Existing info
    $build = [
      '#type' => 'table',
      '#header' => [$this->t('Property'), $this->t('Value')],
      '#rows' => [
        [$this->t('Database'), $database['driver'] . '://' . $database['host']],
        [$this->t('PHP Version'), phpversion()],
        [$this->t('Drupal Version'), \Drupal::VERSION],
        [$this->t('Web Server'), $_SERVER['SERVER_SOFTWARE']],
        [$this->t('Memory Usage'), memory_get_usage()],
        [$this->t('PHP Max Execution Time'), ini_get('max_execution_time') . 's'],
        [$this->t('Loaded PHP Extensions'), implode(', ', get_loaded_extensions())],
        [$this->t('Max File Upload Size'), ini_get('upload_max_filesize')],
        [$this->t('Operating System'), php_uname()],
        [$this->t('Database Size'), $this->formatSize($database_size)],
        [$this->t('Number of Database Tables'), $table_count],
        [$this->t('PHP Post Max Size'), ini_get('post_max_size')],
        [$this->t('PHP Time Zone'), ini_get('date.timezone')],
        [$this->t('Last Drupal Update'), $last_update_formatted],
      ],
      '#attributes' => [
        'class' => ['enhanced-system-info-table'],
      ],
    ];

    $build['#cache'] = [
      'max-age' => 300, // Cache for 5 minutes
      'contexts' => ['url'], // Cache per URL (optional, if you want caching to be per URL)
    ];

    // Return the render array.
    return $build;
  }

  /**
   * Formats a size in bytes into a human-readable format.
   *
   * @param int $size
   *   The size in bytes.
   *
   * @return string
   *   The formatted size.
   */
  protected function formatSize($size)
  {
    if ($size < 1024) {
      return $size . ' B';
    }

    $unit = ['B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
    $exp = floor(log($size, 1024));

    return sprintf('%.2f %s', ($size / pow(1024, $exp)), $unit[$exp]);
  }
}
