<?php

namespace Drupal\content_planner\Plugin\DashboardBlock;

use Drupal\content_planner\DashboardBlockBase;
use Drupal\webform\Entity\WebformSubmission;
use Drupal\Core\Url;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\webform\Entity\Webform;

/**
 * Provides a Dashboard block with the last 10 webform submissions.
 *
 * @DashboardBlock(
 *   id = "webforms_last_submissions_block",
 *   name = @Translation("Webform: Latest Submissions"),
 *   category = "Webform Reports",
 *   id_category = "webform_reports"
 * )
 */
class WebformsLastSubmissionsBlock extends DashboardBlockBase
{
  /**
   * {@inheritdoc}
   */
  public function build()
  {
    // Check if the Webform module is enabled.
    if (!\Drupal::moduleHandler()->moduleExists('webform')) {
      return [
        '#markup' => $this->t('Webform module is not enabled.'),
      ];
    }

    $config = $this->getConfiguration();
    $selected_webforms = !empty($config['plugin_specific_config']['selected_webforms']) ? array_filter($config['plugin_specific_config']['selected_webforms']) : [];
    $submission_limit = !empty($config['plugin_specific_config']['submission_limit']) ? $config['plugin_specific_config']['submission_limit'] : 10;


    $rows = [];

    // Load the last 10 webform submissions, optionally filtered by selected webforms.
    $query = \Drupal::entityQuery('webform_submission')
      ->sort('created', 'DESC')
      ->accessCheck(TRUE)
      ->range(0, $submission_limit);

    // Filter by selected webforms if any.
    if (!empty($selected_webforms)) {
      $query->condition('webform_id', array_keys($selected_webforms), 'IN');
    }

    $ids = $query->execute();
    $submissions = WebformSubmission::loadMultiple($ids);

    foreach ($submissions as $submission) {
      /** @var \Drupal\webform\Entity\WebformSubmission $submission */
      $submission_url = Url::fromRoute('entity.webform_submission.canonical', ['webform' => $submission->getWebform()->id(), 'webform_submission' => $submission->id()]);

      $webform_label = $submission->getWebform()->label();

      $data_preview = [];
      $data = $submission->getData();
      foreach ($data as $field_name => $value) {
        // Assuming the value is simple. For more complex fields, you might need additional formatting.
        $data_preview[] = str_replace('_', ' ', mb_strtoupper($field_name) . ': ' . (is_array($value) ? implode(', ', $value) : $value));
      }

      // Limit the preview to a certain number of lines for brevity.
      $preview_limit = 5;
      $data_preview = array_slice($data_preview, 0, $preview_limit);
      if (count($data_preview) >= $preview_limit) {
        $data_preview[] = '...';  // Indicate that there's more data not shown.
      }

      $created = \Drupal::service('date.formatter')->formatInterval(time() - $submission->getCreatedTime()) . ' ago';

      $rows[] = [
        //'webform' => $webform_label,
        'submission' => [
          'class' => ['class' => 'align-top'],
          'data' => [
            '#markup' => '<strong>' . $created . '</strong><br />' . $webform_label
          ],
        ],


        'values' => [
          'class' => ['class' => 'align-top'],
          'data' => [
            '#markup' => '<div class="super-small">' . implode('<br>', $data_preview) . '</div>',
          ],
        ],

        'link' => [
          'class' => ['class' => 'align-top'],
          'data' => [
            '#type' => 'link',
            '#title' => 'View',
            '#url' => $submission_url,
            '#attributes' => [
              'class' => ['button', 'button--small'],
            ],
          ],

        ],

      ];
    }

    // If no submissions are found, display a message.
    if (empty($rows)) {
      return [
        '#markup' => 'No webform submissions found.',
      ];
    }

    // Build the table with the last submissions.
    $build['table'] = [
      '#theme' => 'table',
      '#rows' => $rows,
      '#empty' => 'No webform submissions found.',
    ];

    // Add a button linking to the webform submissions overview.
    $build['submissions_overview_link'] = [
      '#type' => 'link',
      '#title' => t('View All Submissions'),
      '#url' => Url::fromRoute('entity.webform_submission.collection'),
      '#attributes' => [
        'class' => ['button', 'button--small'],
      ],
    ];

    $build['#cache'] = [
      'max-age' => 300, // Cache for 5 minutes
      'contexts' => ['url'], // Cache per URL (optional, if you want caching to be per URL)
    ];

    return $build;
  }

  public function getConfigSpecificFormFields(FormStateInterface &$form_state, Request &$request, array $block_configuration)
  {
    $form = [];

    // Load all webforms for options.
    $webforms = Webform::loadMultiple();
    $webform_options = [];
    foreach ($webforms as $webform) {
      $webform_options[$webform->id()] = $webform->label();
    }

    // Get the selected webforms from the block configuration.
    $selected_webforms = isset($block_configuration['plugin_specific_config']['selected_webforms']) ? $block_configuration['plugin_specific_config']['selected_webforms'] : [];

    $form['selected_webforms'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Select Webforms'),
      '#options' => $webform_options,
      '#default_value' => $selected_webforms,
      '#description' => $this->t('Select one or more webforms to show submissions from. If no webform is selected, submissions from all webforms will be shown.'),
    ];

    $form['submission_limit'] = [
      '#type' => 'number',
      '#title' => $this->t('Submission Limit'),
      '#description' => $this->t('The number of submissions to show.'),
      '#default_value' => !empty($block_configuration['plugin_specific_config']['submission_limit']) ? $block_configuration['plugin_specific_config']['submission_limit'] : 10,
      '#min' => 1,
      '#max' => 50,
      '#step' => 1,
    ];

    return $form;
  }
}
