<?php

namespace Drupal\content_planner\Plugin\DashboardBlock;

use Drupal\content_planner\DashboardBlockBase;
use Drupal\webform\Entity\Webform;
use Drupal\Core\Url;
use Drupal\Core\Datetime\DrupalDateTime;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a Dashboard block with submission statistics from webforms .
 *
 * @DashboardBlock(
 *   id = "webforms_statistics_block",
 *   name = @Translation("Active Webforms and Submissions"),
 *   category = "Webform Reports",
 *   id_category = "webform_reports"
 * )
 */
class WebformsStatisticsBlock extends DashboardBlockBase
{

  /**
   * {@inheritdoc}
   */
  public function build()
  {
    // Check if the Webform module is enabled.
    if (!\Drupal::moduleHandler()->moduleExists('webform')) {
      return [
        '#markup' => 'Webform module is not enabled.',
      ];
    }

    // Define base header.
    $header = [
      'webform' => 'Webform',
    ];

    // Get the current month and year
    $today = new \DateTime();

    $config = $this->getConfiguration();
    $selected_webforms = !empty($config['plugin_specific_config']['selected_webforms']) ? array_filter($config['plugin_specific_config']['selected_webforms']) : [];
    // Get number of months from configuration, default to 12
    $numberOfMonths = !empty($config['plugin_specific_config']['number_of_months']) ? $config['plugin_specific_config']['number_of_months'] : 12;


    // Generate dates for the last 'n' months, including the current month.
    $dates = [];
    for ($i = 0; $i < $numberOfMonths; $i++) {
      $dates[] = (clone $today)->modify("-$i months");
    }
    $dates = array_reverse($dates);

    // Add each month's header.
    foreach ($dates as $date) {
      $header[$date->format('Y-m')] = $date->format('M y');
    }

    // Add the total submissions header.
    $header['submissions'] = 'Total';

    $rows = [];

    // Load selected webforms if any selected, otherwise load all.
    if (!empty($selected_webforms)) {
      $webforms = Webform::loadMultiple($selected_webforms);
    } else {
      $webforms = Webform::loadMultiple();
    }

    foreach ($webforms as $webform) {
      if ($webform->isOpen()) {
        // Create a link to the webform.
        $webform_link = \Drupal\Core\Link::createFromRoute(
          $webform->label(),
          'entity.webform.canonical',
          ['webform' => $webform->id()]
        )->toString();

        $row = ['webform' => $webform_link];

        /** @var \Drupal\webform\WebformSubmissionStorage $submission_storage */
        $submission_storage = \Drupal::entityTypeManager()->getStorage('webform_submission');

        // Get monthly submissions for the past six months.
        foreach ($dates as $date) {
          $start_date = strtotime($date->format('Y-m-01 00:00:00'));
          $end_date = strtotime($date->format('Y-m-t 23:59:59'));

          $count = \Drupal::entityQuery('webform_submission')
            ->condition('webform_id', $webform->id())
            ->condition('created', $start_date, '>=')
            ->condition('created', $end_date, '<=')
            ->accessCheck(TRUE)
            ->count()
            ->execute();

          $row[$date->format('Y-m')] = $count;
        }

        // Get total submissions.
        $total_submissions = $submission_storage->getTotal($webform);
        $results_url = Url::fromRoute('entity.webform.results_submissions', ['webform' => $webform->id()]);

        $row['submissions'] = [
          'data' => [
            '#type' => 'link',
            '#title' => $total_submissions,
            '#url' => $results_url,
          ],
        ];

        $rows[] = $row;
      }
    }

    // If no active webforms are found, display a message with a link to create a new webform.
    if (empty($rows)) {
      $create_webform_url = Url::fromRoute('entity.webform.add_form');
      return [
        '#markup' => t('No active webforms found. <a href=":url">Create a new webform</a>.', [':url' => $create_webform_url->toString()]),
      ];
    }

    // Build the table with the webform statistics.
    $build['table'] = [
      '#theme' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => 'No active webforms found.',
    ];

    $build['#cache'] = [
      'max-age' => 300, // Cache for 5 minutes
      'contexts' => ['url'], // Cache per URL (optional, if you want caching to be per URL)
    ];

    return $build;
  }

  public function getConfigSpecificFormFields(
    FormStateInterface &$form_state,
    Request &$request,
    array $block_configuration
  ) {
    $form = [];

    // Load all webforms for options.
    $webforms = Webform::loadMultiple();
    $webform_options = [];
    foreach ($webforms as $webform) {
      $webform_options[$webform->id()] = $webform->label();
    }

    // Get the selected webforms from the block configuration.
    $selected_webforms = isset($block_configuration['plugin_specific_config']['selected_webforms']) ? $block_configuration['plugin_specific_config']['selected_webforms'] : [];

    $form['selected_webforms'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Select Webforms'),
      '#options' => $webform_options,
      '#default_value' => $selected_webforms,
      '#description' => $this->t('Select one or more webforms to show submissions from. If no webform is selected, submissions from all webforms will be shown.'),
    ];

    // Number of months field
    $form['number_of_months'] = [
      '#type' => 'select',
      '#title' => $this->t('Number of Months'),
      '#options' => array_combine(range(1, 12), range(1, 12)),
      '#default_value' => $block_configuration['plugin_specific_config']['number_of_months'] ?? 12,
      '#description' => $this->t('Select the number of months to display.'),
    ];

    return $form;
  }
}
