<?php

namespace Drupal\content_planner\Plugin\DashboardBlock;

use Drupal\content_planner\DashboardBlockBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'WelcomeUser' dashboard block.
 *
 * @DashboardBlock(
 *   id = "welcome_user",
 *   name = @Translation("Welcome User Widget"),
 *   category = "User",
 *   id_category = "user"
 * )
 */
class WelcomeUser extends DashboardBlockBase implements ContainerFactoryPluginInterface
{

  protected $currentUser;
  protected $userProfileImage; // The user profile image service

  public function __construct(array $configuration, $plugin_id, $plugin_definition, AccountProxyInterface $current_user, $user_profile_image)
  {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->currentUser = $current_user;
    $this->userProfileImage = $user_profile_image;
  }

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition)
  {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('current_user'),
      $container->get('content_planner.user_profile_image') // Inject the user profile image service
    );
  }

  public function build()
  {
    $build = [];

    // Load the current user entity
    $user = \Drupal\user\Entity\User::load($this->currentUser->id());
    $username = $user->getDisplayName();

    // Retrieve user profile image URL or use default SVG if not available
    $profileImageUrl = $this->userProfileImage->generateProfileImageUrl($user, 'thumbnail');

    if (empty($profileImageUrl)) {
      // Inline SVG as a fallback
      $profileImageMarkup = '
      <svg width="80" height="80" viewBox="0 0 80 80" fill="none" xmlns="http://www.w3.org/2000/svg">
      <path fill-rule="evenodd" clip-rule="evenodd" d="M22.4125 53.8848C19.6397 50.1444 18 45.5137 18 40.5C18 28.0735 28.0737 18 40.5 18C52.9265 18 63 28.0735 63 40.5C63 52.9263 52.9265 63 40.5 63C33.9805 63 28.1086 60.2272 23.9991 55.7964C23.4364 55.1893 22.9064 54.5513 22.4125 53.8848ZM23.66 51.75C21.5062 48.5323 20.2502 44.6627 20.2502 40.5C20.2502 29.3162 29.3162 20.25 40.5002 20.25C51.6838 20.25 60.75 29.3162 60.75 40.5C60.75 44.663 59.494 48.5323 57.34 51.75C54.6225 47.6903 50.4759 44.668 45.6199 43.4029C47.9645 41.7781 49.5 39.0685 49.5 35.9999C49.5 31.0294 45.4705 26.9998 40.5 26.9998C35.5295 26.9998 31.5 31.0294 31.5 35.9999C31.5 39.0683 33.0357 41.7781 35.3801 43.4027C30.5243 44.668 26.3775 47.6901 23.66 51.75ZM25.1093 53.6607C28.267 48.4681 33.9783 44.9999 40.4998 44.9999C47.0214 44.9999 52.733 48.4681 55.8907 53.6609C52.1767 58 46.6596 60.75 40.5 60.75C34.3404 60.75 28.8233 57.9998 25.1093 53.6607ZM40.5 42.7499C36.772 42.7499 33.7499 39.7281 33.7499 35.9999C33.7499 32.2719 36.7718 29.2498 40.5 29.2498C44.2282 29.2498 47.2501 32.2721 47.2501 36.0001C47.2501 39.7281 44.228 42.7499 40.5 42.7499Z" fill="black"/>
      </svg>

      ';
    } else {
      // Use profile image URL
      $profileImageMarkup = "<img src='{$profileImageUrl}' alt='{$username}' />";
    }

    // Edit button - link to the user edit form
    $editUrl = Url::fromRoute('entity.user.edit_form', ['user' => $user->id()])->toString();

    // Logout button - link to the user logout route
    $logoutUrl = Url::fromRoute('user.logout')->toString();

    // Inline template with styling
    $build['content'] = [
      '#markup' => "
            <div id='dashboard-welcome-user-block'>
                <div class='portrait-wrapper'>
                    {$profileImageMarkup}
                </div>
                <div class='user-info'>
                    <h3 class='mb-4>{$username}</h3>
                    <a href='{$editUrl}' class='button button--extrasmall'>Edit</a>
                    <a href='{$logoutUrl}' class='button button--extrasmall'>Logout</a>
                </div>
            </div>
        ",
      '#allowed_tags' => ['svg', 'path', 'rect', 'div', 'text', 'line', 'circle', 'g', 'title', 'a', 'h3', 'img'],
    ];

    return $build;
  }




}

