<?php

namespace Drupal\content_planner\Routing;

use Drupal\content_planner\DashboardSettingsService;
use Drupal\content_planner\DashboardTypePluginManager;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Routing\Route;

/**
 * Defines dynamic dashboard routes.
 */
class DashboardRoutes implements ContainerInjectionInterface {

  /**
   * The dashboard settings service.
   */
  protected DashboardSettingsService $dashboardSettings;

  /**
   * The manager to be used for instantiating plugins.
   */
  protected DashboardTypePluginManager $dashboardTypePluginManager;

  /**
   * Constructs a new dashboard route subscriber.
   *
   * @param \Drupal\content_planner\DashboardSettingsService $dashboard_settings
   *   The dashboard settings.
   * @param \Drupal\content_planner\DashboardSettingsService $dashboard_type_plugin_manager
   *   The dashboard type plugin manager.
   */
  public function __construct(
    DashboardSettingsService $dashboard_settings,
    DashboardTypePluginManager $dashboard_type_plugin_manager,
  ) {
    $this->dashboardSettings = $dashboard_settings;
    $this->dashboardTypePluginManager = $dashboard_type_plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('content_planner.dashboard_settings_service'),
      $container->get('content_planner.dashboard_type_plugin_manager')
    );
  }

  /**
   * Provides dynamic routes.
   */
  public function routes(): array {
    $routes = [];

    $types = $this->dashboardSettings->getDashboardTypes();
    foreach ($types as $id => $label) {
      /** @var \Drupal\content_planner\DashboardTypeInterface $plugin */
      $plugin = $this->dashboardTypePluginManager->createInstance($id);

      /** @var \Symfony\Component\Routing\Route $route */
      if ($route = $plugin->getBaseRoute()) {
        // Base route.
        $routes['content_planner.' . $plugin->getId() . '.dashboard'] = $route;

        // Dashboard route.
        $dashboard_route = clone $route;
        $dashboard_path = $dashboard_route->getPath();
        $dashboard_path = $dashboard_path . '/{dashboard_id}';
        $dashboard_route->setPath($dashboard_path);

        assert($dashboard_route instanceof Route);

        // Default to master.
        $dashboard_route->setDefault('dashboard_id', 'master');
        $routes['content_planner.' . $plugin->getId() . '.dashboard'] = $dashboard_route;
      }

      // Create list route.
      $list_route = new Route(
        $plugin->getBaseRouteName() . "/list",
        [
          '_form' => '\Drupal\content_planner\Form\DashboardTypeListBuilderForm',
          '_dashboard_type' => $plugin->getId(),
          '_title' => "Configure dashboards",
        ],
        [
          '_permission'  => 'administer content planner dashboard settings',
        ],
      );

      $routes['content_planner.' . $plugin->getId() . '.dashboards.list'] = $list_route;

      // Create add dashboard route.
      $create_route = new Route(
        $plugin->getBaseRouteName() . "/create",
        [
          '_form' => '\Drupal\content_planner\Form\DashboardCreateForm',
          '_dashboard_type' => $plugin->getId(),
          '_title' => "Create dashboard",
        ],
        [
          '_custom_access'  => 'Drupal\content_planner\Access\DashboardAccessCheck::checkAccess',
        ],
      );

      $routes['content_planner.' . $plugin->getId() . '.dashboards.create'] = $create_route;
    }

    return $routes;
  }

}
